@@ -12,9 +12,12 @@
 #include "uae/types.h"
 #include "readcpu.h"
 #include "machdep/m68k.h"
+#include "events.h"
+#include <softfloat/softfloat.h>
+
+#ifdef WINUAE_FOR_HATARI
 #include "compat.h"
 #include "maccess.h"
-#include "events.h"
 #include "memory.h"
 #include "custom.h"
 
@@ -24,24 +27,9 @@
 #define M68000_EXC_SRC_AUTOVEC	2	/* Auto-vector exception (e.g. VBL) */
 #define M68000_EXC_SRC_INT_MFP	3	/* MFP interrupt exception */
 #define M68000_EXC_SRC_INT_DSP	4	/* DSP interrupt exception */
+#define M68000_EXC_SRC_INT_SCC	5	/* SCC interrupt exception */
 
-
-/* Special flags (from custom.h) */
-#define SPCFLAG_DEBUGGER 1
-#define SPCFLAG_STOP 2
-#define SPCFLAG_BUSERROR 4
-#define SPCFLAG_INT 8
-#define SPCFLAG_BRK 0x10
-#define SPCFLAG_EXTRA_CYCLES 0x20
-#define SPCFLAG_TRACE 0x40
-#define SPCFLAG_DOTRACE 0x80
-#define SPCFLAG_DOINT 0x100
-#define SPCFLAG_MFP 0x200
-#define SPCFLAG_EXEC 0x400
-#define SPCFLAG_MODE_CHANGE 0x800
-#define SPCFLAG_DSP 0x1000
-
-#include <softfloat/softfloat.h>
+#endif
 
 #ifndef SET_CFLG
 
@@ -80,20 +68,29 @@ extern int fpp_movem_index2[256];
 extern int fpp_movem_next[256];
 #endif
 
-extern int bus_error_offset;
+extern int hardware_bus_error;
 
-typedef uae_u32 REGPARAM3 cpuop_func (uae_u32) REGPARAM;
-typedef void REGPARAM3 cpuop_func_ce (uae_u32) REGPARAM;
+typedef uae_u32 REGPARAM3 cpuop_func(uae_u32) REGPARAM;
+typedef void REGPARAM3 cpuop_func_noret(uae_u32) REGPARAM;
 
 struct cputbl {
-	cpuop_func *handler;
+	cpuop_func *handler_ff;
+#ifdef NOFLAGS_SUPPORT_GENCPU
+	cpuop_func_ret *handler_nf;
+#endif
+	cpuop_func_noret *handler_ff_noret;
+#ifdef NOFLAGS_SUPPORT_GENCPU
+	cpuop_func_ret *handler_nf_noret;
+#endif
 	uae_u16 opcode;
 	uae_s8 length;
 	uae_s8 disp020[2];
-	uae_u8 branch;
+	uae_s8 branch;
+	uae_u16 specific;
 };
 
 #ifdef JIT
+#define MIN_JIT_CACHE 128
 #define MAX_JIT_CACHE 16384
 typedef uae_u32 REGPARAM3 compop_func (uae_u32) REGPARAM;
 
@@ -111,8 +108,11 @@ struct comptbl {
 };
 #endif
 
-extern uae_u32 REGPARAM3 op_illg (uae_u32) REGPARAM;
-extern void REGPARAM3 op_unimpl (uae_u16) REGPARAM;
+extern cpuop_func *loop_mode_table[];
+
+extern uae_u32 REGPARAM3 op_illg(uae_u32) REGPARAM;
+extern void REGPARAM3 op_illg_noret(uae_u32) REGPARAM;
+extern void REGPARAM3 op_unimpl(uae_u32) REGPARAM;
 
 typedef uae_u8 flagtype;
 
@@ -120,10 +120,8 @@ typedef uae_u8 flagtype;
 
 #ifdef USE_LONG_DOUBLE
 typedef long double fptype;
-#define LDPTR tbyte ptr
 #else
 typedef double fptype;
-#define LDPTR qword ptr
 #endif
 #endif
 
@@ -171,10 +169,25 @@ struct mmufixup
 };
 extern struct mmufixup mmufixup[2];
 
+#ifdef MSVC_LONG_DOUBLE
+typedef struct {
+	uae_u64 m;
+	uae_u16 e;
+	uae_u16 dummy;
+} fprawtype;
+#endif
+
 typedef struct
 {
 	floatx80 fpx;
+#ifdef MSVC_LONG_DOUBLE
+	union {
+		fptype fp;
+		fprawtype rfp;
+	};
+#else
 	fptype fp;
+#endif
 } fpdata;
 
 struct regstruct
@@ -186,13 +199,18 @@ struct regstruct
 	uae_u8 *pc_oldp;
 	uae_u16 opcode;
 	uae_u32 instruction_pc;
+	uae_u32 instruction_pc_user_exception;
+	uae_u32 trace_pc;
 
-	uae_u16 irc, ir, db;
+	uae_u16 irc, ir, ird;
 	volatile uae_atomic spcflags;
 	uae_u32 last_prefetch;
 	uae_u32 chipset_latch_rw;
 	uae_u32 chipset_latch_read;
 	uae_u32 chipset_latch_write;
+	uae_u16 db, write_buffer, read_buffer;
+	int loop_mode;
+	int instruction_cnt;
 
 	uaecptr usp, isp, msp;
 	uae_u16 sr;
@@ -200,25 +218,27 @@ struct regstruct
 	flagtype t0;
 	flagtype s;
 	flagtype m;
-	flagtype x;
 	flagtype stopped;
 	int halted;
 	int exception;
 	int intmask;
-	int ipl, ipl_pin;
+	int ipl[2], ipl_pin, ipl_pin_p;
+	int lastipl;
+	evt_t ipl_pin_change_evt, ipl_pin_change_evt_p;
+	evt_t ipl_evt, ipl_evt_pre;
+	int ipl_evt_pre_mode;
 
 	uae_u32 vbr, sfc, dfc;
 
 #ifdef FPUEMU
 	fpdata fp[8];
-#ifdef JIT
 	fpdata fp_result;
-#endif
 	uae_u32 fpcr, fpsr, fpiar;
 	uae_u32 fpu_state;
 	uae_u32 fpu_exp_state;
 	uae_u16 fp_opword;
 	uaecptr fp_ea;
+	bool fp_ea_set;
 	uae_u32 fp_exp_pend, fp_unimp_pend;
 	bool fpu_exp_pre;
 	bool fp_unimp_ins;
@@ -255,14 +275,14 @@ struct regstruct
 
 	uae_u32 prefetch040[CPU_PIPELINE_MAX];
 
-	int ce020endcycle;
-	int ce020startcycle;
-	int ce020prefetchendcycle;
+	evt_t ce020endcycle;
+	evt_t ce020startcycle;
+	evt_t ce020prefetchendcycle;
 
-	int ce020extracycles;
+	evt_t ce020extracycles;
 	bool ce020memcycle_data;
 	int ce020_tail;
-	frame_time_t ce020_tail_cycles;
+	evt_t ce020_tail_cycles;
 	int memory_waitstate_cycles;
 };
 
@@ -274,13 +294,14 @@ struct cputracememory
 	uae_u32 addr;
 	uae_u32 data;
 	int mode;
+	uae_u32 flags;
 };
 
 struct cputracestruct
 {
 	uae_u32 regs[16];
 	uae_u32 usp, isp, pc;
-	uae_u16 ir, irc, sr, opcode;
+	uae_u16 ir, irc, ird, sr, opcode;
 	int intmask, stopped, state;
 
 	uae_u32 msp, vbr;
@@ -294,8 +315,9 @@ struct cputracestruct
 	int pipeline_pos;
 	int pipeline_r8[2];
 	int pipeline_stop;
+	uae_u16 read_buffer, write_buffer;
 
-	uae_u32 startcycles;
+	evt_t startcycles;
 	int needendcycles;
 	int memoryoffset;
 	int cyclecounter, cyclecounter_pre, cyclecounter_post;
@@ -310,8 +332,41 @@ STATIC_INLINE uae_u32 munge24 (uae_u32 x)
 
 extern int mmu_enabled, mmu_triggered;
 extern int cpu_cycles;
-extern int cpucycleunit;
+extern int cpucycleunit, cpuipldelay2, cpuipldelay4;
 extern int m68k_pc_indirect;
+extern bool m68k_interrupt_delay;
+
+extern bool cpu_bus_rmw;			// WINUAE_FOR_HATARI
+
+extern void safe_interrupt_set(int, int, bool);
+
+#define SPCFLAG_CPUINRESET 2
+#define SPCFLAG_INT 8
+#define SPCFLAG_BRK 16
+#define SPCFLAG_UAEINT 32
+#define SPCFLAG_TRACE 64
+#define SPCFLAG_DOTRACE 128
+#define SPCFLAG_DOINT 256 /* arg, JIT fails without this.. */
+//#define SPCFLAG_BLTNASTY 512
+#define SPCFLAG_EXEC 1024
+//#define SPCFLAG_ACTION_REPLAY 2048
+//#define SPCFLAG_TRAP 4096 /* enforcer-hack */
+#define SPCFLAG_MODE_CHANGE 8192
+#ifdef JIT
+#define SPCFLAG_END_COMPILE 16384
+#endif
+#define SPCFLAG_CHECK 32768
+#define SPCFLAG_MMURESTART 65536
+
+#ifdef WINUAE_FOR_HATARI
+// Hatari's specific flags
+#define SPCFLAG_DEBUGGER 0x1000000
+#define SPCFLAG_STOP 0x2000000
+#define SPCFLAG_BUSERROR 0x4000000
+#define SPCFLAG_EXTRA_CYCLES 0x8000000
+#define SPCFLAG_MFP 0x10000000
+#define SPCFLAG_DSP 0x20000000			// TODO : remove with do_specialties_interrupt()
+#endif
 
 #ifndef WINUAE_FOR_HATARI
 STATIC_INLINE void set_special_exter(uae_u32 x)
@@ -321,7 +376,7 @@ STATIC_INLINE void set_special_exter(uae_u32 x)
 STATIC_INLINE void set_special (uae_u32 x)
 {
 	atomic_or(&regs.spcflags, x);
-	cycles_do_special ();
+	cycles_do_special();
 }
 
 STATIC_INLINE void unset_special (uae_u32 x)
@@ -345,6 +400,13 @@ STATIC_INLINE void unset_special (uae_u32 x)
 #define m68k_dreg(r,num) ((r).regs[(num)])
 #define m68k_areg(r,num) (((r).regs + 8)[(num)])
 
+// JIT only
+#ifdef HAVE_GET_WORD_UNSWAPPED
+	#define GET_OPCODE (do_get_mem_word_unswapped((uae_u16*)(pc + pc_offset)));
+#else
+	#define GET_OPCODE (do_get_mem_word((uae_u16*)(pc + pc_offset)));
+#endif
+
 extern uae_u32(*x_prefetch)(int);
 extern uae_u32(*x_get_byte)(uaecptr addr);
 extern uae_u32(*x_get_word)(uaecptr addr);
@@ -386,6 +448,13 @@ uae_u32 mem_access_delay_longi_read_c040 (uaecptr addr);
 
 extern uae_u32(REGPARAM3 *x_cp_get_disp_ea_020)(uae_u32 base, int idx) REGPARAM;
 
+#ifndef WINUAE_FOR_HATARI
+extern bool debugmem_trace;
+extern void branch_stack_push(uaecptr, uaecptr);
+extern void branch_stack_pop_rte(uaecptr);
+extern void branch_stack_pop_rts(uaecptr);
+#endif
+
 /* direct (regs.pc_p) access */
 
 STATIC_INLINE void m68k_setpc(uaecptr newpc)
@@ -393,6 +462,11 @@ STATIC_INLINE void m68k_setpc(uaecptr newpc)
 	regs.pc_p = regs.pc_oldp = get_real_address(newpc);
 	regs.instruction_pc = regs.pc = newpc;
 }
+STATIC_INLINE void m68k_setpc_j(uaecptr newpc)
+{
+	regs.pc_p = regs.pc_oldp = get_real_address(newpc);
+	regs.pc = newpc;
+}
 STATIC_INLINE uaecptr m68k_getpc(void)
 {
 	return (uaecptr)(regs.pc + ((uae_u8*)regs.pc_p - (uae_u8*)regs.pc_oldp));
@@ -451,6 +525,10 @@ STATIC_INLINE void m68k_setpci(uaecptr newpc)
 {
 	regs.instruction_pc = regs.pc = newpc;
 }
+STATIC_INLINE void m68k_setpci_j(uaecptr newpc)
+{
+	regs.pc = newpc;
+}
 STATIC_INLINE uaecptr m68k_getpci(void)
 {
 	return regs.pc;
@@ -576,6 +654,10 @@ STATIC_INLINE void m68k_setpc_normal(uaecptr pc)
 	}
 }
 
+extern void cpu_invalidate_cache(uaecptr, int);
+
+extern bool(*is_super_access)(bool);
+
 extern uae_u32(*read_data_030_bget)(uaecptr);
 extern uae_u32(*read_data_030_wget)(uaecptr);
 extern uae_u32(*read_data_030_lget)(uaecptr);
@@ -593,9 +675,11 @@ extern void(*write_data_030_fc_lput)(uaecptr, uae_u32, uae_u32);
 extern void write_dcache030_bput(uaecptr, uae_u32, uae_u32);
 extern void write_dcache030_wput(uaecptr, uae_u32, uae_u32);
 extern void write_dcache030_lput(uaecptr, uae_u32, uae_u32);
+extern void write_dcache030_retry(uaecptr addr, uae_u32 v, uae_u32 fc, int size, int flags);
 extern uae_u32 read_dcache030_bget(uaecptr, uae_u32);
 extern uae_u32 read_dcache030_wget(uaecptr, uae_u32);
 extern uae_u32 read_dcache030_lget(uaecptr, uae_u32);
+extern uae_u32 read_dcache030_retry(uaecptr addr, uae_u32 fc, int size, int flags);
 
 extern void write_dcache030_mmu_bput(uaecptr, uae_u32);
 extern void write_dcache030_mmu_wput(uaecptr, uae_u32);
@@ -604,7 +688,9 @@ extern uae_u32 read_dcache030_mmu_bget(uaecptr);
 extern uae_u32 read_dcache030_mmu_wget(uaecptr);
 extern uae_u32 read_dcache030_mmu_lget(uaecptr);
 extern void write_dcache030_lrmw_mmu(uaecptr, uae_u32, uae_u32);
+extern void write_dcache030_lrmw_mmu_fcx(uaecptr, uae_u32, uae_u32, int);
 extern uae_u32 read_dcache030_lrmw_mmu(uaecptr, uae_u32);
+extern uae_u32 read_dcache030_lrmw_mmu_fcx(uaecptr, uae_u32, int);
 
 extern void check_t0_trace(void);
 extern uae_u32 get_word_icache030(uaecptr addr);
@@ -624,9 +710,16 @@ extern uae_u32 next_ilong_cache040(void);
 extern uae_u32 get_word_icache040(uaecptr addr);
 extern uae_u32 get_long_icache040(uaecptr addr);
 
-extern void (*x_do_cycles)(unsigned long);
-extern void (*x_do_cycles_pre)(unsigned long);
-extern void (*x_do_cycles_post)(unsigned long, uae_u32);
+extern uae_u32 sfc_nommu_get_byte(uaecptr);
+extern uae_u32 sfc_nommu_get_word(uaecptr);
+extern uae_u32 sfc_nommu_get_long(uaecptr);
+extern void dfc_nommu_put_byte(uaecptr, uae_u32);
+extern void dfc_nommu_put_word(uaecptr, uae_u32);
+extern void dfc_nommu_put_long(uaecptr, uae_u32);
+
+extern void (*x_do_cycles)(int);
+extern void (*x_do_cycles_pre)(int);
+extern void (*x_do_cycles_post)(int, uae_u32);
 #ifdef WINUAE_FOR_HATARI
 void set_x_funcs_hatari_blitter (int flag);
 #endif
@@ -638,23 +731,26 @@ extern uae_u32 REGPARAM3 x_get_disp_ea_040(uae_u32 base, int idx) REGPARAM;
 extern uae_u32 REGPARAM3 x_get_bitfield (uae_u32 src, uae_u32 bdata[2], uae_s32 offset, int width) REGPARAM;
 extern void REGPARAM3 x_put_bitfield (uae_u32 dst, uae_u32 bdata[2], uae_u32 val, uae_s32 offset, int width) REGPARAM;
 
-extern void m68k_setstopped (void);
-extern void m68k_resumestopped (void);
+extern void m68k_setstopped(int stoptype);
+extern void m68k_resumestopped(void);
 extern void m68k_cancel_idle(void);
+extern void do_cycles_stop(int);
 
 extern uae_u32 REGPARAM3 get_disp_ea_020 (uae_u32 base, int idx) REGPARAM;
 extern uae_u32 REGPARAM3 get_bitfield (uae_u32 src, uae_u32 bdata[2], uae_s32 offset, int width) REGPARAM;
 extern void REGPARAM3 put_bitfield (uae_u32 dst, uae_u32 bdata[2], uae_u32 val, uae_s32 offset, int width) REGPARAM;
 
-extern void m68k_disasm_ea (uaecptr addr, uaecptr *nextpc, int cnt, uae_u32 *seaddr, uae_u32 *deaddr);
-extern void m68k_disasm (uaecptr addr, uaecptr *nextpc, int cnt);
-extern void m68k_disasm_2 (TCHAR *buf, int bufsize, uaecptr addr, uaecptr *nextpc, int cnt, uae_u32 *seaddr, uae_u32 *deaddr, int safemode);
-extern void sm68k_disasm (TCHAR*, TCHAR*, uaecptr addr, uaecptr *nextpc);
-extern int m68k_asm(TCHAR *buf, uae_u16 *out, uaecptr pc);
-extern int get_cpu_model (void);
+extern void m68k_disasm_ea (uaecptr addr, uaecptr *nextpc, int cnt, uae_u32 *seaddr, uae_u32 *deaddr, uaecptr lastpc);
+extern void m68k_disasm (uaecptr addr, uaecptr *nextpc, uaecptr lastpc, int cnt);
+extern uae_u32 m68k_disasm_2(TCHAR *buf, int bufsize, uaecptr pc, uae_u16 *bufpc, int bufpccount, uaecptr *nextpc, int cnt, uae_u32 *seaddr, uae_u32 *deaddr, uaecptr lastpc, int safemode);
 #ifdef WINUAE_FOR_HATARI
-extern void m68k_disasm_file (FILE *f, uaecptr addr, uaecptr *nextpc, int cnt);
+extern void m68k_disasm_file (FILE *f, uaecptr addr, uaecptr *nextpc, uaecptr lastpc, int cnt);
+extern void m68k_disasm_file_wrapper (FILE *f, uaecptr addr, uaecptr *nextpc, uaecptr lastpc, int cnt);
 #endif
+extern void sm68k_disasm (TCHAR*, TCHAR*, uaecptr addr, uaecptr *nextpc, uaecptr lastpc);
+extern int m68k_asm(TCHAR *buf, uae_u16 *out, uaecptr pc);
+extern uaecptr ShowEA(void *f, uaecptr pc, uae_u16 opcode, int reg, amodes mode, wordsizes size, TCHAR *buf, uae_u32 *eaddr, int *actualea, int safemode);
+extern int get_cpu_model (void);
 
 extern void set_cpu_caches (bool flush);
 #ifdef WINUAE_FOR_HATARI
@@ -665,33 +761,61 @@ extern void flush_cpu_caches_040(uae_u16 opcode);
 extern void REGPARAM3 MakeSR (void) REGPARAM;
 extern void REGPARAM3 MakeFromSR(void) REGPARAM;
 extern void REGPARAM3 MakeFromSR_T0(void) REGPARAM;
+extern void REGPARAM3 MakeFromSR_STOP(void) REGPARAM;
 extern void REGPARAM3 Exception (int) REGPARAM;
 extern void REGPARAM3 Exception_cpu(int) REGPARAM;
+extern void REGPARAM3 Exception_cpu_oldpc(int, uaecptr) REGPARAM;
 extern void REGPARAM3 ExceptionL (int, uaecptr) REGPARAM;
 extern void NMI (void);
-extern void NMI_delayed (void);
+extern void IRQ_forced(int, int);
 extern void prepare_interrupt (uae_u32);
-extern void doint (void);
+extern void doint(void);
+extern void checkint(void);
+extern void intlev_load(void);
+extern void ipl_fetch_now_pre(void);
+extern void ipl_fetch_next_pre(void);
+extern void ipl_fetch_now(void);
+extern void ipl_fetch_next(void);
 extern void dump_counts (void);
 extern int m68k_move2c (int, uae_u32 *);
 extern int m68k_movec2 (int, uae_u32 *);
-extern bool m68k_divl (uae_u32, uae_u32, uae_u16);
-extern bool m68k_mull (uae_u32, uae_u32, uae_u16);
+extern int m68k_divl (uae_u32, uae_u32, uae_u16, uaecptr);
+extern int m68k_mull (uae_u32, uae_u32, uae_u16);
 extern void init_m68k (void);
 extern void m68k_go (int);
-extern void m68k_dumpstate (uaecptr *);
-extern void m68k_dumpstate_2 (uaecptr, uaecptr *);
-extern void m68k_dumpstate_file (FILE *f, uaecptr *);
-extern void m68k_dumpcache (bool);
+extern void m68k_dumpstate(uaecptr *, uaecptr);
+#ifdef WINUAE_FOR_HATARI
+extern void m68k_dumpstate_file (FILE *f, uaecptr *nextpc, uaecptr prevpc);
+#endif
+extern void m68k_dumpcache(bool);
+extern int getMulu68kCycles(uae_u16 src);
+extern int getMuls68kCycles(uae_u16 src);
 extern int getDivu68kCycles (uae_u32 dividend, uae_u16 divisor);
-extern int getDivs68kCycles (uae_s32 dividend, uae_s16 divisor);
+extern int getDivs68kCycles (uae_s32 dividend, uae_s16 divisor, int *extra);
 extern void divbyzero_special(bool issigned, uae_s32 dst);
-extern void setdivuoverflowflags(uae_u32 dividend, uae_u16 divisor);
-extern void setdivsoverflowflags(uae_s32 dividend, uae_s16 divisor);
+extern void setdivuflags(uae_u32 dividend, uae_u16 divisor);
+extern void setdivsflags(uae_s32 dividend, uae_s16 divisor);
+extern void setchkundefinedflags(uae_s32 src, uae_s32 dst, int size);
+extern void setchk2undefinedflags(uae_s32 lower, uae_s32 upper, uae_s32 val, int size);
 extern void protect_roms (bool);
 extern void unprotect_maprom (void);
 extern bool is_hardreset(void);
 extern bool is_keyboardreset(void);
+extern void Exception_build_stack_frame_common(uae_u32 oldpc, uae_u32 currpc, uae_u32 ssw, int nr, int vector_nr);
+extern void Exception_build_stack_frame(uae_u32 oldpc, uae_u32 currpc, uae_u32 ssw, int nr, int format);
+extern void Exception_build_68000_address_error_stack_frame(uae_u16 mode, uae_u16 opcode, uaecptr fault_addr, uaecptr pc);
+extern uae_u32 exception_pc(int nr);
+extern void cpu_restore_fixup(void);
+extern bool privileged_copro_instruction(uae_u16 opcode);
+extern bool generates_group1_exception(uae_u16 opcode);
+
+void ccr_68000_long_move_ae_LZN(uae_s32 src);
+void ccr_68000_long_move_ae_LN(uae_s32 src);
+void ccr_68000_long_move_ae_HNZ(uae_s32 src);
+void ccr_68000_long_move_ae_normal(uae_s32 src);
+void ccr_68000_word_move_ae_normal(uae_s16 src);
+void dreg_68000_long_replace_low(int reg, uae_u16 v);
+void areg_68000_long_replace_low(int reg, uae_u16 v);
 
 extern void mmu_op (uae_u32, uae_u32);
 extern bool mmu_op30 (uaecptr, uae_u32, uae_u16, uaecptr);
@@ -710,20 +834,30 @@ extern void fpux_restore (int*);
 extern bool fpu_get_constant(fpdata *fp, int cr);
 extern int fpp_cond(int condition);
 
-extern void exception3_read(uae_u32 opcode, uaecptr addr);
-extern void exception3_write(uae_u32 opcode, uaecptr addr);
+extern void exception3_read(uae_u32 opcode, uaecptr addr, int size, int fc);
+extern void exception3_write(uae_u32 opcode, uaecptr addr, int size, uae_u32 val, int fc);
+extern void exception3_read_access(uae_u32 opcode, uaecptr addr, int size, int fc);
+extern void exception3_read_access2(uae_u32 opcode, uaecptr addr, int size, int fc);
+extern void exception3_write_access(uae_u32 opcode, uaecptr addr, int size, uae_u32 val, int fc);
+extern void exception3_read_prefetch(uae_u32 opcode, uaecptr addr);
+extern void exception3_read_prefetch_68040bug(uae_u32 opcode, uaecptr addr, uae_u16 secondarysr);
+extern void exception3_read_prefetch_only(uae_u32 opcode, uaecptr addr);
 extern void exception3_notinstruction(uae_u32 opcode, uaecptr addr);
-extern void exception3i (uae_u32 opcode, uaecptr addr);
-extern void exception3b (uae_u32 opcode, uaecptr addr, bool w, bool i, uaecptr pc);
-extern void exception2 (uaecptr addr, bool read, int size, uae_u32 fc);
-extern void m68k_reset (void);
-extern void cpureset (void);
-extern void cpu_halt (int id);
+extern void hardware_exception2(uaecptr addr, uae_u32 v, bool read, bool ins, int size);
+extern void exception2_setup(uae_u32 opcode, uaecptr addr, bool read, int size, uae_u32 fc);
+extern void exception2_read(uae_u32 opcode, uaecptr addr, int size, int fc);
+extern void exception2_write(uae_u32 opcode, uaecptr addr, int size, uae_u32 val, int fc);
+extern void exception2_fetch_opcode(uae_u32 opcode, int offset, int pcoffset);
+extern void exception2_fetch(uae_u32 opcode, int offset, int pcoffset);
+extern void m68k_reset(void);
+extern bool cpureset(void);
+extern void cpu_halt(int id);
+extern void cpu_inreset(void);
 extern int cpu_sleep_millis(int ms);
 extern void cpu_change(int newmodel);
 extern void cpu_fallback(int mode);
 
-extern void fill_prefetch (void);
+extern void fill_prefetch(void);
 extern void fill_prefetch_020_ntx(void);
 extern void fill_prefetch_030_ntx(void);
 extern void fill_prefetch_030_ntx_continue(void);
@@ -733,50 +867,50 @@ extern void fill_prefetch_030(void);
 #define CPU_OP_NAME(a) op ## a
 
 /* 68060 */
-extern const struct cputbl op_smalltbl_0_ff[];
-extern const struct cputbl op_smalltbl_40_ff[];
-extern const struct cputbl op_smalltbl_50_ff[];
-extern const struct cputbl op_smalltbl_24_ff[]; // CE
-extern const struct cputbl op_smalltbl_33_ff[]; // MMU
+extern const struct cputbl op_smalltbl_0[];
+extern const struct cputbl op_smalltbl_40[];
+extern const struct cputbl op_smalltbl_50[];
+extern const struct cputbl op_smalltbl_24[]; // CE
+extern const struct cputbl op_smalltbl_33[]; // MMU
 /* 68040 */
-extern const struct cputbl op_smalltbl_1_ff[];
-extern const struct cputbl op_smalltbl_41_ff[];
-extern const struct cputbl op_smalltbl_51_ff[];
-extern const struct cputbl op_smalltbl_25_ff[]; // CE
-extern const struct cputbl op_smalltbl_31_ff[]; // MMU
+extern const struct cputbl op_smalltbl_1[];
+extern const struct cputbl op_smalltbl_41[];
+extern const struct cputbl op_smalltbl_51[];
+extern const struct cputbl op_smalltbl_25[]; // CE
+extern const struct cputbl op_smalltbl_31[]; // MMU
 /* 68030 */
-extern const struct cputbl op_smalltbl_2_ff[];
-extern const struct cputbl op_smalltbl_42_ff[];
-extern const struct cputbl op_smalltbl_52_ff[];
-extern const struct cputbl op_smalltbl_22_ff[]; // prefetch
-extern const struct cputbl op_smalltbl_23_ff[]; // CE
-extern const struct cputbl op_smalltbl_32_ff[]; // MMU
-extern const struct cputbl op_smalltbl_34_ff[]; // MMU + cache
-extern const struct cputbl op_smalltbl_35_ff[]; // MMU + CE + cache
+extern const struct cputbl op_smalltbl_2[];
+extern const struct cputbl op_smalltbl_42[];
+extern const struct cputbl op_smalltbl_52[];
+extern const struct cputbl op_smalltbl_22[]; // prefetch
+extern const struct cputbl op_smalltbl_23[]; // CE
+extern const struct cputbl op_smalltbl_32[]; // MMU
+extern const struct cputbl op_smalltbl_34[]; // MMU + cache
+extern const struct cputbl op_smalltbl_35[]; // MMU + CE + cache
 /* 68020 */
-extern const struct cputbl op_smalltbl_3_ff[];
-extern const struct cputbl op_smalltbl_43_ff[];
-extern const struct cputbl op_smalltbl_53_ff[];
-extern const struct cputbl op_smalltbl_20_ff[]; // prefetch
-extern const struct cputbl op_smalltbl_21_ff[]; // CE
+extern const struct cputbl op_smalltbl_3[];
+extern const struct cputbl op_smalltbl_43[];
+extern const struct cputbl op_smalltbl_53[];
+extern const struct cputbl op_smalltbl_20[]; // prefetch
+extern const struct cputbl op_smalltbl_21[]; // CE
 /* 68010 */
-extern const struct cputbl op_smalltbl_4_ff[];
-extern const struct cputbl op_smalltbl_44_ff[];
-extern const struct cputbl op_smalltbl_54_ff[];
-extern const struct cputbl op_smalltbl_11_ff[]; // prefetch
-extern const struct cputbl op_smalltbl_13_ff[]; // CE
+extern const struct cputbl op_smalltbl_4[];
+extern const struct cputbl op_smalltbl_44[];
+extern const struct cputbl op_smalltbl_54[];
+extern const struct cputbl op_smalltbl_11[]; // prefetch
+extern const struct cputbl op_smalltbl_13[]; // CE
 /* 68000 */
-extern const struct cputbl op_smalltbl_5_ff[];
-extern const struct cputbl op_smalltbl_45_ff[];
-extern const struct cputbl op_smalltbl_55_ff[];
-extern const struct cputbl op_smalltbl_12_ff[]; // prefetch
-extern const struct cputbl op_smalltbl_14_ff[]; // CE
+extern const struct cputbl op_smalltbl_5[];
+extern const struct cputbl op_smalltbl_45[];
+extern const struct cputbl op_smalltbl_55[];
+extern const struct cputbl op_smalltbl_12[]; // prefetch
+extern const struct cputbl op_smalltbl_14[]; // CE
 
-extern cpuop_func *cpufunctbl[65536] ASM_SYM_FOR_FUNC ("cpufunctbl");
+extern cpuop_func_noret *cpufunctbl_noret[65536] ASM_SYM_FOR_FUNC("cpufunctbl_noret");
+extern cpuop_func *cpufunctbl[65536] ASM_SYM_FOR_FUNC("cpufunctbl");
 
 #ifdef JIT
-extern void flush_icache(int);
-extern void flush_icache_hard(int);
+extern void (*flush_icache)(int);
 extern void compemu_reset(void);
 #else
 #define flush_icache(int) do {} while (0)
@@ -793,6 +927,7 @@ extern uae_u32 val_move2c (int regno);
 extern void val_move2c2 (int regno, uae_u32 val);
 struct cpum2c {
 	int regno;
+	int flags;
 	const TCHAR *regname;
 };
 extern struct cpum2c m2cregs[];
@@ -813,11 +948,11 @@ extern bool can_cpu_tracer (void);
 #define CPU_HALT_CPU_STUCK 9
 #define CPU_HALT_SSP_IN_NON_EXISTING_ADDRESS 10
 #define CPU_HALT_INVALID_START_ADDRESS 11
+#define CPU_HALT_68060_HALT 12
+#define CPU_HALT_BKPT 13
 
-void cpu_semaphore_get(void);
-void cpu_semaphore_release(void);
-bool execute_other_cpu(int until);
-void execute_other_cpu_single(void);
+uae_u32 process_cpu_indirect_memory_read(uae_u32 addr, int size);
+void process_cpu_indirect_memory_write(uae_u32 addr, uae_u32 data, int size);
 
 
 /* From uae.h */
@@ -826,7 +961,7 @@ void execute_other_cpu_single(void);
 #define UAE_RESET_KEYBOARD 3
 #define UAE_RESET_HARD 4
 
-extern int uae_quit_program;
+extern int quit_program;
 /* From uae.h */
 
 
