@@ -17,49 +17,49 @@ int nr_cpuop_funcs;
 
 struct mnemolookup lookuptab[] = {
 	{ i_ILLG, _T("ILLEGAL"), NULL, 0 },
-	{ i_OR, _T("OR"), NULL, 1 },
+	{ i_OR, _T("OR"), NULL, 1 | MNEMOFLAG_LOOPMODE },
 	{ i_CHK, _T("CHK"), NULL, 0 },
 	{ i_CHK2, _T("CHK2"), NULL, 0 },
-	{ i_AND, _T("AND"), NULL, 1 },
-	{ i_EOR, _T("EOR"), NULL, 1 },
-	{ i_ORSR, _T("ORSR"), NULL, 0 },
-	{ i_ANDSR, _T("ANDSR"), NULL, 0 },
-	{ i_EORSR, _T("EORSR"), NULL, 0 },
-	{ i_SUB, _T("SUB"), NULL, 1 },
-	{ i_SUBA, _T("SUBA"), NULL, 1 },
-	{ i_SUBX, _T("SUBX"), NULL, 0 },
-	{ i_SBCD, _T("SBCD"), NULL, 0 },
-	{ i_ADD, _T("ADD"), NULL, 1 },
-	{ i_ADDA, _T("ADDA"), NULL, 1 },
-	{ i_ADDX, _T("ADDX"), NULL, 0 },
-	{ i_ABCD, _T("ABCD"), NULL, 0 },
-	{ i_NEG, _T("NEG"), NULL, 1 },
-	{ i_NEGX, _T("NEGX"), NULL, 1 },
-	{ i_NBCD, _T("NBCD"), NULL, 0 },
-	{ i_CLR, _T("CLR"), NULL, 0 },
-	{ i_NOT, _T("NOT"), NULL, 1 },
-	{ i_TST, _T("TST"), NULL, 1 },
+	{ i_AND, _T("AND"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_EOR, _T("EOR"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_ORSR, _T("ORSR"), _T("OR"), 0 },
+	{ i_ANDSR, _T("ANDSR"), _T("AND"), 0 },
+	{ i_EORSR, _T("EORSR"), _T("EOR"), 0 },
+	{ i_SUB, _T("SUB"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_SUBA, _T("SUBA"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_SUBX, _T("SUBX"), NULL, 0 | MNEMOFLAG_LOOPMODE },
+	{ i_SBCD, _T("SBCD"), NULL, 0 | MNEMOFLAG_LOOPMODE },
+	{ i_ADD, _T("ADD"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_ADDA, _T("ADDA"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_ADDX, _T("ADDX"), NULL, 0 | MNEMOFLAG_LOOPMODE },
+	{ i_ABCD, _T("ABCD"), NULL, 0 | MNEMOFLAG_LOOPMODE },
+	{ i_NEG, _T("NEG"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_NEGX, _T("NEGX"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_NBCD, _T("NBCD"), NULL, 0 | MNEMOFLAG_LOOPMODE },
+	{ i_CLR, _T("CLR"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_NOT, _T("NOT"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_TST, _T("TST"), NULL, 1 | MNEMOFLAG_LOOPMODE },
 	{ i_BTST, _T("BTST"), NULL, 1 },
 	{ i_BCHG, _T("BCHG"), NULL, 1 },
 	{ i_BCLR, _T("BCLR"), NULL, 1 },
 	{ i_BSET, _T("BSET"), NULL, 1 },
-	{ i_CMP, _T("CMP"), NULL, 1 },
-	{ i_CMPM, _T("CMPM"), NULL, 1 },
-	{ i_CMPA, _T("CMPA"), NULL, 1 },
-	{ i_MVPRM, _T("MVPRM"), NULL, 0 },
-	{ i_MVPMR, _T("MVPMR"), NULL, 0 },
-	{ i_MOVE, _T("MOVE"), NULL, 1 },
-	{ i_MOVEA, _T("MOVEA"), NULL, 1 },
-	{ i_MVSR2, _T("MVSR2"), NULL, 0 },
-	{ i_MV2SR, _T("MV2SR"), NULL, 0 },
+	{ i_CMP, _T("CMP"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_CMPM, _T("CMPM"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_CMPA, _T("CMPA"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_MVPRM, _T("MVPRM"), _T("MOVEP"), 0 },
+	{ i_MVPMR, _T("MVPMR"), _T("MOVEP"), 0 },
+	{ i_MOVE, _T("MOVE"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_MOVEA, _T("MOVEA"), NULL, 1 | MNEMOFLAG_LOOPMODE },
+	{ i_MVSR2, _T("MVSR2"), _T("MOVE"), 0 },
+	{ i_MV2SR, _T("MV2SR"), _T("MOVE"), 0 },
 	{ i_SWAP, _T("SWAP"), NULL, 0 },
 	{ i_EXG, _T("EXG"), NULL, 0 },
 	{ i_EXT, _T("EXT"), NULL, 0 },
 	{ i_MVMEL, _T("MVMEL"), _T("MOVEM"), 0 },
 	{ i_MVMLE, _T("MVMLE"), _T("MOVEM"), 0 },
 	{ i_TRAP, _T("TRAP"), NULL, 0 },
-	{ i_MVR2USP, _T("MVR2USP"), NULL, 0 },
-	{ i_MVUSP2R, _T("MVUSP2R"), NULL, 0 },
+	{ i_MVR2USP, _T("MVR2USP"), _T("MOVE"), 0 },
+	{ i_MVUSP2R, _T("MVUSP2R"), _T("MOVE"), 0 },
 	{ i_NOP, _T("NOP"), NULL, 0 },
 	{ i_RESET, _T("RESET"), NULL, 0 },
 	{ i_RTE, _T("RTE"), NULL, 0 },
@@ -90,14 +90,14 @@ struct mnemolookup lookuptab[] = {
 	{ i_ROR, _T("ROR"), NULL, 0 },
 	{ i_ROXL, _T("ROXL"), NULL, 1 },
 	{ i_ROXR, _T("ROXR"), NULL, 1 },
-	{ i_ASRW, _T("ASRW"), NULL, 1 },
-	{ i_ASLW, _T("ASLW"), NULL, 1 },
-	{ i_LSRW, _T("LSRW"), NULL, 1 },
-	{ i_LSLW, _T("LSLW"), NULL, 1 },
-	{ i_ROLW, _T("ROLW"), NULL, 1 },
-	{ i_RORW, _T("RORW"), NULL, 1 },
-	{ i_ROXLW, _T("ROXLW"), NULL, 1 },
-	{ i_ROXRW, _T("ROXRW"), NULL, 1 },
+	{ i_ASRW, _T("ASRW"), _T("ASR"), 1 | MNEMOFLAG_LOOPMODE },
+	{ i_ASLW, _T("ASLW"), _T("ASL"), 1 | MNEMOFLAG_LOOPMODE },
+	{ i_LSRW, _T("LSRW"), _T("LSR"), 1 | MNEMOFLAG_LOOPMODE },
+	{ i_LSLW, _T("LSLW"), _T("LSL"), 1 | MNEMOFLAG_LOOPMODE },
+	{ i_ROLW, _T("ROLW"), _T("ROL"), 1 | MNEMOFLAG_LOOPMODE },
+	{ i_RORW, _T("RORW"), _T("ROR"), 1 | MNEMOFLAG_LOOPMODE },
+	{ i_ROXLW, _T("ROXLW"), _T("ROXL"), 1 | MNEMOFLAG_LOOPMODE },
+	{ i_ROXRW, _T("ROXRW"), _T("ROXR"), 1 | MNEMOFLAG_LOOPMODE },
 
 	{ i_MOVE2C, _T("MOVE2C"), _T("MOVEC"), 0 },
 	{ i_MOVEC2, _T("MOVEC2"), _T("MOVEC"), 0 },
@@ -105,6 +105,7 @@ struct mnemolookup lookuptab[] = {
 	{ i_CAS2, _T("CAS2"), NULL, 1 },
 	{ i_MULL, _T("MULL"), NULL, 0 },
 	{ i_DIVL, _T("DIVL"), NULL, 0 },
+
 	{ i_BFTST, _T("BFTST"), NULL, 0 },
 	{ i_BFEXTU, _T("BFEXTU"), NULL, 0 },
 	{ i_BFCHG, _T("BFCHG"), NULL, 0 },
@@ -113,6 +114,7 @@ struct mnemolookup lookuptab[] = {
 	{ i_BFFFO, _T("BFFFO"), NULL, 0 },
 	{ i_BFSET, _T("BFSET"), NULL, 0 },
 	{ i_BFINS, _T("BFINS"), NULL, 0 },
+
 	{ i_PACK, _T("PACK"), NULL, 0 },
 	{ i_UNPK, _T("UNPK"), NULL, 0 },
 	{ i_TAS, _T("TAS"), NULL, 1 },
@@ -121,12 +123,12 @@ struct mnemolookup lookuptab[] = {
 	{ i_RTM, _T("RTM"), NULL, 0 },
 	{ i_TRAPcc, _T("TRAPcc"), NULL, 0 },
 	{ i_MOVES, _T("MOVES"), NULL, 1 },
+
 	{ i_FPP, _T("FPP"), NULL, 0 },
 	{ i_FDBcc, _T("FDBcc"), NULL, 0 },
 	{ i_FScc, _T("FScc"), NULL, 0 },
 	{ i_FTRAPcc, _T("FTRAPcc"), NULL, 0 },
 	{ i_FBcc, _T("FBcc"), NULL, 0 },
-	{ i_FBcc, _T("FBcc"), NULL, 0 },
 	{ i_FSAVE, _T("FSAVE"), NULL, 0 },
 	{ i_FRESTORE, _T("FRESTORE"), NULL, 0 },
 
@@ -150,6 +152,8 @@ struct mnemolookup lookuptab[] = {
 	{ i_PTESTW, _T("PTESTW"), NULL, 0 },
 
 	{ i_LPSTOP, _T("LPSTOP"), NULL, 0 },
+	{ i_HALT, _T("HALT"), NULL, 0 },
+	{ i_PULSE, _T("PULSE"), NULL, 0 },
 	{ i_ILLG, _T(""), NULL, 0 },
 };
 
@@ -170,7 +174,7 @@ static amodes mode_from_str (const TCHAR *str)
 	if (_tcsncmp (str, _T("PC8r"), 4) == 0) return PC8r;
 	if (_tcsncmp (str, _T("Immd"), 4) == 0) return imm;
 	abort ();
-	return 0;
+	return Dreg;
 }
 
 STATIC_INLINE amodes mode_from_mr (int mode, int reg)
@@ -196,37 +200,48 @@ STATIC_INLINE amodes mode_from_mr (int mode, int reg)
 		}
 	}
 	abort ();
-	return 0;
+	return Dreg;
 }
 
 static void build_insn (int insn)
 {
 	int find = -1;
 	int variants;
-	int isjmp = 0;
 	struct instr_def id;
 	const TCHAR *opcstr;
-	int i;
+	int i, n;
 
 	int flaglive = 0, flagdead = 0;
+	int cflow = 0;
 
 	id = defs68k[insn];
 
-	/* Note: We treat anything with unknown flags as a jump. That
-	is overkill, but "the programmer" was lazy quite often, and
-	*this* programmer can't be bothered to work out what can and
-	can't trap. Usually, this will be overwritten with the gencomp
-	based information, anyway. */
+	// Control flow information
+	cflow = id.cflow;
+
+	// Mask of flags set/used
+	unsigned char flags_set = 0, flags_used = 0;
+
+	for (i = 0, n = 4; i < 5; i++, n--) {
+		switch (id.flaginfo[i].flagset) {
+		case fa_unset: case fa_isjmp: break;
+		default: flags_set |= (1 << n);
+		}
+
+		switch (id.flaginfo[i].flaguse) {
+		case fu_unused: case fu_isjmp: break;
+		default: flags_used |= (1 << n);
+		}
+	}
 
 	for (i = 0; i < 5; i++) {
-		switch (id.flaginfo[i].flagset){
+		switch (id.flaginfo[i].flagset) {
 		case fa_unset: break;
-		case fa_isjmp: isjmp = 1; break;
-		case fa_isbranch: isjmp = 1; break;
+		case fa_isjmp: break;
 		case fa_zero: flagdead |= 1 << i; break;
 		case fa_one: flagdead |= 1 << i; break;
 		case fa_dontcare: flagdead |= 1 << i; break;
-		case fa_unknown: isjmp = 1; flagdead = -1; goto out1;
+		case fa_unknown: flagdead = -1; goto out1;
 		case fa_set: flagdead |= 1 << i; break;
 		}
 	}
@@ -235,12 +250,13 @@ out1:
 	for (i = 0; i < 5; i++) {
 		switch (id.flaginfo[i].flaguse) {
 		case fu_unused: break;
-		case fu_isjmp: isjmp = 1; flaglive |= 1 << i; break;
-		case fu_maybecc: isjmp = 1; flaglive |= 1 << i; break;
-		case fu_unknown: isjmp = 1; flaglive |= 1 << i; break;
+		case fu_isjmp: flaglive |= 1 << i; break;
+		case fu_maybecc: flaglive |= 1 << i; break;
+		case fu_unknown: flaglive = -1; goto out2;
 		case fu_used: flaglive |= 1 << i; break;
 		}
 	}
+out2:
 
 	opcstr = id.opcstr;
 	for (variants = 0; variants < (1 << id.n_variable); variants++) {
@@ -262,12 +278,14 @@ out1:
 		int usesrc = 0, usedst = 0;
 		int srctype = 0;
 		int srcpos = -1, dstpos = -1;
+		int usecc = 0;
 
 		amodes srcmode = am_unknown, destmode = am_unknown;
 		int srcreg = -1, destreg = -1;
 
-		for (i = 0; i < lastbit; i++)
+		for (i = 0; i < lastbit; i++) {
 			bitcnt[i] = bitval[i] = 0;
+		}
 
 		vmsk = 1 << id.n_variable;
 
@@ -283,6 +301,8 @@ out1:
 				bitcnt[currbit]++;
 				bitval[currbit] <<= 1;
 				bitval[currbit] |= bit_set;
+				if (currbit == bitC || currbit == bitc)
+					usecc = 1;
 			}
 		}
 
@@ -346,6 +366,7 @@ out1:
 
 		/* parse the source address */
 		usesrc = 1;
+
 		switch (opcstr[pos++]) {
 		case 'D':
 			srcmode = Dreg;
@@ -358,6 +379,7 @@ out1:
 		case 'A':
 			srcmode = Areg;
 			switch (opcstr[pos++]) {
+			case 'l': srcmode = absl; break;
 			case 'r': srcreg = bitval[bitr]; srcgather = 1; srcpos = bitpos[bitr]; break;
 			case 'R': srcreg = bitval[bitR]; srcgather = 1; srcpos = bitpos[bitR]; break;
 			default: abort();
@@ -368,9 +390,6 @@ out1:
 			case 'a': srcmode = Aind; pos++; break;
 			}
 			break;
-		case 'L':
-			srcmode = absl;
-			break;
 		case '#':
 			switch (opcstr[pos++]) {
 			case 'z': srcmode = imm; break;
@@ -416,8 +435,16 @@ out1:
 					srcpos = bitpos[bitK];
 				}
 				break;
+			case 'E': srcmode = immi; srcreg = bitval[bitE];
+				if (CPU_EMU_SIZE < 5) { // gb-- what is CPU_EMU_SIZE used for ??
+					/* 1..255 */
+					srcgather = 1;
+					srctype = 6;
+					srcpos = bitpos[bitE];
+				}
+				break;
 			case 'p': srcmode = immi; srcreg = bitval[bitK];
-				if (CPU_EMU_SIZE < 5) {
+				if (CPU_EMU_SIZE < 5) { // gb-- what is CPU_EMU_SIZE used for ??
 					/* 0..3 */
 					srcgather = 1;
 					srctype = 7;
@@ -554,21 +581,17 @@ out1:
 		case 'A':
 			destmode = Areg;
 			switch (opcstr[pos++]) {
+			case 'l': destmode = absl; break;
 			case 'r': destreg = bitval[bitr]; dstgather = 1; dstpos = bitpos[bitr]; break;
 			case 'R': destreg = bitval[bitR]; dstgather = 1; dstpos = bitpos[bitR]; break;
 			case 'x': destreg = 0; dstgather = 0; dstpos = 0; break;
 			default: abort();
 			}
-			if (dstpos < 0 || dstpos >= 32)
-				abort ();
 			switch (opcstr[pos]) {
 			case 'p': destmode = Apdi; pos++; break;
 			case 'P': destmode = Aipi; pos++; break;
 			}
 			break;
-		case 'L':
-			destmode = absl;
-			break;
 		case '#':
 			switch (opcstr[pos++]) {
 			case 'z': destmode = imm; break;
@@ -699,15 +722,15 @@ out1:
 #endif
 endofline:
 		/* now, we have a match */
-		if (table68k[opc].mnemo != i_ILLG)
-			;//write_log (_T("Double match: %x: %s\n"), opc, opcstr);
+		//if (table68k[opc].mnemo != i_ILLG)
+		//	write_log (_T("Double match: %x: %s\n"), opc, opcstr);
 		if (find == -1) {
 			for (find = 0;; find++) {
 				if (_tcscmp (mnemonic, lookuptab[find].name) == 0) {
 					table68k[opc].mnemo = lookuptab[find].mnemo;
 					break;
 				}
-				if (_tcslen (lookuptab[find].name) == 0)
+				if (lookuptab[find].name[0] == '\0')
 					abort();
 			}
 		}
@@ -715,6 +738,8 @@ endofline:
 			table68k[opc].mnemo = lookuptab[find].mnemo;
 		}
 		table68k[opc].cc = bitval[bitc];
+		table68k[opc].ccuse = usecc != 0;
+
 		mnemo = table68k[opc].mnemo;
 		if (mnemo == i_BTST
 			|| mnemo == i_BSET
@@ -754,30 +779,50 @@ endofline:
 			table68k[opc].flaginfo[i].flaguse = id.flaginfo[i].flaguse;
 		}
 #endif
+
+		// Fix flags used information for Scc, Bcc, TRAPcc, DBcc instructions
+		if (table68k[opc].mnemo == i_Scc
+			|| table68k[opc].mnemo == i_Bcc
+			|| table68k[opc].mnemo == i_DBcc
+			|| table68k[opc].mnemo == i_TRAPcc
+			) {
+			switch (table68k[opc].cc) {
+				// CC mask:	XNZVC
+				// 			 8421
+			case 0: flags_used = 0x00; break;	/*  T */
+			case 1: flags_used = 0x00; break;	/*  F */
+			case 2: flags_used = 0x05; break;	/* HI */
+			case 3: flags_used = 0x05; break;	/* LS */
+			case 4: flags_used = 0x01; break;	/* CC */
+			case 5: flags_used = 0x01; break;	/* CS */
+			case 6: flags_used = 0x04; break;	/* NE */
+			case 7: flags_used = 0x04; break;	/* EQ */
+			case 8: flags_used = 0x02; break;	/* VC */
+			case 9: flags_used = 0x02; break;	/* VS */
+			case 10:flags_used = 0x08; break;	/* PL */
+			case 11:flags_used = 0x08; break;	/* MI */
+			case 12:flags_used = 0x0A; break;	/* GE */
+			case 13:flags_used = 0x0A; break;	/* LT */
+			case 14:flags_used = 0x0E; break;	/* GT */
+			case 15:flags_used = 0x0E; break;	/* LE */
+			}
+		}
+
+#if 1
+		/* gb-- flagdead and flaglive would not have correct information */
+		table68k[opc].flagdead = flags_set;
+		table68k[opc].flaglive = flags_used;
+#else
 		table68k[opc].flagdead = flagdead;
 		table68k[opc].flaglive = flaglive;
-		table68k[opc].isjmp = isjmp;
-nomatch:
+#endif
+		table68k[opc].cflow = cflow;
+
+	nomatch:
 		/* FOO! */;
 	}
 }
 
-
-void read_table68k (void)
-{
-	int i;
-
-	free (table68k);
-	table68k = xmalloc (struct instr, 65536);
-	for (i = 0; i < 65536; i++) {
-		table68k[i].mnemo = i_ILLG;
-		table68k[i].handler = -1;
-	}
-	for (i = 0; i < n_defs68k; i++) {
-		build_insn (i);
-	}
-}
-
 static int imismatch;
 
 static void handle_merges (long int opcode)
@@ -803,6 +848,8 @@ static void handle_merges (long int opcode)
 			smsk = 7; sbitdst = 8; break;
 		case 5:
 			smsk = 63; sbitdst = 64; break;
+		case 6:
+			smsk = 255; sbitdst = 256; break;
 		case 7:
 			smsk = 3; sbitdst = 4; break;
 		default:
@@ -821,9 +868,11 @@ static void handle_merges (long int opcode)
 	for (srcreg=0; srcreg < sbitdst; srcreg++) {
 		for (dstreg=0; dstreg < dstend; dstreg++) {
 			uae_u16 code = (uae_u16)opcode;
+			uae_u8 spos = (table68k[opcode].spos < 0) ? 0 : table68k[opcode].spos;
+			uae_u8 dpos = (table68k[opcode].dpos < 0) ? 0 : table68k[opcode].dpos;
 
-			code = (code & ~smsk) | (srcreg << table68k[opcode].spos);
-			code = (code & ~dmsk) | (dstreg << table68k[opcode].dpos);
+			code = (code & ~smsk) | (srcreg << spos);
+			code = (code & ~dmsk) | (dstreg << dpos);
 
 			/* Check whether this is in fact the same instruction.
 			* The instructions should never differ, except for the
@@ -873,3 +922,63 @@ int get_no_mismatches (void)
 {
 	return imismatch;
 }
+
+static int isreg(amodes mode)
+{
+	if (mode == Dreg || mode == Areg)
+		return 1;
+	return 0;
+}
+
+bool opcode_loop_mode(uae_u16 opcode)
+{
+	struct instr *c = &table68k[opcode];
+	bool loopmode = false;
+	int i;
+	for (i = 0; lookuptab[i].name[0]; i++) {
+		if (c->mnemo == lookuptab[i].mnemo)
+			break;
+	}
+	if (lookuptab[i].flags & MNEMOFLAG_LOOPMODE) {
+		// Source is Dn,An,(An),(An)+,-(An)
+		// Destination is Dn,An,(An),(An)+,-(An)
+		// Both source and destination must not be Dn or An.
+		// RMW instruction must not be Dn or An
+		if (((isreg(c->smode) || c->smode == Aind || c->smode == Apdi || c->smode == Aipi)) &&
+			((!c->duse && !isreg(c->smode)) || (c->duse && (isreg(c->dmode) || c->dmode == Aind || c->dmode == Apdi || c->dmode == Aipi))) &&
+			(!c->duse || (isreg(c->smode) && !isreg(c->dmode)) || (!isreg(c->smode) && isreg(c->dmode)) || (!isreg(c->smode) && !isreg(c->dmode)))) {
+			loopmode = true;
+		}
+		if (c->mnemo == i_MOVE || c->mnemo == i_MOVEA) {
+			// move x,reg: not supported
+			if (isreg(c->dmode))
+				loopmode = false;
+			// move reg,-(an): not supported
+			if (isreg(c->smode) && c->dmode == Apdi)
+				loopmode = false;
+		}
+	}
+	return loopmode;
+}
+
+void init_table68k(void)
+{
+	int i;
+
+	free(table68k);
+	table68k = xmalloc(struct instr, 65536);
+	for (i = 0; i < 65536; i++) {
+		table68k[i].mnemo = i_ILLG;
+		table68k[i].handler = -1;
+	}
+	for (i = 0; i < n_defs68k; i++) {
+		build_insn(i);
+	}
+	do_merges();
+}
+
+void exit_table68k(void)
+{
+	free(table68k);
+	table68k = NULL;
+}
