@@ -10,18 +10,13 @@
 
 /* for debugui.c */
 extern bool BreakCond_Save(const char *filename);
-extern char *BreakCond_MatchCpuVariable(const char *text, int state);
-extern char *BreakCond_MatchDspVariable(const char *text, int state);
-
-/* for evaluate.c */
-extern bool BreakCond_GetHatariVariable(const char *name, Uint32 *value);
 
 /* for debugcpu.c & debugdsp.c */
 extern const char BreakCond_Description[];
 extern const char BreakAddr_Description[];
 
-extern int BreakCond_MatchCpu(void);
-extern int BreakCond_MatchDsp(void);
+extern bool BreakCond_MatchCpu(void);
+extern bool BreakCond_MatchDsp(void);
 extern int BreakCond_CpuBreakPointCount(void);
 extern int BreakCond_DspBreakPointCount(void);
 extern bool BreakCond_Command(const char *expression, bool bForDsp);
