@@ -8,11 +8,12 @@
   pressed, the emulator is (hopefully) halted and this little CLI can be used
   (in the terminal box) for debugging tasks like memory and register dumps.
 */
-const char DebugUI_fileid[] = "Hatari debugui.c : " __DATE__ " " __TIME__;
+const char DebugUI_fileid[] = "Hatari debugui.c";
 
 #include <ctype.h>
 #include <stdio.h>
 #include <unistd.h>
+#include <SDL.h>
 
 #include "config.h"
 
@@ -28,6 +29,7 @@ const char DebugUI_fileid[] = "Hatari debugui.c : " __DATE__ " " __TIME__;
 #include "log.h"
 #include "m68000.h"
 #include "memorySnapShot.h"
+#include "screenSnapShot.h"
 #include "options.h"
 #include "reset.h"
 #include "screen.h"
@@ -43,6 +45,7 @@ const char DebugUI_fileid[] = "Hatari debugui.c : " __DATE__ " " __TIME__;
 #include "debugui.h"
 #include "evaluate.h"
 #include "history.h"
+#include "profile.h"
 #include "symbols.h"
 #include "vars.h"
 
@@ -54,8 +57,9 @@ static int debugCommands;
 /* stores last 'e' command result as hex, used for TAB-completion */
 static char lastResult[10];
 
-/* parse debugger commands from here on init */
-static const char *parseFileName;
+/* array of files from which to read debugger commands after debugger is initialized */
+static char **parseFileNames;
+static int parseFiles;
 
 /* to which directory to change after (potentially recursed) scripts parsing finishes */
 static char *finalDir;
@@ -68,8 +72,7 @@ void DebugUI_MemorySnapShot_Capture(const char *path, bool bSave)
 {
 	char *filename;
 
-	filename = malloc(strlen(path) + strlen(".debug") + 1);
-	assert(filename);
+	filename = Str_Alloc(strlen(path) + strlen(".debug"));
 	strcpy(filename, path);
 	strcat(filename, ".debug");
 	
@@ -87,7 +90,7 @@ void DebugUI_MemorySnapShot_Capture(const char *path, bool bSave)
 		if (File_Exists(filename))
 		{
 			/* and parse back the saved breakpoints */
-			DebugUI_ParseFile(filename, true);
+			DebugUI_ParseFile(filename, true, true);
 		}
 	}
 	free(filename);
@@ -138,28 +141,36 @@ static int DebugUI_SetLogFile(int nArgc, char *psArgs[])
 	return DEBUGGER_CMDDONE;
 }
 
-
 /**
- * Helper to print given value in all supported number bases
+ * Helper to output given value as binary number
  */
-static void DebugUI_PrintValue(Uint32 value)
+void DebugUI_PrintBinary(FILE *fp, int minwidth, uint32_t value)
 {
 	bool one, ones;
 	int bit;
 
-	fputs("= %", stderr);
 	ones = false;
 	for (bit = 31; bit >= 0; bit--)
 	{
-		one = value & (1<<bit);
-		if (one || ones)
+		one = value & (1U << bit);
+		if (ones || bit < minwidth || one)
 		{
-			fputc(one ? '1':'0', stderr);
+			fputc(one ? '1':'0', fp);
 			ones = true;
 		}
 	}
 	if (!ones)
-		fputc('0', stderr);
+		fputc('0', fp);
+}
+
+/**
+ * Helper to print given value in all supported number bases
+ */
+static void DebugUI_PrintValue(uint32_t value)
+{
+	fputs("= %", stderr);
+	DebugUI_PrintBinary(stderr, 0, value);
+
 	if (value & 0x80000000)
 		fprintf(stderr, " (bin), #%u/%d (dec), $%x (hex)\n", value, (int)value, value);
 	else
@@ -170,12 +181,12 @@ static void DebugUI_PrintValue(Uint32 value)
 
 
 /**
- * Commmand: Evaluate an expression with CPU reg and symbol parsing.
+ * Command: Evaluate an expression with CPU reg and symbol parsing.
  */
 static int DebugUI_Evaluate(int nArgc, char *psArgs[])
 {
 	const char *errstr, *expression = (const char *)psArgs[1];
-	Uint32 result;
+	uint32_t result;
 	int offset;
 
 	if (nArgc < 2)
@@ -219,7 +230,7 @@ static char *DebugUI_EvaluateExpressions(const char *initial)
 	char *end, *start, *input;
 	const char *errstr;
 	char valuestr[12];
-	Uint32 value;
+	uint32_t value;
 	bool fordsp;
 
 	/* input is split later on, need to save len here */
@@ -310,10 +321,12 @@ static int DebugUI_DoMemorySnap(int argc, char *argv[])
 	else
 		file = ConfigureParams.Memory.szMemoryCaptureFileName;
 
+	/* [NP] TODO : we need to restart emulation to complete restore, */
+	/* it can't be done immediately. Try to call m68k_go() and go back automatically to debugger ? */
 	if (strcmp(argv[0], "stateload") == 0)
 		MemorySnapShot_Restore(file, true);
 	else
-		MemorySnapShot_Capture(file, true);
+		MemorySnapShot_Capture_Immediate(file, true);
 
 	return DEBUGGER_CMDDONE;
 }
@@ -377,6 +390,18 @@ static int DebugUI_SetOptions(int argc, char *argv[])
 }
 
 
+/**
+ * Command: Screenshot
+ */
+static int DebugUI_Screenshot(int argc, char *argv[])
+{
+	if (argc == 2)
+		ScreenSnapShot_SaveToFile(argv[1]);
+	else
+		return DebugUI_PrintCmdHelp(argv[0]);
+	return DEBUGGER_CMDDONE;
+}
+
 /**
  * Command: Set tracing
  */
@@ -394,7 +419,6 @@ static int DebugUI_SetTracing(int argc, char *argv[])
 	return DEBUGGER_CMDDONE;
 }
 
-
 /**
  * Command: Change Hatari work directory
  */
@@ -417,6 +441,21 @@ static int DebugUI_ChangeDir(int argc, char *argv[])
 	return DebugUI_PrintCmdHelp(argv[0]);
 }
 
+/**
+ * Command: Print strings to debug log output, with escape handling.
+ */
+static int DebugUI_Echo(int argc, char *argv[])
+{
+	if (argc < 2)
+		return DebugUI_PrintCmdHelp(argv[0]);
+	for (int i = 1; i < argc; i++)
+	{
+		Str_UnEscape(argv[i]);
+		fputs(argv[i], debugOutput);
+	}
+	return DEBUGGER_CMDDONE;
+}
+
 /**
  * Command: Rename file
  */
@@ -461,7 +500,7 @@ static int DebugUI_Reset(int argc, char *argv[])
 static int DebugUI_CommandsFromFile(int argc, char *argv[])
 {
 	if (argc == 2)
-		DebugUI_ParseFile(argv[1], true);
+		DebugUI_ParseFile(argv[1], true, true);
 	else
 		DebugUI_PrintCmdHelp(argv[0]);
 	return DEBUGGER_CMDDONE;
@@ -613,7 +652,7 @@ static int DebugUI_ParseCommand(const char *input_orig)
 	if (cmd == -1)
 	{
 		fprintf(stderr, "Command '%s' not found.\n"
-			"Use 'help' to view a list of available commands.\n",
+			"Use 'help' to view a list of available debugger commands.\n",
 			psArgs[0]);
 		free(input);
 		return DEBUGGER_CMDDONE;
@@ -631,14 +670,24 @@ static int DebugUI_ParseCommand(const char *input_orig)
 		if (psArgs[nArgc] == NULL)
 			break;
 	}
-
-	/* ... and execute the function */
-	retval = debugCommand[i].pFunction(nArgc, psArgs);
+	if (nArgc >= ARRAY_SIZE(psArgs))
+	{
+		fprintf(stderr, "Error: too many arguments (currently up to %d supported)\n",
+			ARRAY_SIZE(psArgs));
+		retval = DEBUGGER_CMDCONT;
+	}
+	else
+	{
+		/* ... and execute the function */
+		retval = debugCommand[i].pFunction(nArgc, psArgs);
+	}
 	/* Save commando string if it can be repeated */
-	if (retval == DEBUGGER_CMDCONT)
+	if (retval == DEBUGGER_CMDCONT || retval == DEBUGGER_ENDCONT)
 	{
 		if (psArgs[0] != sLastCmd)
-			strlcpy(sLastCmd, psArgs[0], sizeof(sLastCmd));
+			Str_Copy(sLastCmd, psArgs[0], sizeof(sLastCmd));
+		if (retval == DEBUGGER_ENDCONT)
+			retval = DEBUGGER_END;
 	}
 	else
 		sLastCmd[0] = '\0';
@@ -809,8 +858,12 @@ static void DebugUI_FreeCommand(char *input)
  */
 static char *DebugUI_GetCommand(char *input)
 {
+	/* We need this indirection for libedit's rl_readline_name which is
+	 * not declared as "const char *" (i.e. this is necessary for macOS) */
+	static char hatari_readline_name[] = "Hatari";
+
 	/* Allow conditional parsing of the ~/.inputrc file. */
-	rl_readline_name = "Hatari";
+	rl_readline_name = hatari_readline_name;
 	
 	/* Tell the completer that we want a crack first. */
 	rl_attempted_completion_function = DebugUI_Completion;
@@ -847,7 +900,7 @@ static void DebugUI_GetScreenSize(int *rows, int *cols)
 	*cols = 80;
 	if ((p = getenv("LINES")) != NULL)
 		*rows = (int)strtol(p, NULL, 0);
-	if ((p = getenv("COLUMS")) != NULL)
+	if ((p = getenv("COLUMNS")) != NULL)
 		*cols = (int)strtol(p, NULL, 0);
 }
 
@@ -864,7 +917,8 @@ static char *DebugUI_GetCommand(char *input)
 	if (!input)
 	{
 		input = malloc(256);
-		assert(input);
+		if (!input)
+			return NULL;
 	}
 	input[0] = '\0';
 	if (fgets(input, 256, stdin) == NULL)
@@ -912,6 +966,12 @@ static const dbgcommand_t uicommand[] =
 	  "\tChange Hatari work directory. With '-f', directory is\n"
 	  "\tchanged only after all script files have been parsed.",
 	  false },
+	{ DebugUI_Echo, NULL,
+	  "echo", "",
+	  "output given string(s)",
+	  "<string(s)>\n"
+	  "\tUse e.g. 'echo \\ec' to clear screen in a breakpoint.",
+	  false },
 	{ DebugUI_Evaluate, Vars_MatchCpuVariable,
 	  "evaluate", "e",
 	  "evaluate an expression",
@@ -919,13 +979,13 @@ static const dbgcommand_t uicommand[] =
 	  "\tEvaluate an expression and show the result.  Expression can\n"
 	  "\tinclude CPU register & symbol and Hatari variable names.\n"
 	  "\tThose are replaced by their values. Supported operators in\n"
-	  "\texpressions are, in the decending order of precedence:\n"
+	  "\texpressions are, in the descending order of precedence:\n"
 	  "\t\t(), +, -, ~, *, /, +, -, >>, <<, ^, &, |\n"
-	  "\tParenthesis will fetch a _long_ value from the address\n"
-	  "\tto what the value inside it evaluates to. Prefixes can be\n"
+	  "\tParenthesis fetch long value from the given address,\n"
+	  "\tunless .<width> suffix is given. Prefixes can be\n"
 	  "\tused only in start of line or parenthesis.\n"
 	  "\tFor example:\n"
-	  "\t\t~%101 & $f0f0f ^ (d0 + 0x21)\n"
+	  "\t\t~%101 & $f0f0f ^ (d0 + 0x21).w\n"
 	  "\tResult value is shown as binary, decimal and hexadecimal.\n"
 	  "\tAfter this, '$' will TAB-complete to last result value.",
 	  true },
@@ -937,11 +997,11 @@ static const dbgcommand_t uicommand[] =
 	  false },
 	{ History_Parse, History_Match,
 	  "history", "hi",
-	  "show last CPU/DSP PC values & executed instructions",
+	  "show last CPU and/or DSP PC values + instructions",
 	  "cpu|dsp|on|off|<count> [limit]|save <file>\n"
-	  "\t'cpu' and 'dsp' enable instruction history tracking for just given\n"
+	  "\t'cpu' and 'dsp' enable program counter history tracking for given\n"
 	  "\tprocessor, 'on' tracks them both, 'off' will disable history.\n"
-	  "\tOptional 'limit' will set how many past instructions are tracked.\n"
+	  "\tOptional 'limit' will set how many past addresses are tracked.\n"
 	  "\tGiving just count will show (at max) given number of last saved PC\n"
 	  "\tvalues and instructions currently at corresponding RAM addresses.",
 	  false },
@@ -961,7 +1021,7 @@ static const dbgcommand_t uicommand[] =
 	  false },
 	{ DebugUI_SetLogFile, NULL,
 	  "logfile", "f",
-	  "open or close log file",
+	  "set (memdump/disasm/registers) log file",
 	  "[filename]\n"
 	  "\tOpen log file, no argument closes the log file. Output of\n"
 	  "\tregister & memory dumps and disassembly will be written to it.",
@@ -978,14 +1038,19 @@ static const dbgcommand_t uicommand[] =
 	{ DebugUI_Rename, NULL,
 	  "rename", "",
 	  "rename given file",
-	  "old new\n"
-	  "\tRenames file with 'old' name to 'new'.",
+	  "<old> <new>\n"
+	  "\tRename file with <old> name to <new>.",
 	  false },
 	{ DebugUI_Reset, DebugUI_MatchReset,
 	  "reset", "",
 	  "reset emulation",
 	  "<soft|hard>\n",
 	  false },
+	{ DebugUI_Screenshot, NULL,
+	  "screenshot", "",
+	  "save screenshot to given file",
+	  "<filename>\n",
+	  false },
 	{ DebugUI_SetOptions, Opt_MatchOption,
 	  "setopt", "o",
 	  "set Hatari command line and debugger options",
@@ -1038,7 +1103,9 @@ void DebugUI_Init(void)
 	const dbgcommand_t *cpucmd, *dspcmd;
 	int cpucmds, dspcmds;
 
-	/* already intialized? */
+	Log_ResetMsgRepeat();
+
+	/* already initialized? */
 	if (debugCommands)
 		return;
 
@@ -1062,29 +1129,73 @@ void DebugUI_Init(void)
 	memcpy(&debugCommand[debugCommands], dspcmd, sizeof(dbgcommand_t) * dspcmds);
 	debugCommands += dspcmds;
 
-	if (parseFileName)
-		DebugUI_ParseFile(parseFileName, true);
+
+	if (parseFiles)
+	{
+		int i;
+		for (i = 0; i < parseFiles; i++)
+		{
+			DebugUI_ParseFile(parseFileNames[i], true, true);
+			free(parseFileNames[i]);
+		}
+		free(parseFileNames);
+		parseFileNames = NULL;
+		parseFiles = 0;
+	}
+}
+
+/**
+ * Debugger user interface de-initialization / free.
+ */
+void DebugUI_UnInit(void)
+{
+	Profile_CpuFree();
+	Profile_DspFree();
+	Symbols_FreeAll();
+	free(debugCommand);
+	debugCommands = 0;
 }
 
 
 /**
- * Set debugger commands file during Hatari startup before things
- * needed by the debugger are initialized so that it can be parsed
- * when debugger itself gets initialized.
- * Return true if file exists, false otherwise.
+ * Return true if user wants to quit current command
  */
-bool DebugUI_SetParseFile(const char *path)
+bool DebugUI_DoQuitQuery(const char *info)
 {
-	if (File_Exists(path))
-	{
-		parseFileName = path;
+	char input[8];
+	fprintf(stderr, "--- q to exit %s, enter to continue --- ", info);
+	if (fgets(input, sizeof(input), stdin) == NULL ||
+	    toupper(input[0]) == 'Q') {
 		return true;
 	}
-	fprintf(stderr, "ERROR: debugger input file '%s' missing.\n", path);
 	return false;
 }
 
 
+/**
+ * Add debugger command files during Hatari startup before things
+ * needed by the debugger are initialized so that it can be parsed
+ * when debugger itself gets initialized.
+ * Return true if file exists and it could be added, false otherwise.
+ */
+bool DebugUI_AddParseFile(const char *path)
+{
+	if (!File_Exists(path))
+	{
+		fprintf(stderr, "ERROR: debugger input file '%s' missing.\n", path);
+		return false;
+	}
+	parseFileNames = realloc(parseFileNames, (parseFiles+1)*sizeof(char*));
+	if (!parseFileNames)
+	{
+		perror("DebugUI_AddParseFile");
+		return false;
+	}
+	parseFileNames[parseFiles++] = strdup(path);
+	return true;
+}
+
+
 /**
  * Debugger user interface main function.
  */
@@ -1114,7 +1225,7 @@ void DebugUI(debug_reason_t reason)
 	 * this is invoked from.  E.g. returning from fullscreen
 	 * enables grab if that was enabled on windowed mode.
 	 */
-	SDL_WM_GrabInput(SDL_GRAB_OFF);
+	SDL_SetRelativeMouseMode(false);
 
 	DebugUI_Init();
 
@@ -1170,18 +1281,21 @@ void DebugUI(debug_reason_t reason)
 
 
 /**
- * Read debugger commands from a file.  If 'reinit' is set
- * (as it normally should), reinitialize breakpoints etc.
- * afterwards. return false for error, true for success.
+ * Read debugger commands from a file.  If 'reinit' is set (as it
+ * normally should), reinitialize breakpoints etc. afterwards.
+ * Processed command lines are printed if 'verbose' is set.
+ * return false for error, true for success.
  */
-bool DebugUI_ParseFile(const char *path, bool reinit)
+bool DebugUI_ParseFile(const char *path, bool reinit, bool verbose)
 {
 	int recurse;
 	static int recursing;
-	char *olddir, *dir, *cmd, *input, *expanded, *slash;
+	char *olddir, *dir, *cmd, *expanded, *slash;
+	char input[256];
 	FILE *fp;
 
-	fprintf(stderr, "Reading debugger commands from '%s'...\n", path);
+	if (verbose)
+		fprintf(stderr, "Reading debugger commands from '%s'...\n", path);
 	if (!(fp = fopen(path, "r")))
 	{
 		perror("ERROR");
@@ -1209,24 +1323,16 @@ bool DebugUI_ParseFile(const char *path, bool reinit)
 			fclose(fp);
 			return false;
 		}
-		fprintf(stderr, "Changed to input file dir '%s'.\n", dir);
+		if (verbose)
+			fprintf(stderr, "Changed to input file dir '%s'.\n", dir);
 	}
 	free(dir);
 
 	recurse = recursing;
 	recursing = true;
 
-	input = NULL;
-	for (;;)
+	while (fgets(input, sizeof(input), fp) != NULL)
 	{
-		if (!input)
-		{
-			input = malloc(256);
-			assert(input);
-		}
-		if (!fgets(input, 256, fp))
-			break;
-
 		/* ignore empty and comment lines */
 		cmd = Str_Trim(input);
 		if (!*cmd || *cmd == '#')
@@ -1238,20 +1344,20 @@ bool DebugUI_ParseFile(const char *path, bool reinit)
 			continue;
 
 		cmd = Str_Trim(expanded);
-		fprintf(stderr, "> %s\n", cmd);
+		if (verbose)
+			fprintf(stderr, "> %s\n", cmd);
 		DebugUI_ParseCommand(cmd);
 		free(expanded);
 	}
 	recursing = false;
 
-	free(input);
 	fclose(fp);
 
 	if (olddir)
 	{
 		if (chdir(olddir) != 0)
 			perror("ERROR");
-		else
+		else if (verbose)
 			fprintf(stderr, "Changed back to '%s' dir.\n", olddir);
 		free(olddir);
 	}
@@ -1263,7 +1369,7 @@ bool DebugUI_ParseFile(const char *path, bool reinit)
 		{
 			if (chdir(finalDir) != 0)
 				perror("ERROR");
-			else
+			else if(verbose)
 				fprintf(stderr, "Delayed change to '%s' dir.\n", finalDir);
 			free(finalDir);
 			finalDir = NULL;
@@ -1321,9 +1427,11 @@ void DebugUI_Exceptions(int nr, long pc)
 		{ EXCEPT_ILLEGAL,   "Illegal instruction" },	/* 4 */
 		{ EXCEPT_ZERODIV,   "Div by zero" },		/* 5 */
 		{ EXCEPT_CHK,       "CHK" },			/* 6 */
-		{ EXCEPT_TRAPV,     "TRAPV" },			/* 7 */
+		{ EXCEPT_TRAPV,     "TRAPCc/TRAPV" },		/* 7 */
 		{ EXCEPT_PRIVILEGE, "Privilege violation" },	/* 8 */
-		{ EXCEPT_TRACE,     "Trace" }			/* 9 */
+		{ EXCEPT_TRACE,     "Trace" },			/* 9 */
+		{ EXCEPT_LINEA,     "Line-A" },			/* 10 */
+		{ EXCEPT_LINEF,     "Line-F" }			/* 11 */
 	};
 	nr -= 2;
 	if (nr < 0  || nr >= ARRAY_SIZE(ex))
