@@ -1,18 +1,27 @@
 /*
  * Hatari - profile.c
- * 
- * Copyright (C) 2010-2015 by Eero Tamminen
+ *
+ * Copyright (C) 2010-2023 by Eero Tamminen
  *
  * This file is distributed under the GNU General Public License, version 2
  * or at your option any later version. Read the file gpl.txt for details.
  *
  * profile.c - profile caller info handling and debugger parsing functions
  */
-const char Profile_fileid[] = "Hatari profile.c : " __DATE__ " " __TIME__;
+const char Profile_fileid[] = "Hatari profile.c";
 
 #include <stdio.h>
 #include <assert.h>
 #include <inttypes.h>
+
+#include "config.h"
+
+#if HAVE_LIBREADLINE
+# include <readline/readline.h>
+#else
+# define rl_filename_completion_function(x,y) NULL
+#endif
+
 #include "main.h"
 #include "version.h"
 #include "debugui.h"
@@ -20,12 +29,16 @@ const char Profile_fileid[] = "Hatari profile.c : " __DATE__ " " __TIME__;
 #include "configuration.h"
 #include "clocks_timings.h"
 #include "evaluate.h"
+#include "symbols.h"
 #include "profile.h"
 #include "profile_priv.h"
-#include "symbols.h"
+#include "m68000.h"
+#include "dsp.h"
 
 profile_loop_t profile_loop;
 
+#define CALLER_SEPARATOR ','
+
 
 /* ------------------ CPU/DSP caller information handling ----------------- */
 
@@ -48,8 +61,8 @@ static const struct {
  */
 static int cmp_callers(const void *c1, const void *c2)
 {
-	Uint32 calls1 = ((const caller_t*)c1)->calls;
-	Uint32 calls2 = ((const caller_t*)c2)->calls;
+	uint32_t calls1 = ((const caller_t*)c1)->calls;
+	uint32_t calls2 = ((const caller_t*)c2)->calls;
 	if (calls1 > calls2) {
 		return -1;
 	}
@@ -83,7 +96,7 @@ static bool output_counter_info(FILE *fp, counters_t *counter)
 /**
  * output caller call counts, call type(s) and costs
  */
-static void output_caller_info(FILE *fp, caller_t *info, Uint32 *typeaddr)
+static void output_caller_info(FILE *fp, caller_t *info, uint32_t *typeaddr)
 {
 	int k, typecount;
 
@@ -108,22 +121,19 @@ static void output_caller_info(FILE *fp, caller_t *info, Uint32 *typeaddr)
 			       info->addr, info->calls, info->own.calls);
 		}
 	}
-	fputs(", ", fp);
+	fprintf(fp, "%c ", CALLER_SEPARATOR);
 }
 
-/*
- * Show collected CPU/DSP callee/caller information.
- *
- * Hint: As caller info list is based on number of loaded symbols,
- * load only text symbols to save memory & make things faster...
+/**
+ * Show collected CPU/DSP callee/caller information
  */
-void Profile_ShowCallers(FILE *fp, int sites, callee_t *callsite, const char * (*addr2name)(Uint32, Uint64 *))
+void Profile_ShowCallers(FILE *fp, int sites, callee_t *callsite, const char * (*addr2name)(uint32_t, uint64_t *))
 {
 	int i, j, countissues, countdiff;
 	const char *name;
 	caller_t *info;
-	Uint64 total;
-	Uint32 addr, typeaddr;
+	uint64_t total;
+	uint32_t addr, typeaddr;
 
 	/* legend */
 	fputs("# <callee>: <caller1> = <calls> <types>[ <inclusive/totals>[ <exclusive/totals>]], <caller2> ..., <callee name>", fp);
@@ -153,7 +163,19 @@ void Profile_ShowCallers(FILE *fp, int sites, callee_t *callsite, const char * (
 			total -= info->calls;
 			output_caller_info(fp, info, &typeaddr);
 		}
-		if (name) {
+		/* Symbol name output here is used only for debugging
+		 * profiler functionality, post-processor does not
+		 * need it.
+		 *
+		 * Output them only if they do not include caller
+		 * separator (','), because that would mess up
+		 * post-processor profile data parsing.
+		 *
+		 * Skip also all mangled (C++) symbols as they could
+		 * include that separator after demangling.
+		 */
+		if (name && strchr(name, CALLER_SEPARATOR) == NULL
+		    && strncmp(name, "__Z", 3) != 0) {
 			fprintf(fp, "%s", name);
 		}
 		fputs("\n", fp);
@@ -165,7 +187,7 @@ void Profile_ShowCallers(FILE *fp, int sites, callee_t *callsite, const char * (
 			countissues++;
 		}
 		if (typeaddr) {
-			fprintf(stderr, "WARNING: different types of calls (at least) from 0x%x (to 0x%x),\n\t has its codechanged during profiling?\n",
+			fprintf(stderr, "WARNING: different types of calls (at least) from 0x%x (to 0x%x),\n\t has its code changed during profiling?\n",
 				typeaddr, callsite->addr);
 		}
 	}
@@ -235,11 +257,13 @@ static void add_callee_cost(callee_t *callsite, callstack_t *stack)
 	assert(0);
 }
 
-
-static void add_caller(callee_t *callsite, Uint32 pc, Uint32 prev_pc, calltype_t flag)
+/**
+ * Add new caller or updated earlier caller stats for call site
+ */
+static void add_caller(callee_t *callsite, uint32_t pc, uint32_t prev_pc, calltype_t flag)
 {
+	int i, count, oldcount;
 	caller_t *info;
-	int i, count;
 
 	/* need to store real call addresses as symbols can change
 	 * after profiling has been stopped
@@ -258,21 +282,24 @@ static void add_caller(callee_t *callsite, Uint32 pc, Uint32 prev_pc, calltype_t
 	}
 	/* how many caller slots are currently allocated? */
 	count = callsite->count;
+	oldcount = 0;
 	for (;;) {
-		for (i = 0; i < count; i++, info++) {
+		for (i = oldcount; i < count; i++, info++) {
 			if (info->addr == prev_pc) {
+				/* increment caller */
 				info->flags |= flag;
 				info->calls++;
 				return;
 			}
 			if (!info->addr) {
-				/* empty slot */
+				/* add caller to empty slot */
 				info->addr = prev_pc;
 				info->flags |= flag;
 				info->calls = 1;
 				return;
 			}
 		}
+		oldcount = count;
 		/* not enough, double caller slots */
 		count *= 2;
 		info = realloc(callsite->callers, count * sizeof(*info));
@@ -280,19 +307,20 @@ static void add_caller(callee_t *callsite, Uint32 pc, Uint32 prev_pc, calltype_t
 			fprintf(stderr, "ERROR: caller info alloc failed!\n");
 			return;
 		}
-		memset(info + callsite->count, 0, callsite->count * sizeof(*info));
 		callsite->callers = info;
 		callsite->count = count;
+		info = info + oldcount;
+		memset(info, 0, oldcount * sizeof(*info));
 	}
 }
 
 /**
- * Add information about called symbol, and if it was subroutine
+ * Add information about the called symbol, and if it was a subroutine
  * call, add it to stack of functions which total costs are tracked.
  * callinfo.return_pc needs to be set before invoking this if the call
  * is of type CALL_SUBROUTINE.
  */
-void Profile_CallStart(int idx, callinfo_t *callinfo, Uint32 prev_pc, calltype_t flag, Uint32 pc, counters_t *totalcost)
+void Profile_CallStart(int idx, callinfo_t *callinfo, uint32_t prev_pc, calltype_t flag, uint32_t pc, counters_t *totalcost)
 {
 	callstack_t *stack;
 	int count;
@@ -304,8 +332,8 @@ void Profile_CallStart(int idx, callinfo_t *callinfo, Uint32 prev_pc, calltype_t
 
 	add_caller(callinfo->site + idx, pc, prev_pc, flag);
 
-	/* subroutine call which will return? */
-	if (flag != CALL_SUBROUTINE) {
+	/* subroutine/exception call i.e. one which will return? */
+	if (flag != CALL_SUBROUTINE && flag != CALL_EXCEPTION) {
 		/* no, some other call type */
 		return;
 	}
@@ -360,7 +388,7 @@ void Profile_CallStart(int idx, callinfo_t *callinfo, Uint32 prev_pc, calltype_t
  * costs and update callinfo->return_pc value.  Return address of
  * the instruction which did the returned call.
  */
-Uint32 Profile_CallEnd(callinfo_t *callinfo, counters_t *totalcost)
+uint32_t Profile_CallEnd(callinfo_t *callinfo, counters_t *totalcost)
 {
 	callstack_t *stack;
 
@@ -402,48 +430,144 @@ Uint32 Profile_CallEnd(callinfo_t *callinfo, counters_t *totalcost)
 	return stack->caller_addr;
 }
 
+
 /**
- * Add costs to all functions still in call stack
+ * Add costs to all functions still in call stack and print their names
+ *
+ * Diagram of variables involved in 2 functions deep call stack:
+ *
+ * <pre>
+ *   (caller_addr 1)  bsr symbol1  -1->  symbol1     (callee_addr 1)
+ *      (ret_addr 1)  <instr>     <-.    ...
+ *                                  |    bsr symbol2 (caller_addr 2)  -2->  symbol2 (callee_addr 2)
+ *                                  1    <instr>     (ret_addr 2)    <-.    ...
+ *                                  |    ...                           2    ... (PC)
+ *                                  '-   rts                           '-   rts
+ * </pre>
+ *
+ * When one wants to match callee_addr (= symbol) to caller_addr (= which
+ * place in that function called further functions), it's best to traverse
+ * stack from last item to top, as these items are in following call stack
+ * items, not in same one.
  */
-void Profile_FinalizeCalls(callinfo_t *callinfo, counters_t *totalcost, const char* (*get_symbol)(Uint32 addr))
+void Profile_FinalizeCalls(uint32_t pc, callinfo_t *callinfo, counters_t *totalcost,
+			   const char* (*get_symbol)(uint32_t, symtype_t),
+			   const char* (*get_caller)(uint32_t*))
 {
-	Uint32 addr;
+	const char *sym, *caller;
+	uint32_t sym_addr, caller_addr;
+	int i, lines, offset;
+	bool dots;
+	char sign;
+
 	if (!callinfo->depth) {
 		return;
 	}
 	fprintf(stderr, "Finalizing costs for %d non-returned functions:\n", callinfo->depth);
+
+	i = 0;
+	dots = false;
+	caller_addr = pc;
+	lines = ConfigureParams.Debugger.nBacktraceLines;
 	while (callinfo->depth > 0) {
+
+		/* finalize & decrease callinfo->depth */
 		Profile_CallEnd(callinfo, totalcost);
-		addr = callinfo->stack[callinfo->depth].callee_addr;
-		fprintf(stderr, "- 0x%x: %s (return = 0x%x)\n", addr, get_symbol(addr),
-			callinfo->stack[callinfo->depth].ret_addr);
+		if (++i > lines && lines > 0) {
+			continue;
+		}
+
+		/* Skip showing middle part of a long callstack as messed
+		 * callstacks could be thousands of frames deep...
+		 */
+		if (i >= 32 && callinfo->depth > 32) {
+			if (!dots) {
+				fprintf(stderr, "- ...\n");
+				dots = true;
+			}
+		} else {
+			sym_addr = callinfo->stack[callinfo->depth].callee_addr;
+			sym = get_symbol(sym_addr, SYMTYPE_CODE);
+
+			if (sym) {
+				offset = caller_addr - sym_addr;
+				sign = offset >= 0 ? '+' : '-';
+				fprintf(stderr, "- %d. 0x%06x: %s %c0x%x",
+					i, caller_addr, sym, sign, abs(offset));
+			} else {
+				fprintf(stderr, "- %d. 0x%06x", i, caller_addr);
+			}
+
+			sym_addr = caller_addr;
+			caller = get_caller(&sym_addr);
+			if (caller && caller != sym) {
+				offset = caller_addr - sym_addr;
+				fprintf(stderr, " (%s +0x%x)\n",
+					caller, abs(offset));
+			} else {
+				fprintf(stderr, "\n");
+			}
+		}
+		caller_addr = callinfo->stack[callinfo->depth].caller_addr;
 	}
 }
 
 /**
- * Show current profile stack
+ * Show current profile call stack, up to configured max backtrace depth
  */
 static void Profile_ShowStack(bool forDsp)
 {
-	int i;
-	Uint32 addr;
+	const char *sym, *caller;
+	const char *(*get_caller)(uint32_t*);
+	const char *(*get_symbol)(uint32_t, symtype_t);
+	uint32_t sym_addr, caller_addr;
+	int i, offset, depth, top;
 	callinfo_t *callinfo;
-	const char* (*get_symbol)(Uint32 addr);
 
 	if (forDsp) {
-		Profile_DspGetCallinfo(&callinfo, &get_symbol);
+		Profile_DspGetCallinfo(&callinfo, &get_caller, &get_symbol);
+		caller_addr = DSP_GetPC();
 	} else {
-		Profile_CpuGetCallinfo(&callinfo, &get_symbol);
+		Profile_CpuGetCallinfo(&callinfo, &get_caller, &get_symbol);
+		caller_addr = M68000_GetPC();
 	}
 	if (!callinfo->depth) {
 		fprintf(stderr, "Empty stack.\n");
 		return;
 	}
 
-	for (i = 0; i < callinfo->depth; i++) {
-		addr = callinfo->stack[i].callee_addr;
-		fprintf(stderr, "- 0x%x: %s (return = 0x%x)\n", addr,
-			get_symbol(addr), callinfo->stack[i].ret_addr);
+	depth = callinfo->depth;
+	top = ConfigureParams.Debugger.nBacktraceLines;
+	if (top > 0 && top < depth) {
+		top = depth - top;
+	} else {
+		top = 0;
+	}
+	i = 0;
+	while (depth-- > top) {
+		i++;
+		sym_addr = callinfo->stack[depth].callee_addr;
+		offset = caller_addr - sym_addr;
+
+		sym = get_symbol(sym_addr, SYMTYPE_CODE);
+		if (sym) {
+			char sign = offset >= 0 ? '+' : '-';
+			fprintf(stderr, "- %d. 0x%06x: %s %c0x%x",
+				i, caller_addr, sym, sign, abs(offset));
+		} else {
+			fprintf(stderr, "- %d. 0x%06x", i, caller_addr);
+		}
+
+		sym_addr = caller_addr;
+		caller = get_caller(&sym_addr);
+		if (caller && caller != sym) {
+			offset = caller_addr - sym_addr;
+			fprintf(stderr, " (%s +0x%x)\n",
+				caller, abs(offset));
+		} else {
+			fprintf(stderr, "\n");
+		}
+		caller_addr = callinfo->stack[depth].caller_addr;
 	}
 }
 
@@ -457,10 +581,10 @@ int Profile_AllocCallinfo(callinfo_t *callinfo, int count, const char *name)
 		/* alloc & clear new data */
 		callinfo->site = calloc(count, sizeof(callee_t));
 		if (callinfo->site) {
-			printf("Allocated %s profile callsite buffer for %d symbols.\n", name, count);
+			fprintf(stderr, "Allocated %s profile callsite buffer for %d symbols.\n", name, count);
 			callinfo->prev_pc = callinfo->return_pc = PC_UNDEFINED;
 		} else {
-			fprintf(stderr, "ERROR: callesite buffer alloc failed!\n");
+			fprintf(stderr, "ERROR: callsite buffer alloc failed!\n");
 			callinfo->sites = 0;
 		}
 	}
@@ -498,11 +622,16 @@ void Profile_FreeCallinfo(callinfo_t *callinfo)
  */
 char *Profile_Match(const char *text, int state)
 {
-	static const char *names[] = {
+	static const char *subs[] = {
 		"addresses", "callers", "caches", "counts", "cycles", "d-hits", "i-misses",
 		"loops", "off", "on", "save", "stack", "stats", "symbols"
 	};
-	return DebugUI_MatchHelper(names, ARRAY_SIZE(names), text, state);
+
+	char *ret = DebugUI_MatchHelper(subs, ARRAY_SIZE(subs), text, state);
+	if (ret) {
+		return ret;
+	}
+	return rl_filename_completion_function(text, state);
 }
 
 const char Profile_Description[] =
@@ -559,8 +688,9 @@ const char Profile_Description[] =
 static bool Profile_Save(const char *fname, bool bForDsp)
 {
 	FILE *out;
-	Uint32 freq;
-	const char *proc, *core;
+	uint32_t freq;
+	const char *proc;
+
 	if (!(out = fopen(fname, "w"))) {
 		fprintf(stderr, "ERROR: opening '%s' for writing failed!\n", fname);
 		perror(NULL);
@@ -570,15 +700,11 @@ static bool Profile_Save(const char *fname, bool bForDsp)
 		freq = MachineClocks.DSP_Freq;
 		proc = "DSP";
 	} else {
-		freq = MachineClocks.CPU_Freq;
+		freq = MachineClocks.CPU_Freq_Emul;
 		proc = "CPU";
 	}
-#if ENABLE_WINUAE_CPU
-	core = "WinUAE";
-#else
-	core = "OldUAE";
-#endif
-	fprintf(out, "Hatari %s profile (%s, %s CPU core)\n", proc, PROG_NAME, core);
+
+	fprintf(out, "Hatari %s profile (%s)\n", proc, PROG_NAME);
 	fprintf(out, "Cycles/second:\t%u\n", freq);
 	if (bForDsp) {
 		Profile_DspSave(out);
@@ -643,6 +769,8 @@ static bool Profile_Loops(int nArgc, char *psArgs[])
 			profile_loop.filename = NULL;
 			fclose(profile_loop.fp);
 			profile_loop.fp = NULL;
+		} else {
+			fprintf(stderr, "ERROR: no file name for saving the loop profiling information.\n");
 		}
 	}
 	return true;
@@ -655,7 +783,7 @@ static bool Profile_Loops(int nArgc, char *psArgs[])
 int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 {
 	static int show = 16;
-	Uint32 *disasm_addr;
+	uint32_t *disasm_addr;
 	bool *enabled;
 
 	if (nArgc > 2) {
@@ -669,7 +797,7 @@ int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 
 	/* continue or explicit addresses command? */
 	if (nArgc < 2 || strcmp(psArgs[1], "addresses") == 0) {
-		Uint32 lower, upper = 0;
+		uint32_t lower, upper = 0;
 		if (nArgc > 2) {
 			if (Eval_Range(psArgs[2], &lower, &upper, false) < 0) {
 				return DEBUGGER_CMDDONE;
@@ -678,9 +806,9 @@ int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 			lower = *disasm_addr;
 		}
 		if (bForDsp) {
-			*disasm_addr = Profile_DspShowAddresses(lower, upper, stdout);
+			*disasm_addr = Profile_DspShowAddresses(lower, upper, stderr, PAGING_ENABLED);
 		} else {
-			*disasm_addr = Profile_CpuShowAddresses(lower, upper, stdout);
+			*disasm_addr = Profile_CpuShowAddresses(lower, upper, stderr, PAGING_ENABLED);
 		}
 		return DEBUGGER_CMDCONT;
 
@@ -691,7 +819,7 @@ int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 	} else if (strcmp(psArgs[1], "off") == 0) {
 		*enabled = false;
 		fprintf(stderr, "Profiling disabled.\n");
-	
+
 	} else if (strcmp(psArgs[1], "stats") == 0) {
 		if (bForDsp) {
 			Profile_DspShowStats();
@@ -736,9 +864,9 @@ int Profile_Command(int nArgc, char *psArgs[], bool bForDsp)
 		}
 	} else if (strcmp(psArgs[1], "callers") == 0) {
 		if (bForDsp) {
-			Profile_DspShowCallers(stdout);
+			Profile_DspShowCallers(stderr);
 		} else {
-			Profile_CpuShowCallers(stdout);
+			Profile_CpuShowCallers(stderr);
 		}
 	} else if (strcmp(psArgs[1], "stack") == 0) {
 		Profile_ShowStack(bForDsp);
