@@ -16,20 +16,19 @@
 	GNU General Public License for more details.
 
 	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software Foundation,
-	51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+	along with this program; if not, see <https://www.gnu.org/licenses/>.
 */
 
 #include <ctype.h>
 
 #include "main.h"
 #include "sysdeps.h"
-#include "newcpu.h"
 #include "memorySnapShot.h"
 #include "ioMem.h"
 #include "dsp.h"
 #include "crossbar.h"
 #include "configuration.h"
+#include "cycles.h"
 #include "cycInt.h"
 #include "m68000.h"
 
@@ -59,7 +58,7 @@ static const char* x_ext_memory_addr_name[] = {
 	"", "", "", "", "", "", "BCR", "IPR"
 };
 
-static Sint32 save_cycles;
+static int32_t save_cycles;
 #endif
 
 static bool bDspDebugging;
@@ -67,7 +66,7 @@ static bool bDspDebugging;
 bool bDspEnabled = false;
 bool bDspHostInterruptPending = false;
 
-Uint64	DSP_CyclesGlobalClockCounter = 0;			/* Value of CyclesGlobalClockCounter when DSP_Run was last called */
+uint64_t	DSP_CyclesGlobalClockCounter = 0;			/* Value of CyclesGlobalClockCounter when DSP_Run was last called */
 
 
 /**
@@ -77,6 +76,8 @@ Uint64	DSP_CyclesGlobalClockCounter = 0;			/* Value of CyclesGlobalClockCounter
 static void DSP_TriggerHostInterrupt(int hreq)
 {
 //fprintf ( stderr, "DSP_TriggerHostInterrupt %d %x %x\n" , hreq , regs.sr , regs.intmask );
+
+// TODO [NP] : we should change GPIP bit 3 in MFP instead of using additional SPCFLAG_DSP and DSP_GetHREQ
 	if ( hreq )
 	{
 		M68000_SetSpecial(SPCFLAG_DSP);			// TODO for old cpu core, remove, use level 6 instead and M68000_Update_intlev()
@@ -96,7 +97,7 @@ static void DSP_TriggerHostInterrupt(int hreq)
 /**
  * Return the state of HREQ
  */
-Uint8	DSP_GetHREQ ( void )
+uint8_t	DSP_GetHREQ ( void )
 {
 	if ( bDspHostInterruptPending )
 		return 1;
@@ -118,7 +119,7 @@ int	DSP_ProcessIACK ( void )
 		VecNr = IoMem_ReadByte ( 0xffa203 );
 	else
 		VecNr = -1;
-	
+
 	return VecNr;
 }
 
@@ -188,8 +189,11 @@ void DSP_Reset(void)
 void DSP_Enable(void)
 {
 #if ENABLE_DSP_EMU
-	bDspEnabled = true;
-	DSP_CyclesGlobalClockCounter = CyclesGlobalClockCounter;
+	if ( Config_IsMachineFalcon() )
+	{
+		bDspEnabled = true;
+		DSP_CyclesGlobalClockCounter = CyclesGlobalClockCounter;
+	}
 #endif
 }
 
@@ -217,7 +221,7 @@ void DSP_MemorySnapShot_Capture(bool bSave)
 
 	if ( bDspEnabled )
 		DSP_Enable();
-	else	
+	else
 		DSP_Disable();
 #endif
 }
@@ -261,7 +265,7 @@ void DSP_Run(int nHostCycles)
 	}
 
 #endif
-} 
+}
 
 /**
  * Enable/disable DSP debugging mode
@@ -274,7 +278,7 @@ void DSP_SetDebugging(bool enabled)
 /**
  * Get DSP program counter (for debugging)
  */
-Uint16 DSP_GetPC(void)
+uint16_t DSP_GetPC(void)
 {
 #if ENABLE_DSP_EMU
 	if (bDspEnabled)
@@ -287,12 +291,12 @@ Uint16 DSP_GetPC(void)
 /**
  * Get next DSP PC without output (for debugging)
  */
-Uint16 DSP_GetNextPC(Uint16 pc)
+uint16_t DSP_GetNextPC(uint16_t pc)
 {
 #if ENABLE_DSP_EMU
 	/* code is reduced copy from dsp56k_execute_one_disasm_instruction() */
 	dsp_core_t dsp_core_save;
-	Uint16 instruction_length;
+	uint16_t instruction_length;
 
 	if (!bDspEnabled)
 		return 0;
@@ -319,7 +323,7 @@ Uint16 DSP_GetNextPC(Uint16 pc)
 /**
  * Get current DSP instruction cycles (for profiling)
  */
-Uint16 DSP_GetInstrCycles(void)
+uint16_t DSP_GetInstrCycles(void)
 {
 #if ENABLE_DSP_EMU
 	if (bDspEnabled)
@@ -333,10 +337,10 @@ Uint16 DSP_GetInstrCycles(void)
 /**
  * Disassemble DSP code between given addresses, return next PC address
  */
-Uint16 DSP_DisasmAddress(FILE *out, Uint16 lowerAdr, Uint16 UpperAdr)
+uint16_t DSP_DisasmAddress(FILE *out, uint16_t lowerAdr, uint16_t UpperAdr)
 {
 #if ENABLE_DSP_EMU
-	Uint16 dsp_pc;
+	uint16_t dsp_pc;
 
 	for (dsp_pc=lowerAdr; dsp_pc<=UpperAdr; dsp_pc++) {
 		dsp_pc += dsp56k_execute_one_disasm_instruction(out, dsp_pc);
@@ -357,13 +361,13 @@ Uint16 DSP_DisasmAddress(FILE *out, Uint16 lowerAdr, Uint16 UpperAdr)
  * Return the value at given address. For valid values AND the return
  * value with BITMASK(24).
  */
-Uint32 DSP_ReadMemory(Uint16 address, char space_id, const char **mem_str)
+uint32_t DSP_ReadMemory(uint16_t address, char space_id, const char **mem_str)
 {
 #if ENABLE_DSP_EMU
 	static const char *spaces[3][4] = {
 		{ "X ram", "X rom", "X", "X periph" },
 		{ "Y ram", "Y rom", "Y", "Y periph" },
-		{ "P ram", "P ram", "P ext memory", "P ext memory" }
+		{ "P ram", "P rom", "P ext memory", "P ext memory" }
 	};
 	int idx, space;
 
@@ -386,12 +390,6 @@ Uint32 DSP_ReadMemory(Uint16 address, char space_id, const char **mem_str)
 	}
 	address &= 0xFFFF;
 
-	/* Internal RAM ? */
-	if (address < 0x100) {
-		*mem_str = spaces[idx][0];
-		return dsp_core.ramint[space][address];
-	}
-
 	if (space == DSP_SPACE_P) {
 		/* Internal RAM ? */
 		if (address < 0x200) {
@@ -403,6 +401,12 @@ Uint32 DSP_ReadMemory(Uint16 address, char space_id, const char **mem_str)
 		return dsp_core.ramext[address & (DSP_RAMSIZE-1)];
 	}
 
+	/* Internal RAM ? */
+	if (address < 0x100) {
+		*mem_str = spaces[idx][0];
+		return dsp_core.ramint[space][address];
+	}
+
 	/* Internal ROM ? */
 	if (address < 0x200) {
 		if (dsp_core.registers[DSP_REG_OMR] & (1<<DSP_OMR_DE)) {
@@ -438,10 +442,10 @@ Uint32 DSP_ReadMemory(Uint16 address, char space_id, const char **mem_str)
  * Output memory values between given addresses in given DSP address space.
  * Return next DSP address value.
  */
-Uint16 DSP_DisasmMemory(FILE *fp, Uint16 dsp_memdump_addr, Uint16 dsp_memdump_upper, char space)
+uint16_t DSP_DisasmMemory(FILE *fp, uint16_t dsp_memdump_addr, uint16_t dsp_memdump_upper, char space)
 {
 #if ENABLE_DSP_EMU
-	Uint32 mem, mem2, value;
+	uint32_t mem, mem2, value;
 	const char *mem_str;
 
 	for (mem = dsp_memdump_addr; mem <= dsp_memdump_upper; mem++) {
@@ -483,35 +487,41 @@ Uint16 DSP_DisasmMemory(FILE *fp, Uint16 dsp_memdump_addr, Uint16 dsp_memdump_up
  * Show information on DSP core state which isn't
  * shown by any of the other commands (dd, dm, dr).
  */
-void DSP_Info(FILE *fp, Uint32 dummy)
+void DSP_Info(FILE *fp, uint32_t dummy)
 {
 #if ENABLE_DSP_EMU
 	int i, j;
 	const char *stackname[] = { "SSH", "SSL" };
 
-	fputs("DSP core information:\n", fp);
+	fputs("\nDSP core information:\n", fp);
 
 	for (i = 0; i < ARRAY_SIZE(stackname); i++) {
-		fprintf(fp, "- %s stack:", stackname[i]);
+		fprintf(fp, "  %s stack:", stackname[i]);
 		for (j = 0; j < ARRAY_SIZE(dsp_core.stack[0]); j++) {
 			fprintf(fp, " %04hx", dsp_core.stack[i][j]);
 		}
 		fputs("\n", fp);
 	}
 
-	fprintf(fp, "- Interrupt IPL:");
-	for (i = 0; i < ARRAY_SIZE(dsp_core.interrupt_ipl); i++) {
-		fprintf(fp, " %04hx", dsp_core.interrupt_ipl[i]);
+	fprintf(stderr, "\nInterrupts:\n");
+	for (i = 0; i < 32; i++) {
+		fprintf(stderr, "  %s: ", dsp_interrupt_name[i]);
+		if ((1U << i) & dsp_core.interrupt_status & (dsp_core.interrupt_mask|DSP_INTER_NMI_MASK)) {
+			fprintf(stderr, "Pending ");
+		}
+		if ((1U << i) & DSP_INTER_NMI_MASK) {
+			fprintf(stderr, "at level 3");
+		} else {
+			for (j = 2; j>=0; j--) {
+				if ((1<<i) & dsp_core.interrupt_mask_level[j]) {
+					fprintf(stderr, "at level %i", j);
+				}
+			}
+		}
+		fputs("\n", stderr);
 	}
-	fputs("\n", fp);
 
-	fprintf(fp, "- Pending ints: ");
-	for (i = 0; i < ARRAY_SIZE(dsp_core.interrupt_isPending); i++) {
-		fprintf(fp, " %04hx", dsp_core.interrupt_isPending[i]);
-	}
-	fputs("\n", fp);
-
-	fprintf(fp, "- Hostport:");
+	fprintf(fp, "\nHostport:");
 	for (i = 0; i < ARRAY_SIZE(dsp_core.hostport); i++) {
 		fprintf(fp, " %02x", dsp_core.hostport[i]);
 	}
@@ -525,27 +535,46 @@ void DSP_Info(FILE *fp, Uint32 dummy)
 void DSP_DisasmRegisters(FILE *fp)
 {
 #if ENABLE_DSP_EMU
-	Uint32 i;
+	uint32_t i;
+	char stack_disasm[16][20];
 
-	fprintf(fp, "A: A2: %02x  A1: %06x  A0: %06x\n",
-		dsp_core.registers[DSP_REG_A2], dsp_core.registers[DSP_REG_A1], dsp_core.registers[DSP_REG_A0]);
-	fprintf(fp, "B: B2: %02x  B1: %06x  B0: %06x\n",
-		dsp_core.registers[DSP_REG_B2], dsp_core.registers[DSP_REG_B1], dsp_core.registers[DSP_REG_B0]);
-	
-	fprintf(fp, "X: X1: %06x  X0: %06x\n", dsp_core.registers[DSP_REG_X1], dsp_core.registers[DSP_REG_X0]);
-	fprintf(fp, "Y: Y1: %06x  Y0: %06x\n", dsp_core.registers[DSP_REG_Y1], dsp_core.registers[DSP_REG_Y0]);
-
-	for (i=0; i<8; i++) {
-		fprintf(fp, "R%01x: %04x   N%01x: %04x   M%01x: %04x\n",
-			i, dsp_core.registers[DSP_REG_R0+i],
-			i, dsp_core.registers[DSP_REG_N0+i],
-			i, dsp_core.registers[DSP_REG_M0+i]);
+	/* Prepare the stack disasm */
+	for (i=0; i<16; i++) {
+               if ((dsp_core.registers[DSP_REG_SP] & BITMASK(4)) == i)
+			sprintf(stack_disasm[i], " >%02d:  %04x   %04x\n", i, dsp_core.stack[0][i], dsp_core.stack[1][i]);
+		else
+			sprintf(stack_disasm[i], "  %02d:  %04x   %04x\n", i, dsp_core.stack[0][i], dsp_core.stack[1][i]);
 	}
 
-	fprintf(fp, "LA: %04x   LC: %04x   PC: %04x\n", dsp_core.registers[DSP_REG_LA], dsp_core.registers[DSP_REG_LC], dsp_core.pc);
-	fprintf(fp, "SR: %04x  OMR: %02x\n", dsp_core.registers[DSP_REG_SR], dsp_core.registers[DSP_REG_OMR]);
-	fprintf(fp, "SP: %02x    SSH: %04x  SSL: %04x\n",
-		dsp_core.registers[DSP_REG_SP], dsp_core.registers[DSP_REG_SSH], dsp_core.registers[DSP_REG_SSL]);
+	/* Display the DSP registers and stack state */
+	fprintf(fp, "\t\t\t\t|  SP   SSH    SSL\n");
+
+	fprintf(fp, "A2: %02x  A1: %06x  A0: %06x\t|%s",
+		dsp_core.registers[DSP_REG_A2], dsp_core.registers[DSP_REG_A1], dsp_core.registers[DSP_REG_A0], stack_disasm[0]);
+	fprintf(fp, "B2: %02x  B1: %06x  B0: %06x\t|%s",
+		dsp_core.registers[DSP_REG_B2], dsp_core.registers[DSP_REG_B1], dsp_core.registers[DSP_REG_B0], stack_disasm[1]);
+	fprintf(fp, "        X1: %06x  X0: %06x\t|%s",
+		dsp_core.registers[DSP_REG_X1], dsp_core.registers[DSP_REG_X0], stack_disasm[2]);
+	fprintf(fp, "        Y1: %06x  Y0: %06x\t|%s",
+		dsp_core.registers[DSP_REG_Y1], dsp_core.registers[DSP_REG_Y0], stack_disasm[3]);
+	fprintf(fp, "\t\t\t\t|%s", stack_disasm[4]);
+
+	for (i=0; i<8; i++) {
+		fprintf(fp, "R%01x: %04x   N%01x: %04x   M%01x: %04x\t|%s",
+			i, dsp_core.registers[DSP_REG_R0+i],
+			i, dsp_core.registers[DSP_REG_N0+i],
+			i, dsp_core.registers[DSP_REG_M0+i],
+			stack_disasm[i+5]);
+	}
+
+	fprintf(fp, "\t\t\t\t|%s", stack_disasm[13]);
+
+	fprintf(fp, "LA: %04x   LC: %04x   PC: %04x\t|%s",
+		dsp_core.registers[DSP_REG_LA], dsp_core.registers[DSP_REG_LC], dsp_core.pc, stack_disasm[14]);
+	fprintf(fp, "SR: %04x   OMR: %02x    SP: %02x\t|%s",
+		dsp_core.registers[DSP_REG_SR], dsp_core.registers[DSP_REG_OMR], dsp_core.registers[DSP_REG_SP], stack_disasm[15]);
+
+	fprintf(fp, "\n");
 #endif
 }
 
@@ -557,17 +586,17 @@ void DSP_DisasmRegisters(FILE *fp)
  * need special handling (in DSP*SetRegister()) when they are set.
  * Return the register width in bits or zero for an error.
  */
-int DSP_GetRegisterAddress(const char *regname, Uint32 **addr, Uint32 *mask)
+int DSP_GetRegisterAddress(const char *regname, uint32_t **addr, uint32_t *mask)
 {
 #if ENABLE_DSP_EMU
 #define MAX_REGNAME_LEN 4
 	typedef struct {
 		const char name[MAX_REGNAME_LEN];
-		Uint32 *addr;
+		uint32_t *addr;
 		size_t bits;
-		Uint32 mask;
+		uint32_t mask;
 	} reg_addr_t;
-	
+
 	/* sorted by name so that this can be bisected */
 	static const reg_addr_t registers[] = {
 
@@ -608,7 +637,7 @@ int DSP_GetRegisterAddress(const char *regname, Uint32 **addr, Uint32 *mask)
 		{ "OMR", &dsp_core.registers[DSP_REG_OMR], 32, 0x5f },
 
 		/* 16-bit program counter */
-		{ "PC",  (Uint32*)(&dsp_core.pc),  16, BITMASK(16) },
+		{ "PC",  (uint32_t*)(&dsp_core.pc),  16, BITMASK(16) },
 
 		/* 16-bit DSP R (address) registers */
 		{ "R0",  &dsp_core.registers[DSP_REG_R0],  32, BITMASK(16) },
@@ -620,13 +649,14 @@ int DSP_GetRegisterAddress(const char *regname, Uint32 **addr, Uint32 *mask)
 		{ "R6",  &dsp_core.registers[DSP_REG_R6],  32, BITMASK(16) },
 		{ "R7",  &dsp_core.registers[DSP_REG_R7],  32, BITMASK(16) },
 
-		{ "SSH", &dsp_core.registers[DSP_REG_SSH], 32, BITMASK(16) },
-		{ "SSL", &dsp_core.registers[DSP_REG_SSL], 32, BITMASK(16) },
 		{ "SP",  &dsp_core.registers[DSP_REG_SP],  32, BITMASK(6) },
 
 		/* 16-bit status register */
 		{ "SR",  &dsp_core.registers[DSP_REG_SR],  32, 0xefff },
 
+		{ "SSH", &dsp_core.registers[DSP_REG_SSH], 32, BITMASK(16) },
+		{ "SSL", &dsp_core.registers[DSP_REG_SSL], 32, BITMASK(16) },
+
 		/* 48-bit X register */
 		{ "X0",  &dsp_core.registers[DSP_REG_X0],  32, BITMASK(24) },
 		{ "X1",  &dsp_core.registers[DSP_REG_X1],  32, BITMASK(24) },
@@ -652,7 +682,7 @@ int DSP_GetRegisterAddress(const char *regname, Uint32 **addr, Uint32 *mask)
 		return 0;
 	}
 	len = i;
-	
+
 	/* bisect */
 	l = 0;
 	r = ARRAY_SIZE(registers) - 1;
@@ -684,17 +714,17 @@ int DSP_GetRegisterAddress(const char *regname, Uint32 **addr, Uint32 *mask)
 /**
  * Set given DSP register value, return false if unknown register given
  */
-bool DSP_Disasm_SetRegister(const char *arg, Uint32 value)
+bool DSP_Disasm_SetRegister(const char *arg, uint32_t value)
 {
 #if ENABLE_DSP_EMU
-	Uint32 *addr, mask, sp_value;
+	uint32_t *addr, mask, sp_value;
 	int bits;
 
 	/* first check registers needing special handling... */
 	if (arg[0]=='S' || arg[0]=='s') {
 		if (arg[1]=='P' || arg[1]=='p') {
 			dsp_core.registers[DSP_REG_SP] = value & BITMASK(6);
-			value &= BITMASK(4); 
+			value &= BITMASK(4);
 			dsp_core.registers[DSP_REG_SSH] = dsp_core.stack[0][value];
 			dsp_core.registers[DSP_REG_SSL] = dsp_core.stack[1][value];
 			return true;
@@ -731,7 +761,7 @@ bool DSP_Disasm_SetRegister(const char *arg, Uint32 value)
 		*addr = value & mask;
 		return true;
 	case 16:
-		*(Uint16*)addr = value & mask;
+		*(uint16_t*)addr = value & mask;
 		return true;
 	}
 #endif
@@ -741,7 +771,7 @@ bool DSP_Disasm_SetRegister(const char *arg, Uint32 value)
 /**
  * Read SSI transmit value
  */
-Uint32 DSP_SsiReadTxValue(void)
+uint32_t DSP_SsiReadTxValue(void)
 {
 #if ENABLE_DSP_EMU
 	return dsp_core.ssi.transmit_value;
@@ -753,7 +783,7 @@ Uint32 DSP_SsiReadTxValue(void)
 /**
  * Write SSI receive value
  */
-void DSP_SsiWriteRxValue(Uint32 value)
+void DSP_SsiWriteRxValue(uint32_t value)
 {
 #if ENABLE_DSP_EMU
 	dsp_core.ssi.received_value = value & 0xffffff;
@@ -777,7 +807,7 @@ void DSP_SsiTransmit_SC0(void)
 #endif
 }
 
-void DSP_SsiReceive_SC1(Uint32 FrameCounter)
+void DSP_SsiReceive_SC1(uint32_t FrameCounter)
 {
 #if ENABLE_DSP_EMU
 	dsp_core_ssi_Receive_SC1(FrameCounter);
@@ -791,14 +821,14 @@ void DSP_SsiTransmit_SC1(void)
 #endif
 }
 
-void DSP_SsiReceive_SC2(Uint32 FrameCounter)
+void DSP_SsiReceive_SC2(uint32_t FrameCounter)
 {
 #if ENABLE_DSP_EMU
 	dsp_core_ssi_Receive_SC2(FrameCounter);
 #endif
 }
 
-void DSP_SsiTransmit_SC2(Uint32 frame)
+void DSP_SsiTransmit_SC2(uint32_t frame)
 {
 #if ENABLE_DSP_EMU
 	Crossbar_DmaRecordInHandShakeMode_Frame(frame);
@@ -825,10 +855,10 @@ void DSP_SsiTransmit_SCK(void)
  */
 void DSP_HandleReadAccess(void)
 {
-	Uint32 addr;
-	Uint8 value;
-	bool multi_access = false; 
-	
+	uint32_t addr;
+	uint8_t value;
+	bool multi_access = false;
+
 	for (addr = IoAccessBaseAddress; addr < IoAccessBaseAddress+nIoMemAccessSize; addr++)
 	{
 #if ENABLE_DSP_EMU
@@ -838,7 +868,7 @@ void DSP_HandleReadAccess(void)
 		value = 0xff;
 #endif
 		if (multi_access == true)
-			M68000_AddCycles(4);
+			M68000_WaitState(4);
 		multi_access = true;
 
 		Dprintf(("HWget_b(0x%08x)=0x%02x at 0x%08x\n", addr, value, m68k_getpc()));
@@ -853,18 +883,18 @@ void DSP_HandleReadAccess(void)
  */
 void DSP_HandleWriteAccess(void)
 {
-	Uint32 addr;
-	bool multi_access = false; 
+	uint32_t addr;
+	bool multi_access = false;
 
 	for (addr = IoAccessBaseAddress; addr < IoAccessBaseAddress+nIoMemAccessSize; addr++)
 	{
 #if ENABLE_DSP_EMU
-		Uint8 value = IoMem_ReadByte(addr);
+		uint8_t value = IoMem_ReadByte(addr);
 		Dprintf(("HWput_b(0x%08x,0x%02x) at 0x%08x\n", addr, value, m68k_getpc()));
 		dsp_core_write_host(addr-DSP_HW_OFFSET, value);
 #endif
 		if (multi_access == true)
-			M68000_AddCycles(4);
+			M68000_WaitState(4);
 		multi_access = true;
 	}
 }
