@@ -15,8 +15,7 @@
 	GNU General Public License for more details.
 
 	You should have received a copy of the GNU General Public License
-	along with this program; if not, write to the Free Software
-	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	along with this program; if not, see <https://www.gnu.org/licenses/>.
 */
 
 #ifndef DSP_CPU_H
@@ -113,10 +112,11 @@ extern "C" {
 /* Functions */
 extern void dsp56k_init_cpu(void);		/* Set dsp_core to use */
 extern void dsp56k_execute_instruction(void);	/* Execute 1 instruction */
-extern Uint16 dsp56k_execute_one_disasm_instruction(FILE *out, Uint16 pc);	/* Execute 1 instruction in disasm mode */
+extern uint16_t dsp56k_execute_one_disasm_instruction(FILE *out, uint16_t pc);	/* Execute 1 instruction in disasm mode */
 
 /* Interrupt relative functions */
-void dsp_add_interrupt(Uint16 inter);
+void dsp_set_interrupt(uint32_t intr, uint32_t set);
+void dsp_set_interrupt_mask(uint32_t intr, uint32_t set);
 
 #ifdef __cplusplus
 }
