@@ -166,14 +166,14 @@ char szPath[FILENAME_MAX] ;											// for general use
 /*----------------------------------------------------------------------*/
 - (IBAction)warmReset:(id)sender
 {
-	if ([NSApp myAlerte:NSInformationalAlertStyle Txt:localize(@"Warm reset!") firstB:localize(@"OK") alternateB:localize(@"Cancel")
+	if ([NSApp myAlerte:NSAlertStyleInformational Txt:localize(@"Warm reset!") firstB:localize(@"OK") alternateB:localize(@"Cancel")
 			otherB:nil informativeTxt:localize(@"Really reset the emulator?")] == NSAlertFirstButtonReturn )
 		Reset_Warm();
 } 
 /*----------------------------------------------------------------------*/
 - (IBAction)coldReset:(id)sender
 {
-	if ([NSApp myAlerte:NSInformationalAlertStyle Txt:localize(@"Cold reset") firstB:localize(@"OK") alternateB:localize(@"Cancel")
+	if ([NSApp myAlerte:NSAlertStyleInformational Txt:localize(@"Cold reset") firstB:localize(@"OK") alternateB:localize(@"Cancel")
 			otherB:nil informativeTxt:localize(@"Really reset the emulator?")] == NSAlertFirstButtonReturn )
 		Reset_Cold();
 }
@@ -192,7 +192,7 @@ char szPath[FILENAME_MAX] ;											// for general use
 {
 	NSString	*aDisk ;
 
-	aDisk = [NSApp hopenfile:NO defoDir:nil defoFile:@"" types:@[allF]] ;
+	aDisk = [NSApp hopenfile:NO defoDir:nil defoFile:@""] ;
 	if (aDisk.length == 0) return ;                 		// user canceled
 
 	[aDisk getCString:szPath maxLength:FILENAME_MAX-1 encoding:NSASCIIStringEncoding] ;
@@ -286,7 +286,7 @@ char szPath[FILENAME_MAX] ;											// for general use
 			Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
 					ConfigureParams.Video.AviRecordFps == 0 ?
 					ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
-					(Uint32)ConfigureParams.Video.AviRecordFps << CLOCKS_TIMINGS_SHIFT_VBL ,
+					ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , ConfigureParams.Video.AviRecordFps ) ,
 				1 << CLOCKS_TIMINGS_SHIFT_VBL ,
 				ConfigureParams.Video.AviRecordVcodec );
 		}
@@ -355,7 +355,7 @@ char szPath[FILENAME_MAX] ;											// for general use
 	 {	directoryToOpen = @"~".stringByExpandingTildeInPath ;			// Currently no path: we use user's directory
 		fileToPreselect = nil; } ;
 
-	newPath = [NSApp hopenfile:NO defoDir:directoryToOpen defoFile:fileToPreselect types:@[@"sav"] ] ;
+	newPath = [NSApp hopenfile:NO defoDir:directoryToOpen defoFile:fileToPreselect] ;
 	if (newPath.length != 0)											// Perform the memory snapshot load
 		MemorySnapShot_Restore([newPath cStringUsingEncoding:NSASCIIStringEncoding], TRUE);
 
@@ -426,9 +426,9 @@ char szPath[FILENAME_MAX] ;											// for general use
 	// commit back to the configuration settings if choosing user confirm)
 	CurrentParams = ConfigureParams;
 
-	GuiOsx_Pause(true);
+	bool bWasRunning = GuiOsx_Pause(true);
 
-	newCfg = [NSApp hopenfile:NO defoDir:nil defoFile:ConfigFile types:@[@"cfg"] ] ;
+	newCfg = [NSApp hopenfile:NO defoDir:nil defoFile:ConfigFile] ;
 
 	if (newCfg.length != 0)
 	{
@@ -438,15 +438,17 @@ char szPath[FILENAME_MAX] ;											// for general use
 
 		// Refresh all the controls to match ConfigureParams
 		if (Change_DoNeedReset(&CurrentParams, &ConfigureParams))
-		 	applyChanges = [NSApp myAlerte:NSInformationalAlertStyle Txt:localize(@"Reset the emulator") firstB:localize(@"Don't reset")
-				alternateB:localize(@"Reset") otherB:nil informativeTxt:@"" ] == NSAlertFirstButtonReturn ;
+			applyChanges = [NSApp myAlerte:NSAlertStyleInformational Txt:localize(@"Reset the emulator") firstB:localize(@"Don't reset")
+				alternateB:localize(@"Reset") otherB:nil informativeTxt:@"" ] == NSAlertSecondButtonReturn ;
 		if (applyChanges)
 			Change_CopyChangedParamsToConfiguration(&CurrentParams, &ConfigureParams, true); 	// Ok with Reset
 		else
 			ConfigureParams = CurrentParams;   //Restore previous Params.
 	} ;
 
-	GuiOsx_Resume();
+	if (bWasRunning) {
+		GuiOsx_Resume();
+	}
 }
 
 /*----------------------------------------------------------------------*/
@@ -455,20 +457,19 @@ char szPath[FILENAME_MAX] ;											// for general use
 
 @end
 /*----------------------------------------------------------------------*/
-static int IsRootCwd()
+static int IsRootCwd(void)
 {
 	char buf[MAXPATHLEN];
 	char *cwd = getcwd(buf, sizeof (buf));
 	return (cwd && (strcmp(cwd, "/") == 0));
 }
 /*----------------------------------------------------------------------*/
-static int IsTenPointNineOrLater()
+static int IsTenPointNineOrLater(void)
 {
 	// OK for 10.9, but before ??
-	NSOperatingSystemVersion systemVersion = [[NSProcessInfo processInfo] operatingSystemVersion];	
-	int r=(systemVersion.majorVersion==10) && (systemVersion.minorVersion>=9);
-
-	return r;
+	NSOperatingSystemVersion systemVersion = [[NSProcessInfo processInfo] operatingSystemVersion];
+	return (systemVersion.majorVersion == 10 && systemVersion.minorVersion >= 9)
+			|| systemVersion.majorVersion > 10;
 }
 /*----------------------------------------------------------------------*/
 static int IsFinderLaunch(const int argc, char **argv)
