@@ -6,7 +6,7 @@
 
   A tiny graphical user interface for Hatari.
 */
-const char SDLGui_fileid[] = "Hatari sdlgui.c : " __DATE__ " " __TIME__;
+const char SDLGui_fileid[] = "Hatari sdlgui.c";
 
 #include <SDL.h>
 #include <ctype.h>
@@ -18,6 +18,7 @@ const char SDLGui_fileid[] = "Hatari sdlgui.c : " __DATE__ " " __TIME__;
 #include "screen.h"
 #include "sdlgui.h"
 #include "str.h"
+#include "log.h"
 
 #include "font5x8.h"
 #include "font10x16.h"
@@ -29,16 +30,10 @@ const char SDLGui_fileid[] = "Hatari sdlgui.c : " __DATE__ " " __TIME__;
 # define Dprintf(a)
 #endif
 
-#if WITH_SDL2
-#define SDL_SRCCOLORKEY SDL_TRUE
-#define SDLKey SDL_Keycode
-#endif
-
 static SDL_Surface *pSdlGuiScrn;            /* Pointer to the actual main SDL screen surface */
 static SDL_Surface *pSmallFontGfx = NULL;   /* The small font graphics */
 static SDL_Surface *pBigFontGfx = NULL;     /* The big font graphics */
 static SDL_Surface *pFontGfx = NULL;        /* The actual font graphics */
-static int current_object = SDLGUI_NOTFOUND;/* Current selected object */
 
 static struct {
 	Uint32 darkbar, midbar, lightbar;
@@ -70,7 +65,7 @@ static SDL_Surface *SDLGui_LoadXBM(int w, int h, const Uint8 *pXbmBits)
 	bitmap = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 8, 0, 0, 0, 0);
 	if (bitmap == NULL)
 	{
-		fprintf(stderr, "Failed to allocate bitmap: %s", SDL_GetError());
+		Log_Printf(LOG_ERROR, "SDLGui: failed to allocate bitmap: %s", SDL_GetError());
 		return NULL;
 	}
 
@@ -115,17 +110,17 @@ int SDLGui_Init(void)
 	pBigFontGfx = SDLGui_LoadXBM(font10x16_width, font10x16_height, font10x16_bits);
 	if (pSmallFontGfx == NULL || pBigFontGfx == NULL)
 	{
-		fprintf(stderr, "Error: Can not init font graphics!\n");
+		Log_Printf(LOG_ERROR, "SDLGui: cannot init font graphics!\n");
 		return -1;
 	}
 
 	/* Set color palette of the font graphics: */
-	SDL_SetColors(pSmallFontGfx, blackWhiteColors, 0, 2);
-	SDL_SetColors(pBigFontGfx, blackWhiteColors, 0, 2);
+	SDL_SetPaletteColors(pSmallFontGfx->format->palette, blackWhiteColors, 0, 2);
+	SDL_SetPaletteColors(pBigFontGfx->format->palette, blackWhiteColors, 0, 2);
 
 	/* Set font color 0 as transparent: */
-	SDL_SetColorKey(pSmallFontGfx, (SDL_SRCCOLORKEY|SDL_RLEACCEL), 0);
-	SDL_SetColorKey(pBigFontGfx, (SDL_SRCCOLORKEY|SDL_RLEACCEL), 0);
+	SDL_SetColorKey(pSmallFontGfx, SDL_RLEACCEL, 0);
+	SDL_SetColorKey(pBigFontGfx, SDL_RLEACCEL, 0);
 
 	return 0;
 }
@@ -174,7 +169,7 @@ int SDLGui_SetScreen(SDL_Surface *pScrn)
 
 	if (pFontGfx == NULL)
 	{
-		fprintf(stderr, "Error: A problem with the font occurred!\n");
+		Log_Printf(LOG_ERROR, "SDLGui: a problem with the font occurred!\n");
 		return -1;
 	}
 
@@ -196,7 +191,7 @@ int SDLGui_SetScreen(SDL_Surface *pScrn)
 	if (sdlgui_fontheight < 16)
 		colors.underline = SDL_MapRGB(pSdlGuiScrn->format,255,0,255);
 	else
-		colors.underline = SDL_MapRGB(pSdlGuiScrn->format,0,0,0);		
+		colors.underline = SDL_MapRGB(pSdlGuiScrn->format,0,0,0);
 	colors.editfield = SDL_MapRGB(pSdlGuiScrn->format,160,160,160);
 
 	return 0;
@@ -281,7 +276,7 @@ static void SDLGui_TextInt(int x, int y, const char *txt, bool underline)
 		}
 		else if (c >= 0x80)
 		{
-			printf("Unsupported character '%c' (0x%x)\n", c, c);
+			Log_Printf(LOG_WARN, "Unsupported character '%c' (0x%x)\n", c, c);
 		}
 #endif
 		x += sdlgui_fontwidth;
@@ -495,7 +490,7 @@ static void SDLGui_DrawRadioButton(const SGOBJ *rdlg, int objnum)
 	else
 		str[0]=SGRADIOBUTTON_NORMAL;
 	str[1]=' ';
-	strcpy(&str[2], rdlg[objnum].txt);
+	Str_Copy(&str[2], rdlg[objnum].txt, sizeof(str) - 2);
 
 	SDLGui_TextInt(x, y, str, true);
 }
@@ -519,7 +514,7 @@ static void SDLGui_DrawCheckBox(const SGOBJ *cdlg, int objnum)
 	else
 		str[0]=SGCHECKBOX_NORMAL;
 	str[1]=' ';
-	strcpy(&str[2], cdlg[objnum].txt);
+	Str_Copy(&str[2], cdlg[objnum].txt, sizeof(str) - 2);
 
 	SDLGui_TextInt(x, y, str, true);
 }
@@ -539,7 +534,7 @@ static void SDLGui_DrawScrollbar(const SGOBJ *bdlg, int objnum)
 
 	x += bdlg[0].x*sdlgui_fontwidth;   /* add mainbox absolute coordinates */
 	y += bdlg[0].y*sdlgui_fontheight;  /* add mainbox absolute coordinates */
-	
+
 	w = 1 * sdlgui_fontwidth;
 	h = bdlg[objnum].w;
 
@@ -599,22 +594,14 @@ static void SDLGui_EditField(SGOBJ *dlg, int objnum)
 	char *txt;                          /* Shortcut for dlg[objnum].txt */
 	SDL_Rect rect;
 	SDL_Event event;
-#if !WITH_SDL2
-	int nOldUnicodeMode;
-#endif
 
 	rect.x = (dlg[0].x + dlg[objnum].x) * sdlgui_fontwidth;
 	rect.y = (dlg[0].y + dlg[objnum].y) * sdlgui_fontheight;
 	rect.w = (dlg[objnum].w + 1) * sdlgui_fontwidth - 1;
 	rect.h = dlg[objnum].h * sdlgui_fontheight;
 
-#if WITH_SDL2
 	SDL_SetTextInputRect(&rect);
 	SDL_StartTextInput();
-#else
-	/* Enable unicode translation to get shifted etc chars with SDL_PollEvent */
-	nOldUnicodeMode = SDL_EnableUNICODE(true);
-#endif
 
 	txt = dlg[objnum].txt;
 	cursorPos = strlen(txt);
@@ -642,7 +629,6 @@ static void SDLGui_EditField(SGOBJ *dlg, int objnum)
 				 case SDL_MOUSEBUTTONDOWN:          /* Mouse pressed -> stop editing */
 					bStopEditing = true;
 					break;
-#if WITH_SDL2
 				 case SDL_TEXTINPUT:
 					if (strlen(txt) < (size_t)dlg[objnum].w)
 					{
@@ -652,7 +638,6 @@ static void SDLGui_EditField(SGOBJ *dlg, int objnum)
 						cursorPos += 1;
 					}
 					break;
-#endif
 				 case SDL_KEYDOWN:                  /* Key pressed */
 					switch (event.key.keysym.sym)
 					{
@@ -680,19 +665,6 @@ static void SDLGui_EditField(SGOBJ *dlg, int objnum)
 							memmove(&txt[cursorPos], &txt[cursorPos+1], strlen(&txt[cursorPos+1])+1);
 						break;
 					 default:
-#if !WITH_SDL2
-						/* If it is a "good" key then insert it into the text field */
-						if (event.key.keysym.unicode >= 32 && event.key.keysym.unicode < 128
-						        && event.key.keysym.unicode != PATHSEP)
-						{
-							if (strlen(txt) < (size_t)dlg[objnum].w)
-							{
-								memmove(&txt[cursorPos+1], &txt[cursorPos], strlen(&txt[cursorPos])+1);
-								txt[cursorPos] = event.key.keysym.unicode;
-								cursorPos += 1;
-							}
-						}
-#endif
 						break;
 					}
 					break;
@@ -716,15 +688,11 @@ static void SDLGui_EditField(SGOBJ *dlg, int objnum)
 			SDL_FillRect(pSdlGuiScrn, &cursorrect, colors.cursor);
 		}
 		SDLGui_Text(rect.x, rect.y, dlg[objnum].txt);  /* Draw text */
-		SDL_UpdateRects(pSdlGuiScrn, 1, &rect);
+		Screen_UpdateRects(pSdlGuiScrn, 1, &rect);
 	}
 	while (!bStopEditing);
 
-#if WITH_SDL2
 	SDL_StopTextInput();
-#else
-	SDL_EnableUNICODE(nOldUnicodeMode);
-#endif
 }
 
 
@@ -775,7 +743,7 @@ void SDLGui_DrawDialog(const SGOBJ *dlg)
 	{
 		SDLGui_DrawObj(dlg, i);
 	}
-	SDL_UpdateRect(pSdlGuiScrn, 0,0,0,0);
+	Screen_UpdateRect(pSdlGuiScrn, 0,0,0,0);
 }
 
 
@@ -870,9 +838,9 @@ static void SDLGui_DebugPrintDialog(const SGOBJ *dlg)
 {
 #if DEBUG_INFO
 	int i;
-	printf("obj: flags | state\n");
+	printf("obj: flags | state\n");
 	for (i = 0; dlg[i].type != SGSTOP; i++)
-		printf("%3d:  0x%02x | 0x%02x\n", i, dlg[i].flags, dlg[i].state);
+		printf("%3d:  0x%02x | 0x%02x\n", i, dlg[i].flags, dlg[i].state);
 #endif
 }
 
@@ -972,7 +940,7 @@ static int SDLGui_FocusNext(SGOBJ *dlg, int i, int inc)
 		{
 			dlg[i].state |= SG_FOCUSED;
 			SDLGui_DrawObj(dlg, i);
-			SDL_UpdateRect(pSdlGuiScrn, 0,0,0,0);
+			Screen_UpdateRect(pSdlGuiScrn, 0,0,0,0);
 			return i;
 		}
 		/* wrapped around without even initial one matching */
@@ -1018,7 +986,7 @@ static int SDLGui_HandleSelection(SGOBJ *dlg, int obj, int oldbutton)
 			rct.h = sdlgui_fontheight;
 			SDL_FillRect(pSdlGuiScrn, &rct, colors.midgrey); /* Clear old */
 			SDLGui_DrawRadioButton(dlg, i);
-			SDL_UpdateRects(pSdlGuiScrn, 1, &rct);
+			Screen_UpdateRects(pSdlGuiScrn, 1, &rct);
 		}
 		for (i = obj+1; dlg[i].type == SGRADIOBUT; i++)
 		{
@@ -1029,7 +997,7 @@ static int SDLGui_HandleSelection(SGOBJ *dlg, int obj, int oldbutton)
 			rct.h = sdlgui_fontheight;
 			SDL_FillRect(pSdlGuiScrn, &rct, colors.midgrey); /* Clear old */
 			SDLGui_DrawRadioButton(dlg, i);
-			SDL_UpdateRects(pSdlGuiScrn, 1, &rct);
+			Screen_UpdateRects(pSdlGuiScrn, 1, &rct);
 		}
 		dlg[obj].state |= SG_SELECTED;  /* Select this radio button */
 		rct.x = (dlg[0].x+dlg[obj].x)*sdlgui_fontwidth;
@@ -1038,7 +1006,7 @@ static int SDLGui_HandleSelection(SGOBJ *dlg, int obj, int oldbutton)
 		rct.h = sdlgui_fontheight;
 		SDL_FillRect(pSdlGuiScrn, &rct, colors.midgrey); /* Clear old */
 		SDLGui_DrawRadioButton(dlg, obj);
-		SDL_UpdateRects(pSdlGuiScrn, 1, &rct);
+		Screen_UpdateRects(pSdlGuiScrn, 1, &rct);
 		break;
 	case SGCHECKBOX:
 		dlg[obj].state ^= SG_SELECTED;
@@ -1048,12 +1016,12 @@ static int SDLGui_HandleSelection(SGOBJ *dlg, int obj, int oldbutton)
 		rct.h = sdlgui_fontheight;
 		SDL_FillRect(pSdlGuiScrn, &rct, colors.midgrey); /* Clear old */
 		SDLGui_DrawCheckBox(dlg, obj);
-		SDL_UpdateRects(pSdlGuiScrn, 1, &rct);
+		Screen_UpdateRects(pSdlGuiScrn, 1, &rct);
 		break;
 	case SGPOPUP:
 		dlg[obj].state |= SG_SELECTED;
 		SDLGui_DrawPopupButton(dlg, obj);
-		SDL_UpdateRect(pSdlGuiScrn,
+		Screen_UpdateRect(pSdlGuiScrn,
 			       (dlg[0].x+dlg[obj].x)*sdlgui_fontwidth-2,
 			       (dlg[0].y+dlg[obj].y)*sdlgui_fontheight-2,
 			       dlg[obj].w*sdlgui_fontwidth+4,
@@ -1089,57 +1057,81 @@ static int SDLGui_HandleShortcut(SGOBJ *dlg, int key)
 }
 
 /**
- * Scale mouse coordinates in case we've got a re-sized SDL2 window
+ * Scale mouse state coordinates in case we've got a re-sized SDL2 window
+ *
+ * NOTE: while scaling done here fixes SDL2 reported mouse coords to
+ * match Hatari framebuffer coords in scaled SDL2 windows, there's
+ * another issue with (mouse _state_) coords in _fullscreen_.
+ *
+ * SDL2 deducts fullscreen letterboxing borders from those coords,
+ * but not from the values returns by SDL2 window size functions
+ * (and there's no function providing the letterbox border size).
+ *
+ * Atari resolutions are more narrow than today's widescreen monitor
+ * resolutions, so typically fullscreen letterboxing borders are on
+ * the sides => y-coord gets scaled OK, x-coord will be too small.
+ */
+void SDLGui_ScaleMouseStateCoordinates(int *x, int *y)
+{
+	int win_width, win_height;
+	SDL_GetWindowSize(sdlWindow, &win_width, &win_height);
+
+	*x = *x * pSdlGuiScrn->w / win_width;
+	*y = *y * pSdlGuiScrn->h / win_height;
+}
+
+/**
+ * Scale mouse event coordinates in case we've got a re-sized SDL2 window
  */
 static void SDLGui_ScaleMouseButtonCoordinates(SDL_MouseButtonEvent *bev)
 {
-#if WITH_SDL2
-	int win_width, win_height;
-
 	if (bInFullScreen)
 		return;
 
-	SDL_GetWindowSize(sdlWindow, &win_width, &win_height);
-	bev->x = bev->x * pSdlGuiScrn->w / win_width;
-	bev->y = bev->y * pSdlGuiScrn->h / win_height;
-#endif
+	int x = bev->x, y = bev->y;
+	SDLGui_ScaleMouseStateCoordinates(&x, &y);
+	bev->x = x; bev->y = y;
 }
 
 /*-----------------------------------------------------------------------*/
 /**
- * Show and process a dialog. Returns either:
+ * Show and process a dialog.
+ *
+ * Dialogs using a scrollbar, or other objects with SG_REPEAT flag,
+ * must return the previous return value in 'current_object' arg, as
+ * the same dialog is displayed in a loop to handle scrolling. Other
+ * dialogs should give zero as 'current_object' (ie no object
+ * selected at start when displaying the dialog)
+ *
+ * Returns either:
  * - index of the GUI item that was invoked
- * - SDLGUI_UNKNOWNEVENT if an unsupported event occurred
- *   (will be stored in parameter pEventOut)
  * - SDLGUI_QUIT if user wants to close Hatari
  * - SDLGUI_ERROR if unable to show dialog
- * GUI item indeces are positive, other return values are negative
+ * - for events not handled here, 'isEventOut' callback is checked
+ *   for whether caller is interested about given event type:
+ *   => event is stored to pEventOut and SDLGUI_UNKNOWNEVENT returned
+ * GUI item indices are positive, other return values are negative
  */
-int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
+int SDLGui_DoDialogExt(SGOBJ *dlg, bool (*isEventOut)(SDL_EventType), SDL_Event *pEventOut, int current_object)
 {
 	int oldbutton = SDLGUI_NOTFOUND;
 	int retbutton = SDLGUI_NOTFOUND;
-	int i, j, b, value, obj;
-	SDLKey key;
+	int b, x, y, value, obj;
+	SDL_Keycode key;
 	int focused;
 	SDL_Event sdlEvent;
 	SDL_Surface *pBgSurface;
 	SDL_Rect dlgrect, bgrect;
 	SDL_Joystick *joy = NULL;
-#if !WITH_SDL2
-	int nOldUnicodeMode;
-#endif
+	const Uint8 *keystates;
+	bool ignore_first_keyup;
 
-	/* In the case of dialog using a scrollbar, we must keep the previous */
-	/* value of current_object, as the same dialog is displayed in a loop */
-	/* to handle scrolling. For other dialogs, we need to reset current_object */
-	/* (ie no object selected at start when displaying the dialog) */
-	if ( !KeepCurrentObject )
-		current_object = 0;
+	/* either both, or neither of these should be present */
+	assert((isEventOut && pEventOut) || (!isEventOut && !pEventOut));
 
 	if (pSdlGuiScrn->h / sdlgui_fontheight < dlg[0].h)
 	{
-		fprintf(stderr, "Screen size too small for dialog!\n");
+		Log_Printf(LOG_ERROR, "Screen size too small for dialog!\n");
 		return SDLGUI_ERROR;
 	}
 
@@ -1155,18 +1147,18 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 	/* Save background */
 	pBgSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, dlgrect.w, dlgrect.h, pSdlGuiScrn->format->BitsPerPixel,
 	                                  pSdlGuiScrn->format->Rmask, pSdlGuiScrn->format->Gmask, pSdlGuiScrn->format->Bmask, pSdlGuiScrn->format->Amask);
-	if (pSdlGuiScrn->format->palette != NULL)
-	{
-		SDL_SetColors(pBgSurface, pSdlGuiScrn->format->palette->colors, 0, pSdlGuiScrn->format->palette->ncolors-1);
-	}
 
 	if (pBgSurface != NULL)
 	{
+		if (pSdlGuiScrn->format->palette != NULL)
+		{
+			SDL_SetPaletteColors(pBgSurface->format->palette, pSdlGuiScrn->format->palette->colors, 0, pSdlGuiScrn->format->palette->ncolors-1);
+		}
 		SDL_BlitSurface(pSdlGuiScrn,  &dlgrect, pBgSurface, &bgrect);
 	}
 	else
 	{
-		fprintf(stderr, "SDLGUI_DoDialog: CreateRGBSurface failed: %s\n", SDL_GetError());
+		Log_Printf(LOG_ERROR, "SDLGUI_DoDialog: CreateRGBSurface failed: %s\n", SDL_GetError());
 	}
 	SDLGui_DebugPrintDialog(dlg);
 
@@ -1187,36 +1179,52 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 	/* (Re-)draw the dialog */
 	SDLGui_DrawDialog(dlg);
 
+	/* If one of the keys that could exit the dialog is already held
+	 * before we start, then ignore the first keyup event since the
+	 * key press does not belong to the dialog, but rather to whatever
+	 * happened before the dialog.
+	 *
+	 * Cannot be used when asked to return already on key down
+	 * (e.g. with file selector).
+	 */
+	keystates = SDL_GetKeyboardState(NULL);
+	ignore_first_keyup = !(isEventOut && isEventOut(SDL_KEYDOWN)) && (
+	                     keystates[SDL_GetScancodeFromKey(SDLK_RETURN)] ||
+	                     keystates[SDL_GetScancodeFromKey(SDLK_KP_ENTER)] ||
+	                     keystates[SDL_GetScancodeFromKey(SDLK_SPACE)] ||
+	                     keystates[SDL_GetScancodeFromKey(SDLK_ESCAPE)]
+	);
+
 	/* Is the left mouse button still pressed? Yes -> Handle TOUCHEXIT objects here */
 	SDL_PumpEvents();
-	b = SDL_GetMouseState(&i, &j);
+	b = SDL_GetMouseState(&x, &y);
 
-	/* If current object is the scrollbar, and mouse is still down, we can scroll it */
-	/* also if the mouse pointer has left the scrollbar */
-	if (current_object != SDLGUI_NOTFOUND && dlg[current_object].type == SGSCROLLBAR) {
+	/* Report repeat objects until mouse button is released,
+	 * regardless of mouse position.  Used for scrollbar
+	 * object interactions */
+	if (current_object >= 0 && (dlg[current_object].flags & SG_REPEAT)) {
 		obj = current_object;
-		retbutton = obj;
 		oldbutton = obj;
 		if (b & SDL_BUTTON(1))
 		{
+			retbutton = obj;
 			dlg[obj].state |= SG_MOUSEDOWN;
 		}
 		else
 		{
-			current_object = 0;
 			dlg[obj].state &= ~SG_MOUSEDOWN;
 		}
-	}
-	else {
-		obj = SDLGui_FindObj(dlg, i, j);
-		current_object = obj;
+	} else {
+		SDLGui_ScaleMouseStateCoordinates(&x, &y);
+		obj = SDLGui_FindObj(dlg, x, y);
+
 		if (obj != SDLGUI_NOTFOUND && (dlg[obj].flags&SG_TOUCHEXIT) )
 		{
 			oldbutton = obj;
 			if (b & SDL_BUTTON(1))
 			{
-				dlg[obj].state |= SG_SELECTED;
 				retbutton = obj;
+				dlg[obj].state |= SG_SELECTED;
 			}
 		}
 	}
@@ -1224,17 +1232,13 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 	if (SDL_NumJoysticks() > 0)
 		joy = SDL_JoystickOpen(0);
 
-#if !WITH_SDL2
-	/* Enable unicode translation to get shifted etc chars with SDL_PollEvent */
-	nOldUnicodeMode = SDL_EnableUNICODE(true);
-#endif
 	Dprintf(("ENTER - obj: %d, old: %d, ret: %d\n", obj, oldbutton, retbutton));
 
 	/* The main loop */
 	while (retbutton == SDLGUI_NOTFOUND && !bQuitProgram)
 	{
 		if (SDL_WaitEvent(&sdlEvent) == 1)  /* Wait for events */
-		
+		{
 			switch (sdlEvent.type)
 			{
 			 case SDL_QUIT:
@@ -1245,8 +1249,7 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 				if (sdlEvent.button.button != SDL_BUTTON_LEFT)
 				{
 					/* Not left mouse button -> unsupported event */
-					if (pEventOut)
-						retbutton = SDLGUI_UNKNOWNEVENT;
+					retbutton = SDLGUI_UNKNOWNEVENT;
 					break;
 				}
 				/* It was the left button: Find the object under the mouse cursor */
@@ -1258,7 +1261,7 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 					{
 						dlg[obj].state |= SG_SELECTED;
 						SDLGui_DrawButton(dlg, obj);
-						SDL_UpdateRect(pSdlGuiScrn, (dlg[0].x+dlg[obj].x)*sdlgui_fontwidth-2, (dlg[0].y+dlg[obj].y)*sdlgui_fontheight-2,
+						Screen_UpdateRect(pSdlGuiScrn, (dlg[0].x+dlg[obj].x)*sdlgui_fontwidth-2, (dlg[0].y+dlg[obj].y)*sdlgui_fontheight-2,
 						               dlg[obj].w*sdlgui_fontwidth+4, dlg[obj].h*sdlgui_fontheight+4);
 						oldbutton=obj;
 					}
@@ -1279,8 +1282,7 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 				if (sdlEvent.button.button != SDL_BUTTON_LEFT)
 				{
 					/* Not left mouse button -> unsupported event */
-					if (pEventOut)
-						retbutton = SDLGUI_UNKNOWNEVENT;
+					retbutton = SDLGUI_UNKNOWNEVENT;
 					break;
 				}
 				/* It was the left button: Find the object under the mouse cursor */
@@ -1294,7 +1296,7 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 				{
 					dlg[oldbutton].state &= ~SG_SELECTED;
 					SDLGui_DrawButton(dlg, oldbutton);
-					SDL_UpdateRect(pSdlGuiScrn, (dlg[0].x+dlg[oldbutton].x)*sdlgui_fontwidth-2, (dlg[0].y+dlg[oldbutton].y)*sdlgui_fontheight-2,
+					Screen_UpdateRect(pSdlGuiScrn, (dlg[0].x+dlg[oldbutton].x)*sdlgui_fontwidth-2, (dlg[0].y+dlg[oldbutton].y)*sdlgui_fontheight-2,
 					               dlg[oldbutton].w*sdlgui_fontwidth+4, dlg[oldbutton].h*sdlgui_fontheight+4);
 					oldbutton = SDLGUI_NOTFOUND;
 				}
@@ -1329,16 +1331,36 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 				}
 				break;
 
+			 case SDL_JOYHATMOTION:
+				if (sdlEvent.jhat.value & SDL_HAT_LEFT)
+					retbutton = SDLGui_HandleShortcut(dlg, SG_SHORTCUT_LEFT);
+				else if (sdlEvent.jhat.value & SDL_HAT_RIGHT)
+					retbutton = SDLGui_HandleShortcut(dlg, SG_SHORTCUT_RIGHT);
+				if (sdlEvent.jhat.value & SDL_HAT_UP)
+				{
+					SDLGui_RemoveFocus(dlg, focused);
+					focused = SDLGui_FocusNext(dlg, focused, -1);
+				}
+				else if (sdlEvent.jhat.value & SDL_HAT_DOWN)
+				{
+					SDLGui_RemoveFocus(dlg, focused);
+					focused = SDLGui_FocusNext(dlg, focused, +1);
+				}
+				break;
+
 			 case SDL_JOYBUTTONDOWN:
 				retbutton = SDLGui_HandleSelection(dlg, focused, focused);
 				break;
 
 			 case SDL_JOYBALLMOTION:
-			 case SDL_JOYHATMOTION:
 			 case SDL_MOUSEMOTION:
+			 case SDL_KEYMAPCHANGED:
 				break;
 
 			 case SDL_KEYDOWN:                     /* Key pressed */
+				/* keys that need to support repeat,
+				 * need to be checked on press
+				 */
 				key = sdlEvent.key.keysym.sym;
 				/* keyboard shortcuts are with modifiers */
 				if (sdlEvent.key.keysym.mod & KMOD_LALT
@@ -1354,14 +1376,10 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 						retbutton = SDLGui_HandleShortcut(dlg, SG_SHORTCUT_DOWN);
 					else
 					{
-#if !WITH_SDL2
-						/* unicode member is needed to handle shifted etc special chars */
-						key = sdlEvent.key.keysym.unicode;
-#endif
 						if (key >= 33 && key <= 126)
 							retbutton = SDLGui_HandleShortcut(dlg, toupper(key));
 					}
-					if (!retbutton && pEventOut)
+					if (!retbutton)
 						retbutton = SDLGUI_UNKNOWNEVENT;
 					break;
 				}
@@ -1386,37 +1404,69 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 					SDLGui_RemoveFocus(dlg, focused);
 					focused = SDLGui_FocusNext(dlg, 1, -1);
 					break;
-				 case SDLK_SPACE:
-				 case SDLK_RETURN:
-				 case SDLK_KP_ENTER:
-					retbutton = SDLGui_HandleSelection(dlg, focused, focused);
-					break;
-				 case SDLK_ESCAPE:
-					retbutton = SDLGui_SearchFlags(dlg, SG_CANCEL);
-					break;
 				 default:
-					if (pEventOut)
-						retbutton = SDLGUI_UNKNOWNEVENT;
+					retbutton = SDLGUI_UNKNOWNEVENT;
+					break;
+				}
+				break;
+
+			case SDL_KEYUP:                     /* Key released */
+				/* keys potentially exiting dialog need
+				 * to be handed only on release, to avoid
+				 * leaking release events to emulation
+				 */
+				switch (sdlEvent.key.keysym.sym)
+				{
+				 case SDLK_SPACE:
+				 case SDLK_RETURN:
+				 case SDLK_KP_ENTER:
+					if (ignore_first_keyup)
+					{
+						ignore_first_keyup = false;
+						break;
+					}
+					retbutton = SDLGui_HandleSelection(dlg, focused, focused);
+					break;
+				 case SDLK_ESCAPE:
+					if (ignore_first_keyup)
+					{
+						ignore_first_keyup = false;
+						break;
+					}
+					retbutton = SDLGui_SearchFlags(dlg, SG_CANCEL);
+					break;
+				 default:
+					retbutton = SDLGUI_UNKNOWNEVENT;
 					break;
 				}
 				break;
 
-#if WITH_SDL2
 			 case SDL_WINDOWEVENT:
 				if (sdlEvent.window.event == SDL_WINDOWEVENT_SIZE_CHANGED
 				    || sdlEvent.window.event == SDL_WINDOWEVENT_RESTORED
 				    || sdlEvent.window.event == SDL_WINDOWEVENT_EXPOSED)
 				{
-					SDL_UpdateRect(pSdlGuiScrn, 0, 0, 0, 0);
+					Screen_UpdateRect(pSdlGuiScrn, 0, 0, 0, 0);
 				}
 				break;
-#endif
 
 			 default:
-				if (pEventOut)
-					retbutton = SDLGUI_UNKNOWNEVENT;
+				retbutton = SDLGUI_UNKNOWNEVENT;
 				break;
 			}
+			/* continue if unknown events were not not requested
+			 * specifically for this event type
+			 */
+			if (retbutton == SDLGUI_UNKNOWNEVENT &&
+			    !(isEventOut && isEventOut(sdlEvent.type)))
+			    retbutton = SDLGUI_NOTFOUND;
+		}
+	}
+
+	/* Copy event data of unsupported events if caller wants to have it */
+	if (retbutton == SDLGUI_UNKNOWNEVENT)
+	{
+		memcpy(pEventOut, &sdlEvent, sizeof(SDL_Event));
 	}
 
 	/* Restore background */
@@ -1426,19 +1476,25 @@ int SDLGui_DoDialog(SGOBJ *dlg, SDL_Event *pEventOut, bool KeepCurrentObject)
 		SDL_FreeSurface(pBgSurface);
 	}
 
-	/* Copy event data of unsupported events if caller wants to have it */
-	if (retbutton == SDLGUI_UNKNOWNEVENT && pEventOut)
-		memcpy(pEventOut, &sdlEvent, sizeof(SDL_Event));
-
 	if (retbutton == SDLGUI_QUIT)
 		bQuitProgram = true;
 
-#if !WITH_SDL2
-	SDL_EnableUNICODE(nOldUnicodeMode);
-#endif
 	if (joy)
 		SDL_JoystickClose(joy);
 
-	Dprintf(("EXIT - ret: %d, current: %d\n", retbutton, current_object));
+	Dprintf(("EXIT - ret: %d\n", retbutton));
 	return retbutton;
 }
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Show and process a dialog. Returns either:
+ * - index of the GUI item that was invoked
+ * - SDLGUI_QUIT if user wants to close Hatari
+ * - SDLGUI_ERROR if unable to show dialog
+ * GUI item indices are positive, other return values are negative
+ */
+int SDLGui_DoDialog(SGOBJ *dlg)
+{
+	return SDLGui_DoDialogExt(dlg, NULL, NULL, 0);
+}
