@@ -30,12 +30,14 @@ typedef struct
   int nNumberBase;
   int nSymbolLines;
   int nMemdumpLines;
+  int nFindLines;
   int nDisasmLines;
+  int nBacktraceLines;
   int nExceptionDebugMask;
   int nDisasmOptions;
   bool bDisasmUAE;
-  /* load symbols immediately on program start, and keep them after its termination */
-  bool bSymbolsResident;
+  /* load and free symbols for GEMDOS HD loaded programs automatically */
+  bool bSymbolsAutoLoad;
   /* whether to match all symbols or just types relevant for given command */
   bool bMatchAllSymbols;
 } CNF_DEBUGGER;
@@ -50,6 +52,19 @@ typedef struct
 } CNF_ROM;
 
 
+/* LILO (Linux loader) configuration */
+typedef struct
+{
+  char szCommandLine[256];	/* bootinfo CL_SIZE */
+  char szKernelFileName[FILENAME_MAX];
+  char szKernelSymbols[FILENAME_MAX];
+  char szRamdiskFileName[FILENAME_MAX];
+  bool bRamdiskToFastRam;
+  bool bKernelToFastRam;
+  bool bHaltOnReboot;
+} CNF_LILO;
+
+
 /* Sound configuration */
 typedef struct
 {
@@ -64,12 +79,19 @@ typedef struct
 
 
 
-/* RS232 configuration */
+/* RS232 / SCC configuration */
+#define CNF_SCC_CHANNELS_MAX		3
+#define CNF_SCC_CHANNELS_A_SERIAL	0
+#define CNF_SCC_CHANNELS_A_LAN		1
+#define CNF_SCC_CHANNELS_B		2
 typedef struct
 {
   bool bEnableRS232;
   char szOutFileName[FILENAME_MAX];
   char szInFileName[FILENAME_MAX];
+  bool EnableScc[CNF_SCC_CHANNELS_MAX];
+  char SccInFileName[CNF_SCC_CHANNELS_MAX][FILENAME_MAX];
+  char SccOutFileName[CNF_SCC_CHANNELS_MAX][FILENAME_MAX];
 } CNF_RS232;
 
 
@@ -78,13 +100,16 @@ typedef enum
 {
   KEYMAP_SYMBOLIC,  /* Use keymapping with symbolic (ASCII) key codes */
   KEYMAP_SCANCODE,  /* Use keymapping with PC keyboard scancodes */
-  KEYMAP_LOADED     /* Use keymapping with a map configuration file */
+  KEYMAP_OLD_LOADED /* Used for config file in former versions */
 } KEYMAPTYPE;
 
 typedef struct
 {
-  bool bDisableKeyRepeat;
+  bool bFastForwardKeyRepeat;
   KEYMAPTYPE nKeymapType;
+  int nCountryCode;
+  int nKbdLayout;
+  int nLanguage;
   char szMappingFileName[FILENAME_MAX];
 } CNF_KEYBOARD;
 
@@ -142,6 +167,7 @@ typedef enum
   JOYSTICK_KEYBOARD
 } JOYSTICKMODE;
 #define JOYSTICK_MODES 3
+#define JOYSTICK_BUTTONS 3
 
 typedef struct
 {
@@ -149,15 +175,18 @@ typedef struct
   bool bEnableAutoFire;
   bool bEnableJumpOnFire2;
   int nJoyId;
+  int nJoyButMap[JOYSTICK_BUTTONS];
   int nKeyCodeUp, nKeyCodeDown, nKeyCodeLeft, nKeyCodeRight, nKeyCodeFire;
+  int nKeyCodeB, nKeyCodeC, nKeyCodeOption, nKeyCodePause;
+  int nKeyCodeStar, nKeyCodeHash, nKeyCodeNum[10];
 } JOYSTICK;
 
 enum
 {
 	JOYID_JOYSTICK0,
 	JOYID_JOYSTICK1,
-	JOYID_STEPADA,
-	JOYID_STEPADB,
+	JOYID_JOYPADA,
+	JOYID_JOYPADB,
 	JOYID_PARPORT1,
 	JOYID_PARPORT2,
 	JOYSTICK_COUNT
@@ -211,28 +240,43 @@ typedef struct
 {
   int nGemdosDrive;
   bool bUseHardDiskDirectories;
-  bool bUseIdeMasterHardDiskImage;
-  bool bUseIdeSlaveHardDiskImage;
   WRITEPROTECTION nWriteProtection;
   GEMDOS_CHR_CONV nGemdosCase;
   bool bFilenameConversion;
   bool bGemdosHostTime;
   bool bBootFromHardDisk;
   char szHardDiskDirectories[MAX_HARDDRIVES][FILENAME_MAX];
-  char szIdeMasterHardDiskImage[FILENAME_MAX];
-  char szIdeSlaveHardDiskImage[FILENAME_MAX];
 } CNF_HARDDISK;
 
-/* SCSI/ACSI configuration */
+/* SCSI/ACSI/IDE configuration */
 #define MAX_ACSI_DEVS 8
 #define MAX_SCSI_DEVS 8
+#define MAX_IDE_DEVS 2
 
 typedef struct
 {
   bool bUseDevice;
   char sDeviceFile[FILENAME_MAX];
+  int nBlockSize;
+  int nScsiVersion;
 } CNF_SCSIDEV;
 
+typedef enum
+{
+  BYTESWAP_OFF,
+  BYTESWAP_ON,
+  BYTESWAP_AUTO
+} BYTESWAPPING;
+
+typedef struct
+{
+  bool bUseDevice;
+  BYTESWAPPING nByteSwap;
+  char sDeviceFile[FILENAME_MAX];
+  int nBlockSize;
+  int nDeviceType;
+} CNF_IDEDEV;
+
 /* Falcon register $FFFF8006 bits 6 & 7 (mirrored in $FFFF82C0 bits 0 & 1):
  * 00 Monochrome
  * 01 RGB - Colormonitor
@@ -252,39 +296,34 @@ typedef enum
   MONITOR_TYPE_TV
 } MONITORTYPE;
 
+
 /* Screen configuration */
 typedef struct
 {
   MONITORTYPE nMonitorType;
-  int nFrameSkips;
+  bool DisableVideo;
   bool bFullScreen;
-  bool bKeepResolution;
-#if WITH_SDL2
-  bool bResizable;
-#else
-  bool bKeepResolutionST;
-#endif
   bool bAllowOverscan;
   bool bAspectCorrect;
+  bool bShowStatusbar;
+  bool bShowDriveLed;
+  bool bMouseWarp;
+  bool bCrop;
+  bool bForceMax;
   bool bUseExtVdiResolutions;
+  bool bKeepResolution;
+  bool bResizable;
+  bool bUseVsync;
+  bool bUseSdlRenderer;
+  int ScreenShotFormat;
+  float nZoomFactor;
   int nSpec512Threshold;
-  int nForceBpp;
   int nVdiColors;
   int nVdiWidth;
   int nVdiHeight;
-  bool bMouseWarp;
-  bool bShowStatusbar;
-  bool bShowDriveLed;
-  bool bCrop;
-  bool bForceMax;
   int nMaxWidth;
   int nMaxHeight;
-#if WITH_SDL2
-  bool bUseSdlRenderer;
-  int nRenderScaleQuality;
-  bool bUseVsync;
-#endif
-  bool DisableVideo;
+  int nFrameSkips;
 } CNF_SCREEN;
 
 
@@ -295,6 +334,7 @@ typedef struct
   char szPrintToFileName[FILENAME_MAX];
 } CNF_PRINTER;
 
+#define MAX_MIDI_PORT_NAME 256 /* a guess */
 
 /* Midi configuration */
 typedef struct
@@ -302,8 +342,8 @@ typedef struct
   bool bEnableMidi;
   char sMidiInFileName[FILENAME_MAX];
   char sMidiOutFileName[FILENAME_MAX];
-  char sMidiInPortName[FILENAME_MAX];
-  char sMidiOutPortName[FILENAME_MAX];
+  char sMidiInPortName[MAX_MIDI_PORT_NAME];
+  char sMidiOutPortName[MAX_MIDI_PORT_NAME];
 } CNF_MIDI;
 
 
@@ -325,7 +365,6 @@ typedef enum
   DSP_TYPE_EMU
 } DSPTYPE;
 
-#if ENABLE_WINUAE_CPU
 typedef enum
 {
   FPU_NONE = 0,
@@ -333,7 +372,6 @@ typedef enum
   FPU_68882 = 68882,
   FPU_CPU = 68040
 } FPUTYPE;
-#endif
 
 typedef enum
 {
@@ -352,19 +390,19 @@ typedef struct
   MACHINETYPE nMachineType;
   bool bBlitter;                  /* TRUE if Blitter is enabled */
   DSPTYPE nDSPType;               /* how to "emulate" DSP */
+  int nRtcYear;
   bool bPatchTimerD;
   bool bFastBoot;                 /* Enable to patch TOS for fast boot */
   bool bFastForward;
-  bool bAddressSpace24;           /* Always set to true with old UAE cpu */
+  bool bAddressSpace24;           /* true if using a 24-bit address bus */
   VIDEOTIMINGMODE VideoTimingMode;
 
-#if ENABLE_WINUAE_CPU
   bool bCycleExactCpu;
+  bool bCpuDataCache;
   FPUTYPE n_FPUType;
   bool bCompatibleFPU;            /* More compatible FPU */
   bool bSoftFloatFPU;
   bool bMMU;                      /* TRUE if MMU is enabled */
-#endif
 } CNF_SYSTEM;
 
 typedef struct
@@ -391,7 +429,9 @@ typedef struct
   CNF_HARDDISK HardDisk;
   CNF_SCSIDEV Acsi[MAX_ACSI_DEVS];
   CNF_SCSIDEV Scsi[MAX_SCSI_DEVS];
+  CNF_IDEDEV Ide[MAX_IDE_DEVS];
   CNF_ROM Rom;
+  CNF_LILO Lilo;
   CNF_RS232 RS232;
   CNF_PRINTER Printer;
   CNF_MIDI Midi;
@@ -415,6 +455,11 @@ static inline bool Config_IsMachineSTE(void)
 	       ConfigureParams.System.nMachineType == MACHINE_MEGA_STE;
 }
 
+static inline bool Config_IsMachineMegaSTE(void)
+{
+	return ConfigureParams.System.nMachineType == MACHINE_MEGA_STE;
+}
+
 static inline bool Config_IsMachineTT(void)
 {
 	return ConfigureParams.System.nMachineType == MACHINE_TT;
@@ -431,5 +476,12 @@ extern void Configuration_Load(const char *psFileName);
 extern void Configuration_Save(void);
 extern void Configuration_MemorySnapShot_Capture(bool bSave);
 extern void Configuration_ChangeCpuFreq ( int CpuFreq_new );
+#ifdef EMSCRIPTEN
+extern void Configuration_ChangeMemory(int RamSizeKb);
+extern void Configuration_ChangeSystem(int nMachineType);
+extern void Configuration_ChangeTos(const char* szTosImageFileName);
+extern void Configuration_ChangeUseHardDiskDirectories(bool bUseHardDiskDirectories);
+extern void Configuration_ChangeFastForward(bool bFastForwardActive);
+#endif
 
 #endif
