@@ -9,9 +9,15 @@
 #define HATARI_PATHS_H
 
 extern void Paths_Init(const char *argv0);
+extern void Paths_UnInit(void);
 extern const char *Paths_GetWorkingDir(void);
 extern const char *Paths_GetDataDir(void);
 extern const char *Paths_GetUserHome(void);
 extern const char *Paths_GetHatariHome(void);
+extern const char *Paths_GetScreenShotDir(void);
+extern void Paths_SetScreenShotDir(const char *sNewDir);
+#if defined(__APPLE__)
+extern char *Paths_GetMacScreenShotDir(void);
+#endif
 
 #endif
