@@ -6,7 +6,7 @@
 
   Table with hardware IO handlers for the STE.
 */
-const char IoMemTabSTE_fileid[] = "Hatari ioMemTabSTE.c : " __DATE__ " " __TIME__;
+const char IoMemTabSTE_fileid[] = "Hatari ioMemTabSTE.c";
 
 #include "main.h"
 #include "configuration.h"
@@ -22,39 +22,55 @@ const char IoMemTabSTE_fileid[] = "Hatari ioMemTabSTE.c : " __DATE__ " " __TIME_
 #include "psg.h"
 #include "rs232.h"
 #include "rtc.h"
-#include "screen.h"
 #include "video.h"
 #include "blitter.h"
-#include "statusbar.h"
 #include "stMemory.h"
 
 
 /**
  * Take into account Mega STe Cache/Processor Control register $ff8e21.b
 	$FFFF8E21 Mega STe Cache/Processor Control
-		BIT 1 : Cache (0 - disabled, 1 - enabled)
-		BIT 0 : CPU Speed (0 - 8mhz, 1 - 16mhz)
+		BIT 0 : Cache (0 - disabled, 1 - enabled)
+		BIT 1 : CPU Speed (0 - 8MHz, 1 - 16MHz)
 
-   We handle only bit 0, bit 1 is ignored (cache is not emulated)
+   Cache can only be enabled if CPU speed is 16 MHz. If CPU is set to 8 MHz then cache bit will be forced to 0 :
+     if we write $FD at $FF8E21 (8 MHz with cache) then reading $FF8E21 will return $FC (8 MHz no cache)
 */
 void IoMemTabMegaSTE_CacheCpuCtrl_WriteByte(void)
 {
-	Uint8 busCtrl = IoMem_ReadByte(0xff8e21);
+	uint8_t Ctrl = IoMem_ReadByte(0xff8e21);
 
-	/* 68000 Frequency changed ? We change freq only in 68000 mode for a
-	 * normal MegaSTE, if the user did not request a faster one manually */
-	if (ConfigureParams.System.nCpuLevel == 0 && ConfigureParams.System.nCpuFreq <= 16)
+	/* If cache is enabled at 8 MHz then disable cache */
+	if ( ( ( Ctrl & 0x2 ) == 0 ) && ( Ctrl & 0x1 ) )
 	{
-		if ((busCtrl & 0x1) == 1) {
-			/* 16 Mhz bus for 68000 */
-			Configuration_ChangeCpuFreq ( 16 );
-		}
-		else {
-			/* 8 Mhz bus for 68000 */
-			Configuration_ChangeCpuFreq ( 8 );
-		}
+	      LOG_TRACE ( TRACE_MEM, "cache : megaste cache can't be enabled at 8 MHz $ff8e21=0x%02x pc=%x\n" , Ctrl , M68000_GetPC() );
+	      Ctrl &= 0xFE;					/* clear bit 0 */
+	      IoMem_WriteByte ( 0xff8e21 , Ctrl );
 	}
-	Statusbar_UpdateInfo();			/* Update clock speed in the status bar */
+
+	MegaSTE_CPU_Cache_Update ( Ctrl );
+}
+
+
+/**
+ * The register at $FF9200.b represents the DIP switches from the
+ * MegaSTE motherboard. The meaning of the switches is as follows:
+ *
+ *   1 - 6  off
+ *   7      on = 1.4mb HD floppy drive fitted
+ *   8      off (on = disable the DMA sound hardware)
+ *
+ * Switch 1 is represented by the lowest bit in the $FF9200 register,
+ * and switch 8 is represented by the highest bit. Logic is inverted,
+ * i.e. when the switch is "on", the bit is 0.
+ *
+ * We set the value to 0xBF to enable an HD floppy drive by default
+ * (earliest MegaSTE produced had a DD floppy drive, but later
+ * it was replaced by an HD drive)
+ */
+uint8_t IoMemTabMegaSTE_DIPSwitches_Read(void)
+{
+	return 0xbf;
 }
 
 
@@ -100,8 +116,9 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_STE[] =
 	{ 0xff825a, SIZE_WORD, Video_Color13_ReadWord, Video_Color13_WriteWord },		/* COLOR 13 */
 	{ 0xff825c, SIZE_WORD, Video_Color14_ReadWord, Video_Color14_WriteWord },		/* COLOR 14 */
 	{ 0xff825e, SIZE_WORD, Video_Color15_ReadWord, Video_Color15_WriteWord },		/* COLOR 15 */
-	{ 0xff8260, SIZE_BYTE, Video_Res_ReadByte, Video_Res_WriteByte },
-	{ 0xff8261, 3,         IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus errors here : return 0 not ff */
+	{ 0xff8260, SIZE_BYTE, Video_ResGlueShifter_ReadByte, Video_ResGlueShifter_WriteByte },	/* Resolution in GLUE-GSTMCU/Shifter */
+	{ 0xff8261, SIZE_BYTE, Video_ResShifter_ReadByte, Video_ResShifter_WriteByte },		/* Resolution in Shifter only */
+	{ 0xff8262, 2,         IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus errors here : return 0 not ff */
 	{ 0xff8264, SIZE_BYTE, Video_HorScroll_Read_8264, Video_HorScroll_Write_8264 },		/* STE horizontal fine scrolling (no prefetch) */
 	{ 0xff8265, SIZE_BYTE, Video_HorScroll_Read_8265, Video_HorScroll_Write_8265 },		/* STE horizontal fine scrolling */
 	{ 0xff8266, 26,        IoMem_VoidRead_00, IoMem_VoidWrite },                            /* No bus errors here : return 0 not ff */
@@ -181,15 +198,17 @@ const INTERCEPT_ACCESS_FUNC IoMemTable_STE[] =
 	{ 0xff8a3d, SIZE_BYTE, Blitter_Skew_ReadByte, Blitter_Skew_WriteByte },
 	{ 0xff8a3e, SIZE_WORD, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
 
+	/* SCU 0xff8e01-0xff8e0f registers set at run-time in ioMem.c for MegaSTE */
+
 	{ 0xff9000, SIZE_WORD, IoMem_VoidRead, IoMem_VoidWrite },                               /* No bus error here */
-	{ 0xff9201, SIZE_BYTE, Joy_StePadButtons_ReadByte, IoMem_WriteWithoutInterception },    /* Joypad fire buttons */
+	{ 0xff9200, SIZE_WORD, Joy_StePadButtons_DIPSwitches_ReadWord, Joy_StePadButtons_DIPSwitches_WriteWord },    /* Joypad fire buttons + MegaSTE DIP Switches */
 	{ 0xff9202, SIZE_WORD, Joy_StePadMulti_ReadWord, Joy_StePadMulti_WriteWord },           /* Joypad directions/buttons/selection */
-	{ 0xff9211, SIZE_BYTE, IoMem_VoidRead, IoMem_WriteWithoutInterception }, /* Joypad 0 X position (?) */
-	{ 0xff9213, SIZE_BYTE, IoMem_VoidRead, IoMem_WriteWithoutInterception }, /* Joypad 0 Y position (?) */
-	{ 0xff9215, SIZE_BYTE, IoMem_VoidRead, IoMem_WriteWithoutInterception }, /* Joypad 1 X position (?) */
-	{ 0xff9217, SIZE_BYTE, IoMem_VoidRead, IoMem_WriteWithoutInterception }, /* Joypad 1 Y position (?) */
-	{ 0xff9220, SIZE_WORD, IoMem_VoidRead, IoMem_WriteWithoutInterception }, /* Lightpen X position */
-	{ 0xff9222, SIZE_WORD, IoMem_VoidRead, IoMem_WriteWithoutInterception }, /* Lightpen Y position */
+	{ 0xff9211, SIZE_BYTE, Joy_StePadAnalog0X_ReadByte, IoMem_WriteWithoutInterception }, /* Joypad 0 Analog/Paddle X position */
+	{ 0xff9213, SIZE_BYTE, Joy_StePadAnalog0Y_ReadByte, IoMem_WriteWithoutInterception }, /* Joypad 0 Analog/Paddle Y position */
+	{ 0xff9215, SIZE_BYTE, Joy_StePadAnalog1X_ReadByte, IoMem_WriteWithoutInterception }, /* Joypad 1 Analog/Paddle X position */
+	{ 0xff9217, SIZE_BYTE, Joy_StePadAnalog1Y_ReadByte, IoMem_WriteWithoutInterception }, /* Joypad 1 Analog/Paddle Y position */
+	{ 0xff9220, SIZE_WORD, Joy_SteLightpenX_ReadWord, IoMem_WriteWithoutInterception },     /* Lightpen X position */
+	{ 0xff9222, SIZE_WORD, Joy_SteLightpenY_ReadWord, IoMem_WriteWithoutInterception },     /* Lightpen Y position */
 
 	{ 0xfffa01, SIZE_BYTE, MFP_GPIP_ReadByte, MFP_GPIP_WriteByte },
 	{ 0xfffa03, SIZE_BYTE, MFP_ActiveEdge_ReadByte, MFP_ActiveEdge_WriteByte },
