@@ -6,7 +6,7 @@
 
   Main initialization and event handling routines.
 */
-const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
+const char Main_fileid[] = "Hatari main.c";
 
 #include <time.h>
 #include <errno.h>
@@ -20,10 +20,12 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "dialog.h"
 #include "audio.h"
 #include "joy.h"
+#include "file.h"
 #include "floppy.h"
 #include "floppy_ipf.h"
 #include "floppy_stx.h"
 #include "gemdos.h"
+#include "mfp.h"
 #include "fdc.h"
 #include "hdc.h"
 #include "ide.h"
@@ -35,12 +37,15 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "m68000.h"
 #include "memorySnapShot.h"
 #include "midi.h"
+#include "ncr5380.h"
 #include "nvram.h"
 #include "paths.h"
 #include "printer.h"
 #include "reset.h"
 #include "resolution.h"
 #include "rs232.h"
+#include "rtc.h"
+#include "scc.h"
 #include "screen.h"
 #include "sdlgui.h"
 #include "shortcut.h"
@@ -54,6 +59,7 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "avi_record.h"
 #include "debugui.h"
 #include "clocks_timings.h"
+#include "utils.h"
 
 #include "hatari-glue.h"
 
@@ -68,17 +74,24 @@ const char Main_fileid[] = "Hatari main.c : " __DATE__ " " __TIME__;
 #include "gui-win/opencon.h"
 #endif
 
+#ifdef EMSCRIPTEN
+#include "emscripten.h"
+#endif
+
 bool bQuitProgram = false;                /* Flag to quit program cleanly */
 static int nQuitValue;                    /* exit value */
 
-static Uint32 nRunVBLs;                   /* Whether and how many VBLS to run before exit */
-static Uint32 nFirstMilliTick;            /* Ticks when VBL counting started */
-static Uint32 nVBLCount;                  /* Frame count */
-static int nVBLSlowdown = 1;		  /* host VBL wait multiplier */
+static uint32_t nRunVBLs;                 /* Whether and how many VBLS to run before exit */
+static uint32_t nFirstMilliTick;          /* Ticks when VBL counting started */
+static uint32_t nVBLCount;                /* Frame count */
+static int nVBLSlowdown = 1;              /* host VBL wait multiplier */
 
 static bool bEmulationActive = true;      /* Run emulation when started */
 static bool bAccurateDelays;              /* Host system has an accurate SDL_Delay()? */
+
 static bool bIgnoreNextMouseMotion = false;  /* Next mouse motion will be ignored (needed after SDL_WarpMouse) */
+static bool bAllowMouseWarp = true;       /* disabled when Hatari window loses mouse pointer / key focus */
+
 
 /*-----------------------------------------------------------------------*/
 /**
@@ -90,14 +103,14 @@ static bool bIgnoreNextMouseMotion = false;  /* Next mouse motion will be ignore
 #if HAVE_SYS_TIMES_H
 #include <unistd.h>
 #include <sys/times.h>
-static Uint32 Main_GetTicks(void)
+static uint32_t Main_GetTicks(void)
 {
 	static unsigned int ticks_to_msec = 0;
 	struct tms fields;
 	if (!ticks_to_msec)
 	{
 		ticks_to_msec = sysconf(_SC_CLK_TCK);
-		printf("OS clock ticks / second: %d\n", ticks_to_msec);
+		Log_Printf(LOG_INFO, "OS clock ticks / second: %d\n", ticks_to_msec);
 		/* Linux has 100Hz virtual clock so no accuracy loss there */
 		ticks_to_msec = 1000UL / ticks_to_msec;
 	}
@@ -121,16 +134,16 @@ static Uint32 Main_GetTicks(void)
  * return of SDL_GetTicks in micro sec.
  */
 
-static Sint64	Time_GetTicks ( void )
+static int64_t Time_GetTicks(void)
 {
-	Sint64	ticks_micro;
+	int64_t ticks_micro;
 
 #if HAVE_GETTIMEOFDAY
 	struct timeval	now;
 	gettimeofday ( &now , NULL );
-	ticks_micro = (Sint64)now.tv_sec * 1000000 + now.tv_usec;
+	ticks_micro = (int64_t)now.tv_sec * 1000000 + now.tv_usec;
 #else
-	ticks_micro = (Sint64)SDL_GetTicks() * 1000;		/* milli sec -> micro sec */
+	ticks_micro = (int64_t)SDL_GetTicks() * 1000;		/* milli sec -> micro sec */
 #endif
 
 	return ticks_micro;
@@ -144,8 +157,11 @@ static Sint64	Time_GetTicks ( void )
  * (which is portable, but less accurate as is uses milli-seconds)
  */
 
-static void	Time_Delay ( Sint64 ticks_micro )
+static void Time_Delay(int64_t ticks_micro)
 {
+#ifdef EMSCRIPTEN
+	emscripten_sleep((uint32_t)(ticks_micro / 1000));	/* micro sec -> milli sec */
+#else
 #if HAVE_NANOSLEEP
 	struct timespec	ts;
 	int		ret;
@@ -158,8 +174,35 @@ static void	Time_Delay ( Sint64 ticks_micro )
                 ret = nanosleep(&ts, &ts);
 	} while ( ret && ( errno == EINTR ) );		/* keep on sleeping if we were interrupted */
 #else
-	SDL_Delay ( (Uint32)(ticks_micro / 1000) ) ;	/* micro sec -> milli sec */
+	SDL_Delay((uint32_t)(ticks_micro / 1000)) ;	/* micro sec -> milli sec */
 #endif
+#endif
+}
+
+/*-----------------------------------------------------------------------*/
+/**
+ * Always print speeds in Benchmark mode, otherwise only
+ * if loglevel is "info" or higher (when time is recorded).
+ */
+static void Main_PrintSpeed(void)
+{
+	if (!nFirstMilliTick)
+		return;
+
+	int interval = Main_GetTicks() - nFirstMilliTick;
+	static float previous;
+	float current;
+	int level = LOG_INFO;
+
+	if (BenchmarkMode && ConfigureParams.Log.nTextLogLevel < level)
+		level = ConfigureParams.Log.nTextLogLevel;
+
+	current = (1000.0 * nVBLCount) / interval;
+	Log_Printf(level, "SPEED: %.1f VBL/s (%d/%.1fs), diff=%.1f%%\n",
+		   current, nVBLCount, interval/1000.0,
+		   previous>0.0 ? 100*(current-previous)/previous : 0.0);
+	nVBLCount = nFirstMilliTick = 0;
+	previous = current;
 }
 
 
@@ -179,27 +222,15 @@ bool Main_PauseEmulation(bool visualize)
 	bEmulationActive = false;
 	if (visualize)
 	{
-		if (nFirstMilliTick)
-		{
-			int interval = Main_GetTicks() - nFirstMilliTick;
-			static float previous;
-			float current;
+		Main_PrintSpeed();
 
-			current = (1000.0 * nVBLCount) / interval;
-			printf("SPEED: %.1f VBL/s (%d/%.1fs), diff=%.1f%%\n",
-			       current, nVBLCount, interval/1000.0,
-			       previous>0.0 ? 100*(current-previous)/previous : 0.0);
-			nVBLCount = nFirstMilliTick = 0;
-			previous = current;
-		}
-		
 		Statusbar_AddMessage("Emulation paused", 100);
 		/* make sure msg gets shown */
 		Statusbar_Update(sdlscrn, true);
 
 		if (bGrabMouse && !bInFullScreen)
 			/* Un-grab mouse pointer in windowed mode */
-			SDL_WM_GrabInput(SDL_GRAB_OFF);
+			SDL_SetRelativeMouseMode(false);
 	}
 	return true;
 }
@@ -224,7 +255,7 @@ bool Main_UnPauseEmulation(void)
 
 	if (bGrabMouse)
 		/* Grab mouse pointer again */
-		SDL_WM_GrabInput(SDL_GRAB_ON);
+		SDL_SetRelativeMouseMode(true);
 	return true;
 }
 
@@ -257,31 +288,46 @@ void Main_RequestQuit(int exitval)
 	nQuitValue = exitval;
 }
 
+/**
+ * Set exit value and enable quit flag
+ */
+void Main_SetQuitValue(int exitval)
+{
+	bQuitProgram = true;
+	M68000_SetSpecial(SPCFLAG_BRK);
+	nQuitValue = exitval;
+}
+
 /*-----------------------------------------------------------------------*/
 /**
  * Set how many VBLs Hatari should run, from the moment this function
- * is called.
+ * is called and return zero.
+ *
+ * If zero value given instead, returns earlier set VBL count.
  */
-void Main_SetRunVBLs(Uint32 vbls)
+uint32_t Main_SetRunVBLs(uint32_t vbls)
 {
-	fprintf(stderr, "Exit after %d VBLs.\n", vbls);
+	if (!vbls)
+		return nRunVBLs;
+
 	nRunVBLs = vbls;
 	nVBLCount = 0;
+	return 0;
 }
 
 /*-----------------------------------------------------------------------*/
 /**
  * Set VBL wait slowdown factor/multiplayer
+ *
+ * Return NULL on success, error string on error
  */
-bool Main_SetVBLSlowdown(int factor)
+const char* Main_SetVBLSlowdown(int factor)
 {
-	if (factor < 1 || factor > 8) {
-		fprintf(stderr, "ERROR: invalid VBL slowdown factor %d, should be 1-8!\n", factor);
-		return false;
+	if (factor < 1 || factor > 30) {
+		return "invalid VBL slowdown factor, should be 1-30";
 	}
-	fprintf(stderr, "Slow down host VBL wait by factor of %d.\n", factor);
 	nVBLSlowdown = factor;
-	return true;
+	return NULL;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -296,10 +342,10 @@ bool Main_SetVBLSlowdown(int factor)
  */
 void Main_WaitOnVbl(void)
 {
-	Sint64 CurrentTicks;
-	static Sint64 DestTicks = 0;
-	Sint64 FrameDuration_micro;
-	Sint64 nDelay;
+	int64_t CurrentTicks;
+	static int64_t DestTicks = 0;
+	int64_t FrameDuration_micro;
+	int64_t nDelay;
 
 	nVBLCount++;
 	if (nRunVBLs &&	nVBLCount >= nRunVBLs)
@@ -309,7 +355,7 @@ void Main_WaitOnVbl(void)
 		exit(0);
 	}
 
-//	FrameDuration_micro = (Sint64) ( 1000000.0 / nScreenRefreshRate + 0.5 );	/* round to closest integer */
+//	FrameDuration_micro = (int64_t) ( 1000000.0 / nScreenRefreshRate + 0.5 );	/* round to closest integer */
 	FrameDuration_micro = ClocksTimings_GetVBLDuration_micro ( ConfigureParams.System.nMachineType , nScreenRefreshRate );
 	FrameDuration_micro *= nVBLSlowdown;
 	CurrentTicks = Time_GetTicks();
@@ -339,10 +385,13 @@ void Main_WaitOnVbl(void)
 		if (nFrameSkips < ConfigureParams.Screen.nFrameSkips)
 		{
 			nFrameSkips += 1;
-			// Log_Printf(LOG_DEBUG, "Increased frameskip to %d\n", nFrameSkips);
+			Log_Printf(LOG_DEBUG, "Increased frameskip to %d\n", nFrameSkips);
 		}
 		/* Only update DestTicks for next VBL */
 		DestTicks = CurrentTicks + FrameDuration_micro;
+		#ifdef EMSCRIPTEN
+		emscripten_sleep(0);
+		#endif
 		return;
 	}
 
@@ -354,7 +403,7 @@ void Main_WaitOnVbl(void)
 	    && 2*nDelay > FrameDuration_micro/nFrameSkips)
 	{
 		nFrameSkips -= 1;
-		// Log_Printf(LOG_DEBUG, "Decreased frameskip to %d\n", nFrameSkips);
+		Log_Printf(LOG_DEBUG, "Decreased frameskip to %d\n", nFrameSkips);
 	}
 
 	if (bAccurateDelays)
@@ -418,28 +467,53 @@ static void Main_CheckForAccurateDelays(void)
  * Set mouse pointer to new x,y coordinates and set flag to ignore
  * the mouse event that is generated by SDL_WarpMouse().
  *
- * Skip the request is it's not position restore and mouse warping is disabled.
+ * Skip the request if:
+ * - it's not position restore and mouse warping is disabled, or
+ * - mouse warp disable due to mouse leaving Hatari window or focus loss
+ *   (i.e. user isn't interacting with it)
  */
 void Main_WarpMouse(int x, int y, bool restore)
 {
 	if (!(restore || ConfigureParams.Screen.bMouseWarp))
 		return;
-#if WITH_SDL2
+	if (!bAllowMouseWarp)
+		return;
+
 	SDL_WarpMouseInWindow(sdlWindow, x, y);
-#else
-	SDL_WarpMouse(x, y);
-#endif
 	bIgnoreNextMouseMotion = true;
 }
 
 
+/* ----------------------------------------------------------------------- */
+/**
+ * Set mouse cursor visibility and return if it was visible before.
+ */
+bool Main_ShowCursor(bool show)
+{
+	bool bOldVisibility;
+
+	bOldVisibility = SDL_ShowCursor(SDL_QUERY) == SDL_ENABLE;
+	if (bOldVisibility != show)
+	{
+		if (show)
+		{
+			SDL_ShowCursor(SDL_ENABLE);
+		}
+		else
+		{
+			SDL_ShowCursor(SDL_DISABLE);
+		}
+	}
+	return bOldVisibility;
+}
+
+
 /* ----------------------------------------------------------------------- */
 /**
  * Handle mouse motion event.
  */
-static void Main_HandleMouseMotion(SDL_Event *pEvent)
+static void Main_HandleMouseMotion(int dx, int dy)
 {
-	int dx, dy;
 	static int ax = 0, ay = 0;
 
 	/* Ignore motion when position has changed right after a reset or TOS
@@ -450,9 +524,6 @@ static void Main_HandleMouseMotion(SDL_Event *pEvent)
 		return;
 	}
 
-	dx = pEvent->motion.xrel;
-	dy = pEvent->motion.yrel;
-
 	/* In zoomed low res mode, we divide dx and dy by the zoom factor so that
 	 * the ST mouse cursor stays in sync with the host mouse. However, we have
 	 * to take care of lowest bit of dx and dy which will get lost when
@@ -471,6 +542,27 @@ static void Main_HandleMouseMotion(SDL_Event *pEvent)
 		dy /= nScreenZoomY;
 	}
 
+	if (!bInFullScreen)			/* Consider window scaling? */
+	{
+		static int wx, wy;
+		int win_width, win_height, ndx, ndy;
+
+		SDL_GetWindowSize(sdlWindow, &win_width, &win_height);
+
+		if (sdlscrn->w != win_width)
+		{
+			ndx = dx * sdlscrn->w;
+			dx = (ndx + wx) / win_width;
+			wx = (ndx + wx) % win_width;
+		}
+		if (sdlscrn->h != win_height)
+		{
+			ndy = dy * sdlscrn->h;
+			dy = (ndy + wy) / win_height;
+			wy = (ndy + wy) % win_height;
+		}
+	}
+
 	KeyboardProcessor.Mouse.dx += dx;
 	KeyboardProcessor.Mouse.dy += dy;
 }
@@ -488,6 +580,7 @@ void Main_EventHandler(void)
 	SDL_Event event;
 	int events;
 	int remotepause;
+	static int mleave_x = -1, mleave_y = -1;
 
 	do
 	{
@@ -522,12 +615,10 @@ void Main_EventHandler(void)
 			break;
 
 		 case SDL_KEYDOWN:
-#if WITH_SDL2
 			if (event.key.repeat) {
 				bContinueProcessing = true;
 				break;
 			}
-#endif
 			Keymap_KeyDown(&event.key.keysym);
 			break;
 
@@ -536,7 +627,7 @@ void Main_EventHandler(void)
 			break;
 
 		 case SDL_MOUSEMOTION:               /* Read/Update internal mouse position */
-			Main_HandleMouseMotion(&event);
+			Main_HandleMouseMotion(event.motion.xrel, event.motion.yrel);
 			bContinueProcessing = true;
 			break;
 
@@ -555,18 +646,6 @@ void Main_EventHandler(void)
 				/* Start double-click sequence in emulation time */
 				Keyboard.LButtonDblClk = 1;
 			}
-#if !WITH_SDL2
-			else if (event.button.button == SDL_BUTTON_WHEELDOWN)
-			{
-				/* Simulate pressing the "cursor down" key */
-				IKBD_PressSTKey(0x50, true);
-			}
-			else if (event.button.button == SDL_BUTTON_WHEELUP)
-			{
-				/* Simulate pressing the "cursor up" key */
-				IKBD_PressSTKey(0x48, true);
-			}
-#endif
 			break;
 
 		 case SDL_MOUSEBUTTONUP:
@@ -578,21 +657,8 @@ void Main_EventHandler(void)
 			{
 				Keyboard.bRButtonDown &= ~BUTTON_MOUSE;
 			}
-#if !WITH_SDL2
-			else if (event.button.button == SDL_BUTTON_WHEELDOWN)
-			{
-				/* Simulate releasing the "cursor down" key */
-				IKBD_PressSTKey(0x50, false);
-			}
-			else if (event.button.button == SDL_BUTTON_WHEELUP)
-			{
-				/* Simulate releasing the "cursor up" key */
-				IKBD_PressSTKey(0x48, false);
-			}
-#endif
 			break;
 
-#if WITH_SDL2
 		 case SDL_MOUSEWHEEL:
 			/* Simulate cursor keys on mouse wheel events */
 			if (event.wheel.x > 0)
@@ -618,27 +684,54 @@ void Main_EventHandler(void)
 			break;
 
 		 case SDL_WINDOWEVENT:
-			/* Note: any changes here should most likely be done
-			 * also in sdlgui.c::SDLGui_DoDialog()
-			 */
-			if (event.window.event == SDL_WINDOWEVENT_SIZE_CHANGED
-			    || event.window.event == SDL_WINDOWEVENT_RESTORED
-			    || event.window.event == SDL_WINDOWEVENT_EXPOSED)
-			{
-				SDL_UpdateRect(sdlscrn, 0, 0, 0, 0);
-			}
-			if (event.window.event == SDL_WINDOWEVENT_EXPOSED
-			    && !ConfigureParams.Screen.bUseSdlRenderer)
-			{
-				/* Hack: Redraw screen here when going into
-				 * fullscreen mode without SDL renderer */
-				sdlscrn = SDL_GetWindowSurface(sdlWindow);
-				Screen_SetFullUpdate();
-				Statusbar_Init(sdlscrn);
+			Log_Printf(LOG_DEBUG, "SDL2 window event: 0x%x\n", event.window.event);
+			switch(event.window.event) {
+			case SDL_WINDOWEVENT_EXPOSED:
+				if (!ConfigureParams.Screen.bUseSdlRenderer)
+				{
+					/* Hack: Redraw screen here when going into
+					 * fullscreen mode without SDL renderer */
+					sdlscrn = SDL_GetWindowSurface(sdlWindow);
+					Screen_SetFullUpdate();
+					Statusbar_Init(sdlscrn);
+				}
+				/* fall through */
+			case SDL_WINDOWEVENT_RESTORED:
+				/* Note: any changes here should most likely
+				 * be done also in sdlgui.c::SDLGui_DoDialog()
+				 */
+				Screen_UpdateRect(sdlscrn, 0, 0, 0, 0);
+				break;
+			case SDL_WINDOWEVENT_SIZE_CHANGED:
+				/* internal & external window size changes */
+				Screen_SetTextureScale(sdlscrn->w, sdlscrn->h,
+						       event.window.data1,
+						       event.window.data2, false);
+				Screen_UpdateRect(sdlscrn, 0, 0, 0, 0);
+				break;
+				/* mouse & keyboard focus */
+			case SDL_WINDOWEVENT_ENTER:
+				if (mleave_x != -1)
+				{
+					int new_x, new_y;
+					SDL_GetMouseState(&new_x, &new_y);
+					Main_HandleMouseMotion(new_x - mleave_x,
+					                       new_y - mleave_y);
+					mleave_x = mleave_y = -1;
+				}
+				/* fall through */
+			case SDL_WINDOWEVENT_FOCUS_GAINED:
+				bAllowMouseWarp = true;
+				break;
+			case SDL_WINDOWEVENT_LEAVE:
+				SDL_GetMouseState(&mleave_x, &mleave_y);
+				/* fall through */
+			case SDL_WINDOWEVENT_FOCUS_LOST:
+				bAllowMouseWarp = false;
+				break;
 			}
 			bContinueProcessing = true;
 			break;
-#endif
 
 		 default:
 			/* don't let unknown events delay event processing */
@@ -655,17 +748,10 @@ void Main_EventHandler(void)
  */
 void Main_SetTitle(const char *title)
 {
-#if WITH_SDL2
 	if (title)
 		SDL_SetWindowTitle(sdlWindow, title);
 	else
 		SDL_SetWindowTitle(sdlWindow, PROG_NAME);
-#else
-	if (title)
-		SDL_WM_SetCaption(title, "Hatari");
-	else
-		SDL_WM_SetCaption(PROG_NAME, "Hatari");
-#endif
 }
 
 /*-----------------------------------------------------------------------*/
@@ -692,23 +778,20 @@ static void Main_Init(void)
 	/* Open debug log file */
 	if (!Log_Init())
 	{
-		fprintf(stderr, "Logging/tracing initialization failed\n");
-		exit(-1);
+		Main_ErrorExit("Logging/tracing initialization failed", NULL, -1);
 	}
 	Log_Printf(LOG_INFO, PROG_NAME ", compiled on:  " __DATE__ ", " __TIME__ "\n");
 
 	/* Init SDL's video subsystem. Note: Audio subsystem
 	   will be initialized later (failure not fatal). */
-	if (SDL_Init(SDL_INIT_VIDEO | Opt_GetNoParachuteFlag()) < 0)
+	if (SDL_Init(SDL_INIT_VIDEO) < 0)
 	{
-		fprintf(stderr, "Could not initialize the SDL library:\n %s\n", SDL_GetError() );
-		exit(-1);
+		Main_ErrorExit("Could not initialize the SDL library:", SDL_GetError(), -1);
 	}
 
 	if ( IPF_Init() != true )
 	{
-		fprintf(stderr, "Could not initialize the IPF support\n" );
-		exit(-1);
+		Main_ErrorExit("Could not initialize the IPF support", NULL, -1);
 	}
 
 	ClocksTimings_InitMachine ( ConfigureParams.System.nMachineType );
@@ -717,9 +800,10 @@ static void Main_Init(void)
 	Resolution_Init();
 	SDLGui_Init();
 	Printer_Init();
+	MFP_Init(MFP_Array);
 	RS232_Init();
+	SCC_Init();
 	Midi_Init();
-	Control_CheckUpdates();       /* enable window embedding? */
 	Videl_Init();
 	Screen_Init();
 	Main_SetTitle(NULL);
@@ -737,11 +821,12 @@ static void Main_Init(void)
 
 	/* Init HD emulation */
 	HDC_Init();
+	Ncr5380_Init();
 	Ide_Init();
 	GemDOS_Init();
 	if (ConfigureParams.HardDisk.bUseHardDiskDirectories)
 	{
-		/* uses variables set by HDC_Init()! */
+		/* uses variables set by HDC_Init/Ncr5380_Init/Ide_Init */
 		GemDOS_InitDrives();
 	}
 
@@ -754,7 +839,9 @@ static void Main_Init(void)
 	if (!bTosImageLoaded || bQuitProgram)
 	{
 		if (!bTosImageLoaded)
-			fprintf(stderr, "Failed to load TOS image!\n");
+		{
+			Main_ErrorExit("Failed to load TOS image", NULL, -2);
+		}
 		SDL_Quit();
 		exit(-2);
 	}
@@ -762,7 +849,8 @@ static void Main_Init(void)
 	IoMem_Init();
 	NvRam_Init();
 	Sound_Init();
-	
+	Rtc_Init();
+
 	/* done as last, needs CPU & DSP running... */
 	DebugUI_Init();
 }
@@ -777,10 +865,12 @@ static void Main_UnInit(void)
 	Screen_ReturnFromFullScreen();
 	Floppy_UnInit();
 	HDC_UnInit();
+	Ncr5380_UnInit();
 	Midi_UnInit();
+	SCC_UnInit();
 	RS232_UnInit();
 	Printer_UnInit();
-	IoMem_UnInit();
+	IoMem_UnInit(ConfigureParams.System.nMachineType);
 	NvRam_UnInit();
 	GemDOS_UnInitDrives();
 	Ide_UnInit();
@@ -799,26 +889,31 @@ static void Main_UnInit(void)
 	SDL_Quit();
 
 	/* Close debug log file */
+	DebugUI_UnInit();
 	Log_UnInit();
+
+	Paths_UnInit();
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Load initial configuration file(s)
+ * Load initial configuration files. The global config file is skipped in
+ * test mode (i.e. if the HATARI_TEST environment variable has been set),
+ * so that the test has always full control over the configuration settings.
  */
 static void Main_LoadInitialConfig(void)
 {
 	char *psGlobalConfig;
 
-	psGlobalConfig = malloc(FILENAME_MAX);
+	if (getenv("HATARI_TEST"))
+		psGlobalConfig = NULL;
+	else
+		psGlobalConfig = malloc(FILENAME_MAX);
 	if (psGlobalConfig)
 	{
-#if defined(__AMIGAOS4__)
-		strncpy(psGlobalConfig, CONFDIR"hatari.cfg", FILENAME_MAX);
-#else
-		snprintf(psGlobalConfig, FILENAME_MAX, CONFDIR"%chatari.cfg", PATHSEP);
-#endif
+		File_MakePathBuf(psGlobalConfig, FILENAME_MAX, CONFDIR,
+		                 "hatari", "cfg");
 		/* Try to load the global configuration file */
 		Configuration_Load(psGlobalConfig);
 
@@ -827,6 +922,8 @@ static void Main_LoadInitialConfig(void)
 
 	/* Now try the users configuration file */
 	Configuration_Load(NULL);
+	if (ConfigureParams.Keyboard.nLanguage == TOS_LANG_UNKNOWN)
+		ConfigureParams.Keyboard.nLanguage = TOS_DefaultLanguage();
 }
 
 /*-----------------------------------------------------------------------*/
@@ -835,30 +932,87 @@ static void Main_LoadInitialConfig(void)
  */
 static void Main_StatusbarSetup(void)
 {
-	const char *name = NULL;
-	SDLKey key;
+	struct {
+		const int id;
+		bool mod;
+		char *name;
+	} keys[] = {
+		{ SHORTCUT_OPTIONS, false, NULL },
+		{ SHORTCUT_MOUSEGRAB, false, NULL }
+	};
+	const char *name;
+	bool named;
+	SDL_Keycode key;
+	int i;
 
-	key = ConfigureParams.Shortcut.withoutModifier[SHORTCUT_OPTIONS];
-	if (!key)
-		key = ConfigureParams.Shortcut.withModifier[SHORTCUT_OPTIONS];
-	if (key)
-		name = SDL_GetKeyName(key);
-	if (name)
+	named = false;
+	for (i = 0; i < ARRAY_SIZE(keys); i++)
 	{
-		char message[24], *keyname;
-#ifdef _MUDFLAP
-		__mf_register((void*)name, 32, __MF_TYPE_GUESS, "SDL keyname");
-#endif
-		keyname = Str_ToUpper(strdup(name));
-		snprintf(message, sizeof(message), "Press %s for Options", keyname);
-		free(keyname);
-
+		key = ConfigureParams.Shortcut.withoutModifier[keys[i].id];
+		if (!key)
+		{
+			key = ConfigureParams.Shortcut.withModifier[keys[i].id];
+			if (!key)
+				continue;
+			keys[i].mod = true;
+		}
+		name = SDL_GetKeyName(key);
+		if (!name)
+			continue;
+		keys[i].name = Str_ToUpper(strdup(name));
+		named = true;
+	}
+	if (named)
+	{
+		char message[60];
+		snprintf(message, sizeof(message), "Press %s%s for Options, %s%s for mouse grab toggle",
+			 keys[0].mod ? "AltGr+": "", keys[0].name,
+			 keys[1].mod ? "AltGr+": "", keys[1].name);
+		for (i = 0; i < ARRAY_SIZE(keys); i++)
+		{
+			if (keys[i].name)
+				free(keys[i].name);
+		}
 		Statusbar_AddMessage(message, 5000);
 	}
 	/* update information loaded by Main_Init() */
 	Statusbar_UpdateInfo();
 }
 
+/**
+ * Error exit wrapper, to make sure user sees the error messages
+ * also on Windows.
+ *
+ * If message is given, Windows console is opened to show it,
+ * otherwise it's assumed to be already open and relevant
+ * messages shown before calling this.
+ *
+ * User input is waited on Windows, to make sure user sees
+ * the message before console closes.
+ *
+ * Value overrides nQuitValue as process exit/return value.
+ */
+void Main_ErrorExit(const char *msg1, const char *msg2, int errval)
+{
+	if (msg1)
+	{
+#ifdef WIN32
+		Win_ForceCon();
+#endif
+		if (msg2)
+			fprintf(stderr, "ERROR: %s\n\t%s\n", msg1, msg2);
+		else
+			fprintf(stderr, "ERROR: %s!\n", msg1);
+	}
+
+	SDL_Quit();
+
+#ifdef WIN32
+	fputs("<press Enter to exit>\n", stderr);
+	(void)fgetc(stdin);
+#endif
+	exit(errval);
+}
 
 /**
  * Main
@@ -868,7 +1022,7 @@ static void Main_StatusbarSetup(void)
 int main(int argc, char *argv[])
 {
 	/* Generate random seed */
-	srand(time(NULL));
+	Hatari_srand(time(NULL));
 
 	/* Logs default to stderr at start */
 	Log_Default();
@@ -888,7 +1042,8 @@ int main(int argc, char *argv[])
 	/* Check for any passed parameters */
 	if (!Opt_ParseParameters(argc, (const char * const *)argv))
 	{
-		return 1;
+		Control_RemoveFifo();
+		Main_ErrorExit(NULL, NULL, 1);
 	}
 	/* monitor type option might require "reset" -> true */
 	Configuration_Apply(true);
@@ -901,9 +1056,6 @@ int main(int argc, char *argv[])
 	/* Needed on maemo but useful also with normal X11 window managers for
 	 * window grouping when you have multiple Hatari SDL windows open */
 	setenv("SDL_VIDEO_X11_WMCLASS", "hatari", 1);
-
-	/* Needed for proper behavior of Caps Lock on some systems */
-	setenv("SDL_DISABLE_LOCK_KEYS", "1", 1);
 #endif
 
 	/* Init emulator system */
@@ -919,7 +1071,7 @@ int main(int argc, char *argv[])
 		Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
 			ConfigureParams.Video.AviRecordFps == 0 ?
 				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
-				(Uint32)ConfigureParams.Video.AviRecordFps << CLOCKS_TIMINGS_SHIFT_VBL ,
+				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , ConfigureParams.Video.AviRecordFps ) ,
 			1 << CLOCKS_TIMINGS_SHIFT_VBL ,
 			ConfigureParams.Video.AviRecordVcodec );
 
@@ -927,6 +1079,7 @@ int main(int argc, char *argv[])
 	Main_UnPauseEmulation();
 	M68000_Start();                 /* Start emulation */
 
+	Control_RemoveFifo();
 	if (bRecordingAvi)
 	{
 		/* cleanly close the avi file */
