@@ -16,9 +16,8 @@
   reduce redundancy and the function 'MemorySnapShot_Store' decides if it
   should save or restore the data.
 */
-const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " __TIME__;
+const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c";
 
-#include <SDL_types.h>
 #include <errno.h>
 
 #include "main.h"
@@ -34,8 +33,8 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "gemdos.h"
 #include "acia.h"
 #include "ikbd.h"
-#include "cycInt.h"
 #include "cycles.h"
+#include "cycInt.h"
 #include "ioMem.h"
 #include "log.h"
 #include "m68000.h"
@@ -44,9 +43,11 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "midi.h"
 #include "psg.h"
 #include "reset.h"
+#include "scc.h"
 #include "sound.h"
 #include "str.h"
 #include "stMemory.h"
+#include "scu_vme.h"
 #include "tos.h"
 #include "screen.h"
 #include "screenConvert.h"
@@ -55,10 +56,10 @@ const char MemorySnapShot_fileid[] = "Hatari memorySnapShot.c : " __DATE__ " " _
 #include "falcon/crossbar.h"
 #include "falcon/videl.h"
 #include "statusbar.h"
-#include "cart.h"
+#include "hatari-glue.h"
 
 
-#define VERSION_STRING      "2.1.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
+#define VERSION_STRING      "2.6.0"   /* Version number of compatible memory snapshots - Always 6 bytes (inc' NULL) */
 #define SNAPSHOT_MAGIC      0xDeadBeef
 
 #if HAVE_LIBZ
@@ -83,6 +84,10 @@ static MSS_File CaptureFile;
 static bool bCaptureSave, bCaptureError;
 
 
+static char Temp_FileName[FILENAME_MAX];
+static bool Temp_Confirm;
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Open file.
@@ -161,12 +166,9 @@ static int MemorySnapShot_fseek(MSS_File fhndl, int pos)
 static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave, bool bConfirm)
 {
 	char VersionString[] = VERSION_STRING;
-#if ENABLE_WINUAE_CPU
-# define CORE_VERSION 1
-#else
-# define CORE_VERSION 0
-#endif
-	Uint8 CpuCore;
+
+#define CORE_VERSION 1
+	uint8_t CpuCore;
 
 	/* Set error */
 	bCaptureError = false;
@@ -217,7 +219,7 @@ static bool MemorySnapShot_OpenFile(const char *pszFileName, bool bSave, bool bC
 			Log_AlertDlg(LOG_ERROR,
 				     "Unable to restore Hatari memory state.\n"
 				     "Given state file is compatible only with\n"
-				     "Hatari version " VERSION_STRING ".");
+				     "Hatari version %s", VersionString);
 			bCaptureError = true;
 			return false;
 		}
@@ -299,10 +301,42 @@ void MemorySnapShot_Store(void *pData, int Size)
  */
 void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 {
-	Uint32 magic = SNAPSHOT_MAGIC;
+//fprintf ( stderr , "MemorySnapShot_Capture in\n" );
+	/* Make a temporary copy of the parameters for MemorySnapShot_Capture_Do() */
+	Str_Copy(Temp_FileName, pszFileName, FILENAME_MAX);
+	Temp_Confirm = bConfirm;
+
+	/* With WinUAE cpu core, capture is done from m68k_run_xxx() after the end of the current instruction */
+	UAE_Set_State_Save ();
+//fprintf ( stderr , "MemorySnapShot_Capture out\n" );
+}
+
+
+
+/*
+ * Same as MemorySnapShot_Capture, but snapshot is saved immediately
+ * without restarting emulation (used in the debugger)
+ */
+void MemorySnapShot_Capture_Immediate(const char *pszFileName, bool bConfirm)
+{
+	/* Make a temporary copy of the parameters for MemorySnapShot_Capture_Do() */
+	Str_Copy(Temp_FileName, pszFileName, FILENAME_MAX);
+	Temp_Confirm = bConfirm;
+
+	MemorySnapShot_Capture_Do ();
+}
+
+
+
+/*
+ * Do the real saving (called from newcpu.c / m68k_go()
+ */
+void MemorySnapShot_Capture_Do(void)
+{
+	uint32_t magic = SNAPSHOT_MAGIC;
 
 	/* Set to 'saving' */
-	if (MemorySnapShot_OpenFile(pszFileName, true, bConfirm))
+	if (MemorySnapShot_OpenFile(Temp_FileName, true, Temp_Confirm))
 	{
 		/* Capture each files details */
 		Configuration_MemorySnapShot_Capture(true);
@@ -328,9 +362,11 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 		Crossbar_MemorySnapShot_Capture(true);
 		VIDEL_MemorySnapShot_Capture(true);
 		DSP_MemorySnapShot_Capture(true);
-		DebugUI_MemorySnapShot_Capture(pszFileName, true);
+		DebugUI_MemorySnapShot_Capture(Temp_FileName, true);
 		IoMem_MemorySnapShot_Capture(true);
 		ScreenConv_MemorySnapShot_Capture(true);
+		SCC_MemorySnapShot_Capture(true);
+		SCU_MemorySnapShot_Capture(true);
 
 		/* end marker */
 		MemorySnapShot_Store(&magic, sizeof(magic));
@@ -344,11 +380,11 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
 
 	/* Did error */
 	if (bCaptureError)
-		Log_AlertDlg(LOG_ERROR, "Unable to save memory state to file: %s", pszFileName);
-	else if (bConfirm)
-		Log_AlertDlg(LOG_INFO, "Memory state file saved: %s", pszFileName);
+		Log_AlertDlg(LOG_ERROR, "Unable to save memory state to file: %s", Temp_FileName);
+	else if (Temp_Confirm)
+		Log_AlertDlg(LOG_INFO, "Memory state file saved: %s", Temp_FileName);
 	else
-		Log_Printf(LOG_INFO, "Memory state file saved: %s", pszFileName);
+		Log_Printf(LOG_INFO, "Memory state file saved: %s", Temp_FileName);
 }
 
 
@@ -358,16 +394,44 @@ void MemorySnapShot_Capture(const char *pszFileName, bool bConfirm)
  */
 void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 {
-	Uint32 magic;
+//fprintf ( stderr , "MemorySnapShot_Restore in\n" );
+	/* Make a temporary copy of the parameters for MemorySnapShot_Restore_Do() */
+	Str_Copy(Temp_FileName, pszFileName, FILENAME_MAX);
+	Temp_Confirm = bConfirm;
 
+	/* With WinUAE cpu core, restore is done from m68k_go() after the end of the current instruction */
+	UAE_Set_State_Restore ();
+	UAE_Set_Quit_Reset ( false );					/* Ask for "quit" to start restoring state */
+	set_special(SPCFLAG_MODE_CHANGE);				/* exit m68k_run_xxx() loop and check "quit" */
+//fprintf ( stderr , "MemorySnapShot_Restore out\n" );
+}
+
+
+
+/*
+ * Do the real restoring (called from newcpu.c / m68k_go()
+ */
+void MemorySnapShot_Restore_Do(void)
+{
+	uint32_t magic;
+
+//fprintf ( stderr , "MemorySnapShot_Restore_Do in\n" );
 	/* Set to 'restore' */
-	if (MemorySnapShot_OpenFile(pszFileName, false, bConfirm))
+	if (MemorySnapShot_OpenFile(Temp_FileName, false, Temp_Confirm))
 	{
 		Configuration_MemorySnapShot_Capture(false);
 		TOS_MemorySnapShot_Capture(false);
 
+		/* FIXME [NP] : Reset_Cold calls TOS_InitImage which calls */
+		/* memory_init. memory_init allocs STRam and TTRam, but TTRam */
+		/* requires currprefs.address_space_24 which is not restored yet */
+		/* (it's from M68000_MemorySnapShot_Capture). To resolve this */
+		/* circular dependency, we init currprefs.address_space_24 here */
+		/* This should be split in different functions / order to avoid this loop */
+		currprefs.address_space_24 = ConfigureParams.System.bAddressSpace24;
+
 		/* Reset emulator to get things running */
-		IoMem_UnInit();  IoMem_Init();
+		IoMem_UnInit(ConfigureParams.System.nMachineType);  IoMem_Init();
 		Reset_Cold();
 
 		/* Capture each files details */
@@ -392,9 +456,11 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 		Crossbar_MemorySnapShot_Capture(false);
 		VIDEL_MemorySnapShot_Capture(false);
 		DSP_MemorySnapShot_Capture(false);
-		DebugUI_MemorySnapShot_Capture(pszFileName, false);
+		DebugUI_MemorySnapShot_Capture(Temp_FileName, false);
 		IoMem_MemorySnapShot_Capture(false);
 		ScreenConv_MemorySnapShot_Capture(false);
+		SCC_MemorySnapShot_Capture(false);
+		SCU_MemorySnapShot_Capture(false);
 
 		/* version string check catches release-to-release
 		 * state changes, bCaptureError catches too short
@@ -407,10 +473,6 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 		/* And close */
 		MemorySnapShot_CloseFile();
 
-		/* Apply patches for gemdos HD if needed */
-		/* (we need to do it after cpu tables for all opcodes were rebuilt) */
-		Cart_Patch();
-
 		/* changes may affect also info shown in statusbar */
 		Statusbar_UpdateInfo();
 
@@ -419,15 +481,29 @@ void MemorySnapShot_Restore(const char *pszFileName, bool bConfirm)
 			Log_AlertDlg(LOG_ERROR, "Full memory state restore failed!\nPlease reboot emulation.");
 			return;
 		}
+
+
+		/*
+		 * Apply some specific changes after everything is restored
+		 */
+		if ( ConfigureParams.System.nMachineType == MACHINE_MEGA_STE )
+		{
+			/* Restore CPU Freq and cache */
+			MegaSTE_CPU_Cache_Update ( IoMem_ReadByte(0xff8e21) );
+		}
+
+
 	}
 
+//fprintf ( stderr , "MemorySnapShot_Restore_Do out\n" );
+
 	/* Did error? */
 	if (bCaptureError)
-		Log_AlertDlg(LOG_ERROR, "Unable to restore memory state from file: %s", pszFileName);
-	else if (bConfirm)
-		Log_AlertDlg(LOG_INFO, "Memory state file restored: %s", pszFileName);
+		Log_AlertDlg(LOG_ERROR, "Unable to restore memory state from file: %s", Temp_FileName);
+	else if (Temp_Confirm)
+		Log_AlertDlg(LOG_INFO, "Memory state file restored: %s", Temp_FileName);
 	else
-		Log_Printf(LOG_INFO, "Memory state file restored: %s", pszFileName);
+		Log_Printf(LOG_INFO, "Memory state file restored: %s", Temp_FileName);
 }
 
 
@@ -461,9 +537,16 @@ void save_u8(uae_u8 data)
 //printf ("s8 %x\n", data);
 }
 
+void save_s8(uae_s8 data)
+{
+	MemorySnapShot_Store(&data, 1);
+//printf ("s8s %x\n", data);
+}
+
 uae_u64 restore_u64(void)
 {
 	uae_u64 data;
+	bCaptureSave=false;			/* (re)force bCaptureSave=false to prevent gcc11 warning */
 	MemorySnapShot_Store(&data, 8);
 	return data;
 }
@@ -471,6 +554,7 @@ uae_u64 restore_u64(void)
 uae_u32 restore_u32(void)
 {
 	uae_u32 data;
+	bCaptureSave=false;
 	MemorySnapShot_Store(&data, 4);
 //printf ("r32 %x\n", data);
 	return data;
@@ -479,6 +563,7 @@ uae_u32 restore_u32(void)
 uae_u16 restore_u16(void)
 {
 	uae_u16 data;
+	bCaptureSave=false;
 	MemorySnapShot_Store(&data, 2);
 //printf ("r16 %x\n", data);
 	return data;
@@ -487,8 +572,17 @@ uae_u16 restore_u16(void)
 uae_u8 restore_u8(void)
 {
 	uae_u8 data;
+	bCaptureSave=false;
 	MemorySnapShot_Store(&data, 1);
 //printf ("r8 %x\n", data);
 	return data;
 }
 
+uae_s8 restore_s8(void)
+{
+	uae_s8 data;
+	bCaptureSave=false;
+	MemorySnapShot_Store(&data, 1);
+//printf ("r8s %x\n", data);
+	return data;
+}
