@@ -6,7 +6,7 @@
 
   Shortcut keys
 */
-const char ShortCut_fileid[] = "Hatari shortcut.c : " __DATE__ " " __TIME__;
+const char ShortCut_fileid[] = "Hatari shortcut.c";
 
 #include <SDL.h>
 
@@ -44,7 +44,7 @@ static void ShortCut_FullScreen(void)
 	Uint32 cur_ticks;
 
 	/* SDL2 sometimes reports multiple key up and down events when toggling
-	 * fullscreen mode, even though the key has not been released inbetween
+	 * fullscreen mode, even though the key has not been released in between
 	 * (likely because the SDL window focus is lost when switching mode).
 	 * To avoid that we're going back and forth between fullscreen mode and
 	 * windowed mode in this case, we have to ignore full screen shortcut
@@ -85,14 +85,7 @@ static void ShortCut_MouseGrab(void)
 	/* If we are in windowed mode, toggle the mouse cursor mode now: */
 	if (!bInFullScreen)
 	{
-		if (bGrabMouse)
-		{
-			SDL_WM_GrabInput(SDL_GRAB_ON);
-		}
-		else
-		{
-			SDL_WM_GrabInput(SDL_GRAB_OFF);
-		}
+		SDL_SetRelativeMouseMode(bGrabMouse);
 	}
 }
 
@@ -139,7 +132,7 @@ static void ShortCut_RecordAnimation(void)
 		Avi_StartRecording ( ConfigureParams.Video.AviRecordFile , ConfigureParams.Screen.bCrop ,
 			ConfigureParams.Video.AviRecordFps == 0 ?
 				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , nScreenRefreshRate ) :
-				(Uint32)ConfigureParams.Video.AviRecordFps << CLOCKS_TIMINGS_SHIFT_VBL ,
+				ClocksTimings_GetVBLPerSec ( ConfigureParams.System.nMachineType , ConfigureParams.Video.AviRecordFps ) ,
 			1 << CLOCKS_TIMINGS_SHIFT_VBL ,
 			ConfigureParams.Video.AviRecordVcodec );
 	}
@@ -192,9 +185,8 @@ static void ShortCut_FastForward(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
 /**
- * Shortcut to 'Boss' key, ie minmize Window and switch to another application
+ * Shortcut to 'Boss' key, ie minimize Window and switch to another application
  */
 static void ShortCut_BossKey(void)
 {
@@ -203,23 +195,18 @@ static void ShortCut_BossKey(void)
 
 	if (bGrabMouse)
 	{
-		SDL_WM_GrabInput(SDL_GRAB_OFF);
+		SDL_SetRelativeMouseMode(false);
 		bGrabMouse = false;
 	}
 	Main_PauseEmulation(true);
 
 	/* Minimize Window and give up processing to next one! */
-#if WITH_SDL2
 	SDL_MinimizeWindow(sdlWindow);
-#else
-	SDL_WM_IconifyWindow();
-#endif
 }
 
 
-/*-----------------------------------------------------------------------*/
 /**
- * Shorcut to debug interface
+ * Shortcut to debug interface
  */
 static void ShortCut_Debug(void)
 {
@@ -233,9 +220,8 @@ static void ShortCut_Debug(void)
 }
 
 
-/*-----------------------------------------------------------------------*/
 /**
- * Shorcut to pausing
+ * Shortcut to pausing
  */
 static void ShortCut_Pause(void)
 {
@@ -243,8 +229,9 @@ static void ShortCut_Pause(void)
 		Main_PauseEmulation(true);
 }
 
+
 /**
- * Shorcut to load a disk image
+ * Shortcut to load a disk image
  */
 static void ShortCut_InsertDisk(int drive)
 {
@@ -252,6 +239,8 @@ static void ShortCut_InsertDisk(int drive)
 	const char *tmpname;
 	char FileNameB[ FILENAME_MAX ];
 
+	bool bOldMouseMode = SDL_GetRelativeMouseMode();
+
 	if (SDLGui_SetScreen(sdlscrn))
 		return;
 
@@ -264,7 +253,11 @@ static void ShortCut_InsertDisk(int drive)
 		tmpname = ConfigureParams.DiskImage.szDiskImageDirectory;
 
 	Main_PauseEmulation(true);
+
+	SDL_SetRelativeMouseMode(SDL_FALSE);
 	selname = SDLGui_FileSelect("Floppy image:", tmpname, &zip_path, false);
+	SDL_SetRelativeMouseMode(bOldMouseMode);
+
 	if (selname)
 	{
 		if (File_Exists(selname))
@@ -407,7 +400,7 @@ bool Shortcut_Invoke(const char *shortcut)
 
 	if (ShortCutKey != SHORTCUT_NONE)
 	{
-		fprintf(stderr, "Shortcut invocation failed, shortcut already active\n");
+		fprintf(stderr, "WARNING: Shortcut invocation failed, shortcut already active\n");
 		return false;
 	}
 	for (i = 0; shortcuts[i].name; i++)
@@ -450,20 +443,23 @@ static SHORTCUTKEYIDX ShortCut_CheckKey(int symkey, int *keys)
 /**
  * Check which Shortcut key is pressed/released.
  * If press is set, store the key array index.
- * Return zero if key didn't match to a shortcut
+ * Return true if key combo matched to a shortcut
  */
-int ShortCut_CheckKeys(int modkey, int symkey, bool press)
+bool ShortCut_CheckKeys(int modkey, int symkey, bool press)
 {
 	SHORTCUTKEYIDX key;
 
-	if (modkey & (KMOD_RALT|KMOD_LMETA|KMOD_RMETA|KMOD_MODE))
+	if (symkey == SDLK_UNKNOWN)
+		return false;
+
+	if (modkey & (KMOD_RALT|KMOD_LGUI|KMOD_RGUI|KMOD_MODE))
 		key = ShortCut_CheckKey(symkey, ConfigureParams.Shortcut.withModifier);
 	else
 		key = ShortCut_CheckKey(symkey, ConfigureParams.Shortcut.withoutModifier);
 
 	if (key == SHORTCUT_NONE)
-		return 0;
+		return false;
 	if (press)
 		ShortCutKey = key;
-	return 1;
+	return true;
 }
