@@ -32,7 +32,7 @@
     correct in Falcon & TT 8-bit palette modes?
   - call Statusbar_AddMessage() from log.c?
 */
-const char Statusbar_fileid[] = "Hatari statusbar.c : " __DATE__ " " __TIME__;
+const char Statusbar_fileid[] = "Hatari statusbar.c";
 
 #include <assert.h>
 #include "main.h"
@@ -50,6 +50,8 @@ const char Statusbar_fileid[] = "Hatari statusbar.c : " __DATE__ " " __TIME__;
 #include "fdc.h"
 #include "stMemory.h"
 #include "blitter.h"
+#include "str.h"
+#include "lilo.h"
 
 #define DEBUG 0
 #if DEBUG
@@ -90,7 +92,7 @@ static enum {
 	OVERLAY_NONE,
 	OVERLAY_DRAWN,
 	OVERLAY_RESTORED
-} bOverlayState;
+} nOverlayState;
 
 static SDL_Rect RecLedRect;
 static bool bOldRecording;
@@ -105,7 +107,7 @@ static Uint32 RecColorOn, RecColorOff;
 static Uint32 GrayBg, LedColorBg;
 
 /* needs to be enough for all messages, but <= MessageRect width / font width */
-#define MAX_MESSAGE_LEN 60
+#define MAX_MESSAGE_LEN 63
 typedef struct msg_item {
 	struct msg_item *next;
 	char msg[MAX_MESSAGE_LEN+1];
@@ -125,6 +127,8 @@ static int bOldFastForward;
 
 static SDL_Rect FDCTextRect;
 
+static SDL_Rect JoysticksTextRect;
+
 /* screen height above statusbar and height of statusbar below screen */
 static int ScreenHeight;
 static int StatusbarHeight;
@@ -140,10 +144,12 @@ int Statusbar_GetHeightForSize(int width, int height)
 	/* Must arrive at same conclusion about font size as SDLGui_SetScreen(),
 	 * and max size returned by this must correspond to STATUSBAR_MAX_HEIGHT
 	 */
-	if (ConfigureParams.Screen.bShowStatusbar) {
+	if (ConfigureParams.Screen.bShowStatusbar)
+	{
 		/* smaller SDL GUI font height = 8, larger = 16 */
 		h = 8;
-		if (width >= 640 && height >= (400-2*h)) {
+		if (width >= 640 && height >= (400-2*h))
+		{
 			h *= 2;
 		}
 		h += 1+1;
@@ -223,6 +229,35 @@ static int Statusbar_BlitterGetLinesOn ( int max_h )
 }
 
 
+/*-----------------------------------------------------------------------*/
+/**
+ * Return a text corresponding to the status of each emulated joystick
+ * - : off   J : real joystick   K : keyboard emulation
+ */
+static void Statusbar_JoysticksGetText ( char *buf )
+{
+	int i;
+
+	for (i = 0; i < JOYSTICK_COUNT; i++)
+	{
+		switch (ConfigureParams.Joysticks.Joy[i].nJoystickMode)
+		{
+		case JOYSTICK_DISABLED:
+			*buf++ = '-';
+			break;
+		case JOYSTICK_REALSTICK:
+			*buf++ = 'J';
+			break;
+		case JOYSTICK_KEYBOARD:
+			*buf++ = 'K';
+			break;
+		}
+	}
+	*buf = '\0';
+}
+
+
+
 /*-----------------------------------------------------------------------*/
 /**
  * Set overlay led size/pos on given screen to internal Rect
@@ -238,14 +273,15 @@ static void Statusbar_OverlayInit(const SDL_Surface *surf)
 	OverlayLedRect.x = surf->w - 5*h/2;
 	OverlayLedRect.y = h/2;
 	/* free previous restore surface if it's incompatible */
-	if (OverlayUnderside &&
-	    OverlayUnderside->w == OverlayLedRect.w &&
-	    OverlayUnderside->h == OverlayLedRect.h &&
-	    OverlayUnderside->format->BitsPerPixel == surf->format->BitsPerPixel) {
+	if (OverlayUnderside && (
+	    OverlayUnderside->w != OverlayLedRect.w ||
+	    OverlayUnderside->h != OverlayLedRect.h ||
+	    OverlayUnderside->format->BitsPerPixel != surf->format->BitsPerPixel))
+	{
 		SDL_FreeSurface(OverlayUnderside);
 		OverlayUnderside = NULL;
 	}
-	bOverlayState = OVERLAY_NONE;
+	nOverlayState = OVERLAY_NONE;
 }
 
 /*-----------------------------------------------------------------------*/
@@ -262,6 +298,7 @@ void Statusbar_Init(SDL_Surface *surf)
 	const char *text[MAX_DRIVE_LEDS] = { "A:", "B:", "HD:" };
 	char FdcText[FDC_MSG_MAX_LEN];
 	int FdcTextLen;
+	char JoysticksText[JOYSTICK_COUNT+1];
 
 	DEBUGPRINT(("Statusbar_Init()\n"));
 	assert(surf);
@@ -278,17 +315,20 @@ void Statusbar_Init(SDL_Surface *surf)
 	GrayBg      = SDL_MapRGB(surf->format, 0xc0, 0xc0, 0xc0);
 
 	/* disable leds */
-	for (i = 0; i < MAX_DRIVE_LEDS; i++) {
+	for (i = 0; i < MAX_DRIVE_LEDS; i++)
+	{
 		Led[i].state = Led[i].oldstate = LED_STATE_OFF;
 		Led[i].expire = 0;
 	}
 	Statusbar_OverlayInit(surf);
 	
 	/* disable statusbar if it doesn't fit to video mode */
-	if (surf->h < ScreenHeight + StatusbarHeight) {
+	if (surf->h < ScreenHeight + StatusbarHeight)
+	{
 		StatusbarHeight = 0;
 	}
-	if (!StatusbarHeight) {
+	if (!StatusbarHeight)
+	{
 		DEBUGPRINT(("Doesn't fit <- Statusbar_Init()\n"));
 		return;
 	}
@@ -300,11 +340,14 @@ void Statusbar_Init(SDL_Surface *surf)
 
 	/* video mode didn't match, need to recalculate sizes */
 	lineh = 1+fonth+1;
-	if (surf->h > ScreenHeight + StatusbarHeight) {
+	if (surf->h > ScreenHeight + StatusbarHeight)
+	{
 		StatusbarHeight = STATUSBAR_LINES*lineh;
 		/* actually statusbar vertical offset */
 		ScreenHeight = surf->h - StatusbarHeight;
-	} else {
+	}
+	else
+	{
 		assert(STATUSBAR_LINES*lineh <= StatusbarHeight);
 	}
 
@@ -315,12 +358,13 @@ void Statusbar_Init(SDL_Surface *surf)
 	FullRect.h = StatusbarHeight;
 	SDL_FillRect(surf, &FullRect, GrayBg);
 
-	/* intialize messages (first row) */
+	/* initialize messages (first row) */
 	MessageRect.x = fontw;
 	MessageRect.y = ScreenHeight + lineh/2 - fonth/2;
 	MessageRect.w = surf->w - fontw;
 	MessageRect.h = fonth;
-	for (item = MessageList; item; item = item->next) {
+	for (item = MessageList; item; item = item->next)
+	{
 		item->shown = false;
 	}
 
@@ -339,7 +383,8 @@ void Statusbar_Init(SDL_Surface *surf)
 	yoffset = ScreenHeight + lineh + lineh/2 - fonth/2;
 
 	/* draw led texts and boxes + calculate box offsets */
-	for (i = 0; i < MAX_DRIVE_LEDS; i++) {
+	for (i = 0; i < MAX_DRIVE_LEDS; i++)
+	{
 		SDLGui_Text(xoffset, yoffset, text[i]);
 		xoffset += strlen(text[i]) * fontw;
 		xoffset += fontw/2;
@@ -361,7 +406,17 @@ void Statusbar_Init(SDL_Surface *surf)
 	SDLGui_Text(FDCTextRect.x, FDCTextRect.y, FdcText);
 	FDCTextRect.w = FdcTextLen * fontw + fontw/2;
 	FDCTextRect.h = fonth;
-	// xoffset += FDCTextRect.w;
+	xoffset += FDCTextRect.w;
+
+	/* print joysticks' info */
+	xoffset += 2*fontw;
+	JoysticksTextRect.x = xoffset;
+	JoysticksTextRect.y = yoffset;
+	Statusbar_JoysticksGetText(JoysticksText);
+	SDLGui_Text(JoysticksTextRect.x, JoysticksTextRect.y, JoysticksText);
+	JoysticksTextRect.w = JOYSTICK_COUNT * fontw + fontw/2;
+	JoysticksTextRect.h = fonth;
+	// xoffset += JoysticksTextRect.w;
 
 	/* draw frameskip on the right */
 	FrameSkipsRect.x = surf->w - 21*fontw;
@@ -371,9 +426,12 @@ void Statusbar_Init(SDL_Surface *surf)
 	FrameSkipsRect.w = 4 * fontw;
 	FrameSkipsRect.h = fonth;
 
-	if(ConfigureParams.System.bFastForward) {
+	if(ConfigureParams.System.bFastForward)
+	{
 		SDLGui_Text(FrameSkipsRect.x, FrameSkipsRect.y, "0 >>");
-	} else {
+	}
+	else
+	{
 		SDLGui_Text(FrameSkipsRect.x, FrameSkipsRect.y, "0");
 	}
 
@@ -398,20 +456,21 @@ void Statusbar_Init(SDL_Surface *surf)
 	bOldRecording = false;
 
 	/* and blit statusbar on screen */
-	SDL_UpdateRects(surf, 1, &FullRect);
+	Screen_UpdateRects(surf, 1, &FullRect);
 	DEBUGPRINT(("Drawn <- Statusbar_Init()\n"));
 }
 
 
 /*-----------------------------------------------------------------------*/
 /**
- * Qeueue new statusbar message 'msg' to be shown for 'msecs' milliseconds
+ * Queue new statusbar message 'msg' to be shown for 'msecs' milliseconds
  */
 void Statusbar_AddMessage(const char *msg, Uint32 msecs)
 {
 	msg_item_t *item;
 
-	if (!ConfigureParams.Screen.bShowStatusbar) {
+	if (!ConfigureParams.Screen.bShowStatusbar)
+	{
 		/* no sense in queuing messages that aren't shown */
 		return;
 	}
@@ -421,13 +480,15 @@ void Statusbar_AddMessage(const char *msg, Uint32 msecs)
 	item->next = MessageList;
 	MessageList = item;
 
-	strncpy(item->msg, msg, MAX_MESSAGE_LEN);
-	item->msg[MAX_MESSAGE_LEN] = '\0';
+	Str_Copy(item->msg, msg, sizeof(item->msg));
 	DEBUGPRINT(("Add message: '%s'\n", item->msg));
 
-	if (msecs) {
+	if (msecs)
+	{
 		item->timeout = msecs;
-	} else {
+	}
+	else
+	{
 		/* show items by default for 2.5 secs */
 		item->timeout = 2500;
 	}
@@ -440,9 +501,10 @@ void Statusbar_AddMessage(const char *msg, Uint32 msecs)
  */
 static char *Statusbar_AddString(char *buffer, const char *more)
 {
-	while(*more) {
+	if (!more)
+		return buffer;
+	while (*more)
 		*buffer++ = *more++;
-	}
 	return buffer;
 }
 
@@ -452,19 +514,23 @@ static char *Statusbar_AddString(char *buffer, const char *more)
  */
 void Statusbar_UpdateInfo(void)
 {
-	int i;
-	char *end = DefaultMessage.msg;
 	int size;
+	char buffer[200];				/* large enough for any message */
+	char *end;
+
+	end = buffer;
 
 	/* CPU MHz */
-	if (ConfigureParams.System.nCpuFreq > 9) {
+	if (ConfigureParams.System.nCpuFreq > 9)
+	{
 		*end++ = '0' + ConfigureParams.System.nCpuFreq / 10;
 	}
 	*end++ = '0' + ConfigureParams.System.nCpuFreq % 10;
 	end = Statusbar_AddString(end, "MHz");
 
 	/* CPU type */
-	if(ConfigureParams.System.nCpuLevel > 0) {
+	if(ConfigureParams.System.nCpuLevel > 0)
+	{
 		*end++ = '/';
 		*end++ = '0';
 		if ( ConfigureParams.System.nCpuLevel == 5 )	/* Special case : 68060 has nCpuLevel=5 */
@@ -474,10 +540,21 @@ void Statusbar_UpdateInfo(void)
 		*end++ = '0';
 	}
 
+	const char *mode = NULL;
+	bool cache = ConfigureParams.System.bCpuDataCache && ConfigureParams.System.nCpuLevel > 2;
+	/* Prefetch / cycle exact mode and data cache? */
+	if ( ConfigureParams.System.bCycleExactCpu )
+		mode = cache ? "(CED)" : "(CE)";
+	else if ( ConfigureParams.System.bCompatibleCpu )
+		mode = cache ? "(PFD)" : "(PF)";
+	else if (cache)
+		mode = "(D)";
+	end = Statusbar_AddString(end, mode);
+
 	/* additional WinUAE CPU/FPU info */
-#if ENABLE_WINUAE_CPU
 	*end++ = '/';
-	switch (ConfigureParams.System.n_FPUType) {
+	switch (ConfigureParams.System.n_FPUType)
+	{
 	case FPU_68881:
 		end = Statusbar_AddString(end, "68881");
 		break;
@@ -490,10 +567,14 @@ void Statusbar_UpdateInfo(void)
 	default:
 		*end++ = '-';
 	}
-	if (ConfigureParams.System.bMMU) {
+	if (ConfigureParams.System.bSoftFloatFPU && ConfigureParams.System.n_FPUType != FPU_NONE)
+	{
+		end = Statusbar_AddString(end, "(SF)");
+	}
+	if (ConfigureParams.System.bMMU)
+	{
 		end = Statusbar_AddString(end, "/MMU");
 	}
-#endif
 
 	/* amount of memory in MB */
 	*end++ = ' ';
@@ -504,13 +585,15 @@ void Statusbar_UpdateInfo(void)
 	else if ( size % 1024 == 512 )
 		end += sprintf(end, ".5");
 
-	if (TTmemory && ConfigureParams.Memory.TTRamSize_KB) {
+	if (TTmemory && ConfigureParams.Memory.TTRamSize_KB)
+	{
 		end += sprintf(end, "/%i", ConfigureParams.Memory.TTRamSize_KB/1024);
 	}
 	end = Statusbar_AddString(end, "MB ");
 
 	/* machine type */
-	switch (ConfigureParams.System.nMachineType) {
+	switch (ConfigureParams.System.nMachineType)
+	{
 	case MACHINE_ST:
 		end = Statusbar_AddString(end, "ST(");
 		end = Statusbar_AddString(end, Video_GetTimings_Name());
@@ -537,10 +620,30 @@ void Statusbar_UpdateInfo(void)
 
 	/* TOS type/version */
 	end = Statusbar_AddString(end, ", ");
-	if (bIsEmuTOS) {
-		end = Statusbar_AddString(end, "EmuTOS");
-	} else {
-		end = Statusbar_AddString(end, "TOS v");
+	if (bIsEmuTOS)
+	{
+		if (EmuTosVersion > 0)
+		{
+			char str[20];
+			snprintf(str, sizeof(str), "EmuTOS %d.%d.%d",
+			         EmuTosVersion >> 24, (EmuTosVersion >> 16) & 0xff,
+			         (EmuTosVersion >> 8) & 0xff);
+			end = Statusbar_AddString(end, str);
+			if (EmuTosVersion & 0xff)
+				end = Statusbar_AddString(end, "+");
+		}
+		else
+		{
+			end = Statusbar_AddString(end, "EmuTOS");
+		}
+	}
+	else if (bUseLilo)
+	{
+		end = Statusbar_AddString(end, "Linux");
+	}
+	else
+	{
+		end = Statusbar_AddString(end, "TOS ");
 		*end++ = '0' + ((TosVersion & 0xf00) >> 8);
 		*end++ = '.';
 		*end++ = '0' + ((TosVersion & 0xf0) >> 4);
@@ -549,10 +652,14 @@ void Statusbar_UpdateInfo(void)
 
 	/* monitor type */
 	end = Statusbar_AddString(end, ", ");
-	if (bUseVDIRes) {
+	if (bUseVDIRes)
+	{
 		end = Statusbar_AddString(end, "VDI");
-	} else {
-		switch (ConfigureParams.Screen.nMonitorType) {
+	}
+	else
+	{
+		switch (ConfigureParams.Screen.nMonitorType)
+		{
 		case MONITOR_TYPE_MONO:
 			end = Statusbar_AddString(end, "MONO");
 			break;
@@ -560,7 +667,11 @@ void Statusbar_UpdateInfo(void)
 			end = Statusbar_AddString(end, "RGB");
 			break;
 		case MONITOR_TYPE_VGA:
-			end = Statusbar_AddString(end, "VGA");
+			/* There were no VGA monitors for the ST/STE */
+			if (Config_IsMachineST() || Config_IsMachineSTE())
+				end = Statusbar_AddString(end, "RGB");
+			else
+				end = Statusbar_AddString(end, "VGA");
 			break;
 		case MONITOR_TYPE_TV:
 			end = Statusbar_AddString(end, "TV");
@@ -571,24 +682,9 @@ void Statusbar_UpdateInfo(void)
 		end += sprintf(end, " %d Hz" , nScreenRefreshRate);
 	}
 
-	/* joystick type */
-	end = Statusbar_AddString(end, ", ");
-	for (i = 0; i < JOYSTICK_COUNT; i++) {
-		switch (ConfigureParams.Joysticks.Joy[i].nJoystickMode) {
-		case JOYSTICK_DISABLED:
-			*end++ = '-';
-			break;
-		case JOYSTICK_REALSTICK:
-			*end++ = 'J';
-			break;
-		case JOYSTICK_KEYBOARD:
-			*end++ = 'K';
-			break;
-		}
-	}
 	*end = '\0';
 
-	assert(end - DefaultMessage.msg < MAX_MESSAGE_LEN);
+	Str_Copy(DefaultMessage.msg, buffer, MAX_MESSAGE_LEN);
 	DEBUGPRINT(("Set default message: '%s'\n", DefaultMessage.msg));
 	/* make sure default message gets (re-)drawn when next checked */
 	DefaultMessage.shown = false;
@@ -602,7 +698,8 @@ static SDL_Rect* Statusbar_DrawMessage(SDL_Surface *surf, const char *msg)
 {
 	int fontw, fonth, offset;
 	SDL_FillRect(surf, &MessageRect, GrayBg);
-	if (*msg) {
+	if (*msg)
+	{
 		SDLGui_GetFontSize(&fontw, &fonth);
 		offset = (MessageRect.w - strlen(msg) * fontw) / 2;
 		SDLGui_Text(MessageRect.x + offset, MessageRect.y, msg);
@@ -622,12 +719,15 @@ static SDL_Rect* Statusbar_ShowMessage(SDL_Surface *surf, Uint32 ticks)
 {
 	msg_item_t *next;
 
-	if (MessageList->shown) {
-		if (!MessageList->expire) {
+	if (MessageList->shown)
+	{
+		if (!MessageList->expire)
+		{
 			/* last/default message newer expires */
 			return NULL;
 		}
-		if (MessageList->expire > ticks) {
+		if (MessageList->expire > ticks)
+		{
 			/* not timed out yet */
 			return NULL;
 		}
@@ -639,7 +739,8 @@ static SDL_Rect* Statusbar_ShowMessage(SDL_Surface *surf, Uint32 ticks)
 	}
 	/* not shown yet, show */
 	MessageList->shown = true;
-	if (MessageList->timeout && !MessageList->expire) {
+	if (MessageList->timeout && !MessageList->expire)
+	{
 		MessageList->expire = ticks + MessageList->timeout;
 	}
 	return Statusbar_DrawMessage(surf, MessageList->msg);
@@ -653,12 +754,14 @@ static SDL_Rect* Statusbar_ShowMessage(SDL_Surface *surf, Uint32 ticks)
 void Statusbar_OverlayBackup(SDL_Surface *surf)
 {
 	if ((StatusbarHeight && ConfigureParams.Screen.bShowStatusbar)
-	    || !ConfigureParams.Screen.bShowDriveLed) {
+	    || !ConfigureParams.Screen.bShowDriveLed)
+	{
 		/* overlay not used with statusbar */
 		return;
 	}
 	assert(surf);
-	if (!OverlayUnderside) {
+	if (!OverlayUnderside)
+	{
 		SDL_Surface *bak;
 		SDL_PixelFormat *fmt = surf->format;
 		bak = SDL_CreateRGBSurface(surf->flags,
@@ -682,15 +785,17 @@ void Statusbar_OverlayBackup(SDL_Surface *surf)
 void Statusbar_OverlayRestore(SDL_Surface *surf)
 {
 	if ((StatusbarHeight && ConfigureParams.Screen.bShowStatusbar)
-	    || !ConfigureParams.Screen.bShowDriveLed) {
+	    || !ConfigureParams.Screen.bShowDriveLed)
+	{
 		/* overlay not used with statusbar */
 		return;
 	}
-	if (bOverlayState == OVERLAY_DRAWN && OverlayUnderside) {
+	if (nOverlayState == OVERLAY_DRAWN && OverlayUnderside)
+	{
 		assert(surf);
 		SDL_BlitSurface(OverlayUnderside, NULL, surf, &OverlayLedRect);
 		/* this will make the draw function to update this the screen */
-		bOverlayState = OVERLAY_RESTORED;
+		nOverlayState = OVERLAY_RESTORED;
 	}
 }
 
@@ -701,11 +806,12 @@ void Statusbar_OverlayRestore(SDL_Surface *surf)
 static void Statusbar_OverlayDrawLed(SDL_Surface *surf, Uint32 color)
 {
 	SDL_Rect rect;
-	if (bOverlayState == OVERLAY_DRAWN) {
+	if (nOverlayState == OVERLAY_DRAWN)
+	{
 		/* some led already drawn */
 		return;
 	}
-	bOverlayState = OVERLAY_DRAWN;
+	nOverlayState = OVERLAY_DRAWN;
 
 	/* enabled led with border */
 	rect = OverlayLedRect;
@@ -728,12 +834,16 @@ static SDL_Rect* Statusbar_OverlayDraw(SDL_Surface *surf)
 	Uint32 currentticks = SDL_GetTicks();
 	int i;
 
-	if (bRecordingYM || bRecordingWav || bRecordingAvi) {
+	if (bRecordingYM || bRecordingWav || bRecordingAvi)
+	{
 		Statusbar_OverlayDrawLed(surf, RecColorOn);
 	}
-	for (i = 0; i < MAX_DRIVE_LEDS; i++) {
-		if (Led[i].state) {
-			if (Led[i].expire && Led[i].expire < currentticks) {
+	for (i = 0; i < MAX_DRIVE_LEDS; i++)
+	{
+		if (Led[i].state)
+		{
+			if (Led[i].expire && Led[i].expire < currentticks)
+			{
 				Led[i].state = LED_STATE_OFF;
 				continue;
 			}
@@ -745,11 +855,13 @@ static SDL_Rect* Statusbar_OverlayDraw(SDL_Surface *surf)
 	 *   NONE -> DRAWN -> RESTORED -> DRAWN -> RESTORED -> NONE
 	 * Other than NONE state needs to be updated on screen
 	 */
-	switch (bOverlayState) {
+	switch (nOverlayState)
+	{
 	case OVERLAY_RESTORED:
-		bOverlayState = OVERLAY_NONE;
+		nOverlayState = OVERLAY_NONE;
+		/* fall through */
 	case OVERLAY_DRAWN:
-		DEBUGPRINT(("Overlay LED = %s\n", bOverlayState==OVERLAY_DRAWN?"ON":"OFF"));
+		DEBUGPRINT(("Overlay LED = %s\n", nOverlayState==OVERLAY_DRAWN?"ON":"OFF"));
 		return &OverlayLedRect;
 	case OVERLAY_NONE:
 		break;
@@ -770,6 +882,8 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 {
 	static char FdcOld[FDC_MSG_MAX_LEN] = "";
 	char FdcNew[FDC_MSG_MAX_LEN];
+	static char JoysticksOld[JOYSTICK_COUNT+1] = "";
+	char JoysticksNew[JOYSTICK_COUNT+1];
 	Uint32 color, currentticks;
 	static SDL_Rect rect;
 	SDL_Rect *last_rect;
@@ -781,13 +895,16 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 		return NULL;
 
 	assert(surf);
-	if (!(StatusbarHeight && ConfigureParams.Screen.bShowStatusbar)) {
+	if (!(StatusbarHeight && ConfigureParams.Screen.bShowStatusbar))
+	{
 		last_rect = NULL;
 		/* not enabled (anymore), show overlay led instead? */
-		if (ConfigureParams.Screen.bShowDriveLed) {
+		if (ConfigureParams.Screen.bShowDriveLed)
+		{
 			last_rect = Statusbar_OverlayDraw(surf);
-			if (do_update && last_rect) {
-				SDL_UpdateRects(surf, 1, last_rect);
+			if (do_update && last_rect)
+			{
+				Screen_UpdateRects(surf, 1, last_rect);
 				last_rect = NULL;
 			}
 		}
@@ -796,8 +913,9 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 
 	/* Statusbar_Init() not called before this? */
 #if DEBUG
-	if (surf->h != ScreenHeight + StatusbarHeight) {
-		printf("%d != %d + %d\n", surf->h, ScreenHeight, StatusbarHeight);
+	if (surf->h != ScreenHeight + StatusbarHeight)
+	{
+		printf("DEBUG: %d != %d + %d\n", surf->h, ScreenHeight, StatusbarHeight);
 	}
 #endif
 	assert(surf->h == ScreenHeight + StatusbarHeight);
@@ -807,11 +925,14 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 	updates = last_rect ? 1 : 0;
 
 	rect = LedRect;
-	for (i = 0; i < MAX_DRIVE_LEDS; i++) {
-		if (Led[i].expire && Led[i].expire < currentticks) {
+	for (i = 0; i < MAX_DRIVE_LEDS; i++)
+	{
+		if (Led[i].expire && Led[i].expire < currentticks)
+		{
 			Led[i].state = LED_STATE_OFF;
 		}
-		if (Led[i].state == Led[i].oldstate) {
+		if (Led[i].state == Led[i].oldstate)
+		{
 			continue;
 		}
 		Led[i].oldstate = Led[i].state;
@@ -824,7 +945,8 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 	}
 
 	FDC_Get_Statusbar_Text(FdcNew, sizeof(FdcNew));
-	if (strcmp(FdcNew, FdcOld)) {
+	if (strcmp(FdcNew, FdcOld))
+	{
 		strcpy(FdcOld, FdcNew);
 		SDL_FillRect(surf, &FDCTextRect, GrayBg);
 		SDLGui_Text(FDCTextRect.x, FDCTextRect.y, FdcNew);
@@ -832,8 +954,20 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 		updates++;
 	}
 
+	/* joysticks' type */
+	Statusbar_JoysticksGetText(JoysticksNew);
+	if (strcmp(JoysticksNew, JoysticksOld))
+	{
+		strcpy(JoysticksOld, JoysticksNew);
+		SDL_FillRect(surf, &JoysticksTextRect, GrayBg);
+		SDLGui_Text(JoysticksTextRect.x, JoysticksTextRect.y, JoysticksNew);
+		last_rect = &JoysticksTextRect;
+		updates++;
+	}
+
 	if (nOldFrameSkips != nFrameSkips ||
-	    bOldFastForward != ConfigureParams.System.bFastForward) {
+	    bOldFastForward != ConfigureParams.System.bFastForward)
+	{
 		char fscount[5];
 		int end = 2;
 
@@ -845,7 +979,8 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 		else
 			fscount[0] = 'X';
 		fscount[1] = ' ';
-		if(ConfigureParams.System.bFastForward) {
+		if(ConfigureParams.System.bFastForward)
+		{
 			fscount[2] = '>';
 			fscount[3] = '>';
 			end = 4;
@@ -862,7 +997,8 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 	/* Blitter : draw 'BltLed_lines_on_new' lines with color BltColorOn */
 	/* and the rest with color BltColorOff. This gives some kind of vu-meter */
 	BltLed_lines_on_new = Statusbar_BlitterGetLinesOn( BltLedRect.h );
-	if ( BltLed_lines_on_new != BltLed_lines_on ) {
+	if (BltLed_lines_on_new != BltLed_lines_on)
+	{
 		BltLed_lines_on = BltLed_lines_on_new;
 		if ( BltLed_lines_on > 0 )
 		{
@@ -883,12 +1019,15 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 		}
 	}
 
-	if ((bRecordingYM || bRecordingWav || bRecordingAvi)
-	    != bOldRecording) {
+	if ((bRecordingYM || bRecordingWav || bRecordingAvi) != bOldRecording)
+	{
 		bOldRecording = !bOldRecording;
-		if (bOldRecording) {
+		if (bOldRecording)
+		{
 			color = RecColorOn;
-		} else {
+		}
+		else
+		{
 			color = RecColorOff;
 		}
 		SDL_FillRect(surf, &RecLedRect, color);
@@ -897,12 +1036,14 @@ SDL_Rect* Statusbar_Update(SDL_Surface *surf, bool do_update)
 		updates++;
 	}
 
-	if (updates > 1) {
+	if (updates > 1)
+	{
 		/* multiple items updated -> update whole statusbar */
 		last_rect = &FullRect;
 	}
-	if (do_update && last_rect) {
-		SDL_UpdateRects(surf, 1, last_rect);
+	if (do_update && last_rect)
+	{
+		Screen_UpdateRects(surf, 1, last_rect);
 		last_rect = NULL;
 	}
 	return last_rect;
