@@ -16,16 +16,16 @@
 #define CMD_REMOVE_ALL "all"
 
 
-static bool SetCpuRegister(const char *regname, Uint32 value)
+static bool SetCpuRegister(const char *regname, uint32_t value)
 {
-	Uint32 *addr;
+	uint32_t *addr;
 	
 	switch (DebugCpu_GetRegisterAddress(regname, &addr)) {
 	case 32:
 		*addr = value;
 		break;
 	case 16:
-		*(Uint16*)addr = value;
+		*(uint16_t*)addr = value;
 		break;
 	default:
 		fprintf(stderr, "SETUP ERROR: Register '%s' to set (to %x) is unrecognized!\n", regname, value);
@@ -35,16 +35,16 @@ static bool SetCpuRegister(const char *regname, Uint32 value)
 }
 
 #if 0
-static bool SetDspRegister(const char *regname, Uint32 value)
+static bool SetDspRegister(const char *regname, uint32_t value)
 {
-	Uint32 *addr, mask;
+	uint32_t *addr, mask;
 
 	switch (DSP_GetRegisterAddress(regname, &addr, &mask)) {
 	case 32:
 		*addr = value & mask;
 		break;
 	case 16:
-		*(Uint16*)addr = value & mask;
+		*(uint16_t*)addr = value & mask;
 		break;
 	default:
 		return false;
@@ -79,14 +79,15 @@ int main(int argc, const char *argv[])
 		"pc = 2 &&",
 		"255 & 3 = (d0) & && 2 = 2",
 		/* missing options file */
-		"pc>pc :file no-such-file",		
+		"pc>pc :file no-such-file",
+		"VdiOpcode = $8 :info no-info",
 		/* size and mask mismatches with numbers */
 		"d0.w = $ffff0",
 		"(a0).b & 3 < 100",
 		NULL
 	};
 	const char *parser_pass[] = {
-		/* comparisons with normal numbers + indrect addressing */
+		/* comparisons with normal numbers + indirect addressing */
 		" ($200).w > 200 ",
 		" ($200).w < 200 ",
 		" (200).w = $200 ",
@@ -104,10 +105,11 @@ int main(int argc, const char *argv[])
 		/* options */
 		"($200).w ! ($200).w :trace",
 		"($200).w > ($200).w :4 :lock",
+		"VdiOpcode = $8 :quiet :info vdi",
 		"pc>pc :file data/test.ini :once",
 		NULL
 	};
-	/* address breakpoint + expression evalution with register */
+	/* address breakpoint + expression evaluation with register */
 	char addr_pass[] = "pc + ($200*16/2 & 0xffff)";
 
 	const char *nonmatching_tests[] = {
@@ -172,7 +174,7 @@ int main(int argc, const char *argv[])
 	/* set up registers etc */
 
 	/* fail indirect equality checks with zeroed regs */
-	memset(STRam, 0, sizeof(STRam));
+	memset(STRam, 0, STRamEnd);
 	STMemory_WriteByte(0, 1);
 	/* !match: "( $200 ) . b > 200"
 	 *  match: "( $200 ) . b > ( 200 ) . b"
